/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_adc.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_adc.h"
#include "sh3xf2xx_sb0_rcc.h"
/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup ADC_MODULE ADC
  * @brief ADC driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup ADC_Exported_Functions Public Functions
  * @{
  */ 

/** @defgroup ADC_Group1 Initialization and Configuration
 *  @brief    Initialization and Configuration
 *
@verbatim    
 ===============================================================================
                      Initialization and Configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */

/**
  * @brief  Reset the ADC peripherals registers to their default reset values.
  * @retval None
  */
void ADC_Reset(void)
{
    RCC_APB0PeriphReset(RCC_APB0_ADC);
}

/**
  * @brief  Fills each ADC_InitStruct member with its default value. 
  * @param  ADC_InitStruct: pointer to a @ref ADC_InitTypeDef structure which will 
  *         be initialized.
  * @retval None
  */
void ADC_StructInit(ADC_InitTypeDef* ADC_InitStruct)
{
    /* Initialize the ADC_RefVoltage member */
    ADC_InitStruct->ADC_RefVoltage = ADC_RefVoltage_AVDD;

    /* Initialize the ADC_ConversionMode member */
    ADC_InitStruct->ADC_ConversionMode = ADC_ConversionMode_Single;
    
    /* Initialize the ADC_ConversionMode member */
    ADC_InitStruct->ADC_SocOutEn = ADC_SOC_OUT_DISABLE;

    /* Initialize the ADC_ChannelPointer member */
    ADC_InitStruct->ADC_SequencePointer = ADC_Sequence_0;

    /* Initialize the ADC_NumOfConversion member */
    ADC_InitStruct->ADC_NumOfConversion = 1;

    /* Initialize the ADC_ClkPrescaler member */
    ADC_InitStruct->ADC_Prescaler = ADC_Prescaler_2;

    /* Initialize the ADC_TwoSampleGap member */
    ADC_InitStruct->ADC_TwoSampleGap = ADC_TwoSampleGap_0Cycles;

    /* Initialize the ADC_SampleTime member */
    ADC_InitStruct->ADC_SampleTime = ADC_SampleTime_6Cycles;
}

/**
  * @brief  Initializes the ADC peripheral according to the specified parameters 
  *         in the ADC_InitStruct.
  * @param  ADC_InitStruct: pointer to a @ref ADC_InitTypeDef structure that contains
  *         the configuration information for the specified ADC peripheral.
  * @retval None
  */
void ADC_Init(ADC_InitTypeDef* ADC_InitStruct)
{
    uint16_t tempReg1 = 0;
    uint32_t tempReg2 = 0;

    /* Check the parameters */
    assert_param(IS_ADC_REF_VOLTAGE(ADC_InitStruct->ADC_RefVoltage));
    assert_param(IS_ADC_CONVERSION_MODE(ADC_InitStruct->ADC_ConversionMode));
    assert_param(IS_ADC_SOC_OUT_EN(ADC_InitStruct->ADC_SocOutEn));
    assert_param(IS_ADC_CONVERSION_NUMBER(ADC_InitStruct->ADC_NumOfConversion));
    assert_param(IS_ADC_CLK_PRESCALER(ADC_InitStruct->ADC_Prescaler));
    assert_param(IS_ADC_TWO_SAMPLE_GAP(ADC_InitStruct->ADC_TwoSampleGap));
    assert_param(IS_ADC_SAMPLE_TIME(ADC_InitStruct->ADC_SampleTime));
    

    tempReg1 = (uint16_t)ADC->ADCON1.V32;
    tempReg2 = (uint16_t)ADC->ADCON2.V32;

    /* Clear the ADCTU/MOD/REFC/ADSTRS bits */
    tempReg1 &= ~(ADC_ADCON1_ADCTU_Msk | ADC_ADCON1_REFC_Msk | ADC_ADCON1_ADOUT_Msk);
    tempReg1 |= ADC_InitStruct->ADC_ConversionMode  \
                | ADC_InitStruct->ADC_RefVoltage \
                | ADC_InitStruct->ADC_SocOutEn ;
    ADC->ADCON1.V32 = tempReg1;

    /* Clear the TADC/ADMAXCH/TS/TGAP bits */
    tempReg2 &= ~(ADC_ADCON2_TADC_Msk | ADC_ADCON2_ADMAXCH_Msk | ADC_ADCON2_TS_Msk | ADC_ADCON2_TGAP_Msk);
    tempReg2 |= ADC_InitStruct->ADC_Prescaler| ((ADC_InitStruct->ADC_NumOfConversion - 1) << 4) \
                | ADC_InitStruct->ADC_SampleTime | ADC_InitStruct->ADC_TwoSampleGap;
    
    if (ADC_InitStruct->ADC_ConversionMode == ADC_ConversionMode_Discontinuous)
    {
        /* Check the channel point register */
        assert_param(IS_ADC_SEQUENCE(ADC_InitStruct->ADC_SequencePointer));
        
        ADC->ADPCH &= ~ADC_ADPCH_ADPCH_Msk;
        
        /* To change this bit, ADSOC must be 0 */
        ADC->ADPCH |= (ADC_InitStruct->ADC_SequencePointer >> 4);
    }
    
    ADC->ADCON2.V32 = tempReg2;
}

/**
  * @}
  */
  

/** @defgroup ADC_Group2 ADC Config
 *  @brief    ADC Config
 *  @verbatim   
 ===============================================================================
                               ADC Config
 ===============================================================================

  * @endverbatim
  * @{
  */
  
/**
  * @brief  Open or close the specified ADC peripheral.  
  * @param  OnOffState: state of the ADC peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void ADC_OnOff(SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        /* Enable the ADC peripheral */
        ADC->ADCON1.V32 |= ADC_ADCON1_ADON_Msk;
    }
    else
    {
        /* Disable the ADC peripheral */
        ADC->ADCON1.V32 &= ~ADC_ADCON1_ADON_Msk;
    }
}

/**
  * @brief  Start conversion of the specified ADC peripheral.  
  * @retval None
  */
void ADC_SoftwareStartConversion(void)
{
    ADC->ADCON1.V32 |= ADC_ADCON1_ADSOC_Msk;
}

/**
  * @brief  Stop conversion of the specified ADC peripheral.  
  * @note   This function only can be used in ADC Continuous Mode.
   * @retval None
  */
void ADC_SoftwareStopConversion(void)
{
    ADC->ADCON1.V32 &= ~ADC_ADCON1_ADSOC_Msk;
}


/**
  * @brief  Get start conversion of the specified ADC peripheral.  
  * @retval ADC start status
  */
SH_FlagStatus ADC_GetSoftwareStartConversionStatus(void)
{
    SH_FlagStatus ADC_StartStatus = SH_RESET;

    if ((ADC->ADCON1.V32 & ADC_ADCON1_ADSOC_Msk) != (uint32_t)SH_RESET)
    {
        ADC_StartStatus = SH_SET;
    }
    else
    {
        ADC_StartStatus = SH_RESET;
    }

    return ADC_StartStatus;
}


/**
  * @brief  Configure the selected ADC channel for its sample sequence and whether enable or
  *         disable two sample gap function.
  * @param  ADC_Channel: specifie ADC channel which to be configured.
  *          This parameter can be one of the following values:
  *           @arg ADC_Channel_0 
  *           @arg ADC_Channel_1
  *           @arg ADC_Channel_2
  *           @arg ADC_Channel_3
  *           @arg ADC_Channel_4
  *           @arg ADC_Channel_5
  *           @arg ADC_Channel_6
  *           @arg ADC_Channel_7
  *           @arg ADC_Channel_8
  *           @arg ADC_Channel_9
  *           @arg ADC_Channel_10
  *           @arg ADC_Channel_OP0OUT  
  *           @arg ADC_Channel_OP1OUT
  *           @arg ADC_Channel_OP2OUT
  *           @arg ADC_Channel_OP3OUT
  *           @arg ADC_Channel_VBG  
  * @param  ADC_Sequence: specifie sample sequence in sample group. 
  *          This parameter can be one of the following values:
  *            @arg ADC_Sequence_0
  *            @arg ADC_Sequence_1
  *            @arg ADC_Sequence_2
  *            @arg ADC_Sequence_3
  *            @arg ADC_Sequence_4
  *            @arg ADC_Sequence_5
  *            @arg ADC_Sequence_6
  *            @arg ADC_Sequence_7 
  *            @arg ADC_Sequence_8 
  *            @arg ADC_Sequence_9 
  *            @arg ADC_Sequence_10
  *            @arg ADC_Sequence_11
  *            @arg ADC_Sequence_12
  *            @arg ADC_Sequence_13
  *            @arg ADC_Sequence_14
  *            @arg ADC_Sequence_15  
  * @param  ADC_TwoSampleGapCmd: enable or disable ADC two sample gap function.
  *          This parameter can be one of the following values:
  *            @arg ADC_TwoSampleGap_Enable
  *            @arg ADC_TwoSampleGap_Disable
  * @retval None
  */
void ADC_ChannelConfig(uint8_t ADC_Sequence, uint8_t ADC_Channel, uint8_t ADC_TwoSampleGapCmd)
{
    uint8_t pos = 0;
    uint32_t temp = 0;
    
    /* Check the parameters */
    assert_param(IS_ADC_SEQUENCE(ADC_Sequence));
    assert_param(IS_ADC_CHANNEL(ADC_Channel));
    assert_param(IS_ADC_TWO_SAMPLE_GAP_CMD(ADC_TwoSampleGapCmd));

    pos = (ADC_Sequence >> 4);

    ADC->ADGAPON &= ~(0x01 << pos);
    ADC->ADGAPON |= (uint32_t)(ADC_TwoSampleGapCmd << pos);
    
    if(pos<8)
    {
      temp = ADC->SEQCHSEL0.V32;
      temp &= ~(ADC_SEQCHSEL0_SEQCH0_Msk << (pos << 2));
      temp |= (uint32_t)ADC_Channel << (pos << 2);
      ADC->SEQCHSEL0.V32 = temp;
    }
    else
    {
      temp = ADC->SEQCHSEL1.V32;
      temp &= ~(ADC_SEQCHSEL0_SEQCH0_Msk << ((pos-8) << 2));
      temp |= (uint32_t)ADC_Channel << ((pos-8) << 2);
      ADC->SEQCHSEL1.V32 = temp;
    }     
}

/**
  * @brief  Configure external ADTRG pins trigger ADC start to convert.  
  * @param  NewState: New state of ADC ADTRG pin trigger.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void ADC_ExtADTRGTriggerCmd(SH_FunctionalState NewState)
{
    if (NewState != SH_DISABLE)
    {
        ADC->ADCON1.V32 |= ADC_ADCON1_ADTGREN_Msk;
    }
    else
    {
        ADC->ADCON1.V32 &= ~ADC_ADCON1_ADTGREN_Msk;
    }
}

/**
  * @brief  Configure external ADTRG pin output ADC Start status.  
  * @param  NewState: New state of ADC ADTRG pin output ADC Start status.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void ADC_ExtADTRGOutputStartStatusCmd(SH_FunctionalState NewState)
{
    if (NewState != SH_DISABLE)
    {
        ADC->ADCON1.V32 |= ADC_ADCON1_ADOUT_Msk;
    }
    else
    {
        ADC->ADCON1.V32 &= ~ADC_ADCON1_ADOUT_Msk;
    }
} 

/**
  * @brief  Configure the trigger channel number of single res sample. 
  * @param  ADC_TriggerSource: Trigger Source of single res mode.
  * @param  ChannelStart: start of the channels.
  *          This parameter can be 1 to 16;  
  * @param  ChannelNum: number of the channels.
  *          This parameter can be 1 to 16;  
  * @retval None
  */
void ADC_SingleResSampleModeTriggerChannelSet(uint32_t ADC_TriggerSource, uint8_t ChannelStart, uint8_t ChannelNum)
{

    /* Check the parameters */
    assert_param(IS_ADC_TRIGGER(ADC_TriggerSource));
    assert_param(IS_ADC_SEQUENCE(ChannelStart));
    assert_param(IS_ADC_CONVERSION_NUMBER(ChannelNum));

    /* select start channel of hardware trigger */
    ADC->HARDCHSEL.V32 &= ~((uint32_t)0xF << ADC_TriggerSource);          
    ADC->HARDCHSEL.V32 |= ((uint32_t)(ChannelStart>>4) << (ADC_TriggerSource*4)); 
    /* select channel number of hardware trigger */
    ADC->HARDCHLH.V32 &= ~((uint32_t)0xF << ADC_TriggerSource);            
    ADC->HARDCHLH.V32 |=  ((uint32_t)(ChannelNum - 1) << (ADC_TriggerSource*4));       
}

/**
  * @brief  Get the specified ADC's channel data.  
  * @param  ADC_Sequence: specifie the ADC's Sample Sequence to get data.
  *          This parameter can be one of the following values:
  *            @arg ADC_Sequence_0
  *            @arg ADC_Sequence_1
  *            @arg ADC_Sequence_2
  *            @arg ADC_Sequence_3  
  *            @arg ADC_Sequence_4
  *            @arg ADC_Sequence_5
  *            @arg ADC_Sequence_6
  *            @arg ADC_Sequence_7    
  *            @arg ADC_Sequence_8 
  *            @arg ADC_Sequence_9 
  *            @arg ADC_Sequence_10
  *            @arg ADC_Sequence_11
  *            @arg ADC_Sequence_12
  *            @arg ADC_Sequence_13
  *            @arg ADC_Sequence_14
  *            @arg ADC_Sequence_15
  * @retval The specified Channel data.
  */
uint16_t ADC_GetConversionValue(uint16_t ADC_Sequence)
{
    uint16_t temp = 0;
    
    assert_param(IS_ADC_SEQUENCE(ADC_Sequence));
    temp = (uint16_t)(*(volatile unsigned long *)((uint32_t)ADC + ((ADC_Sequence >> 4) + 3)*4));
    return temp;
}

/**
  * @brief  Compare the ADC conversion value with the Max and Min data set in registers.  
  * @param  ADC_UpLimit: specifie the Max value to be compared with ADC conversion value.
  *          This parameter can be 0 to 0xFFFF.
  * @param  ADC_LowerLimit: specifie the Min value to be compared with ADC conversion value.
  *          This parameter can be 0 to 0xFFFF.  
  * @param  ADC_Sequence: specifie the ADC sample sequence to be compared.
  *          This parameter can be one of the following values:
  *            @arg ADC_Sequence_0
  *            @arg ADC_Sequence_1
  *            @arg ADC_Sequence_2
  *            @arg ADC_Sequence_3
  *            @arg ADC_Sequence_4
  *            @arg ADC_Sequence_5
  *            @arg ADC_Sequence_6
  *            @arg ADC_Sequence_7
  *            @arg ADC_Sequence_8 
  *            @arg ADC_Sequence_9 
  *            @arg ADC_Sequence_10
  *            @arg ADC_Sequence_11
  *            @arg ADC_Sequence_12
  *            @arg ADC_Sequence_13
  *            @arg ADC_Sequence_14
  *            @arg ADC_Sequence_15
  * @retval None
  */
void ADC_SequenceResultCompareConfig(uint16_t ADC_UpLimit, uint16_t ADC_LowerLimit, uint8_t ADC_Sequence)
{
    uint16_t temp = 0;
    
    /* Check the parameters */
    assert_param(IS_ADC_SEQUENCE(ADC_Sequence));

    temp = (uint16_t)ADC->ADCMPCON.V32;
    temp &= ~ADC_ADCMPCON_CSEL_Msk;
    temp |= (ADC_Sequence >> 4);
    ADC->ADCMPCON.V32 = temp;

    ADC->ADDGT = ADC_UpLimit;
    ADC->ADDLT = ADC_LowerLimit;
}

/**
  * @}
  */
  

/** @defgroup ADC_Group3 ADC Interrupt
 *  @brief    ADC Interrupt
 *  @verbatim   
 ===============================================================================
                               ADC Interrupt
 ===============================================================================

  * @endverbatim
  * @{
  */

/**
  * @brief  Enable or disable the specified ADC interrupts.  
  * @param  ADC_INT: specifie the ADC interrupt sources to be enabled or disabled.
  *          This parameter can be any combination of the following values:
  *            @arg ADC_INT_EOC
  *            @arg ADC_INT_ADG
  *            @arg ADC_INT_ADL
  * @param  NewState: new state of ADC's interrupts.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void ADC_INTConfig(uint16_t ADC_INT, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_ADC_INT(ADC_INT));
    assert_param(IS_FUNCTION_STATE(NewState));

    if (NewState != SH_DISABLE)
    {
        if ((ADC_INT & ADC_INT_EOC) == ADC_INT_EOC)
        {
            ADC->ADCON1.V32 |= ADC_ADCON1_ADIE_Msk;
        }
        if ((ADC_INT & ADC_INT_ADG) == ADC_INT_ADG)
        {
            ADC->ADCMPCON.V32 |= ADC_ADCMPCON_ADGIE_Msk;
        }
        if ((ADC_INT & ADC_INT_ADL) == ADC_INT_ADL)
        {
            ADC->ADCMPCON.V32 |= ADC_ADCMPCON_ADLIE_Msk;
        }
    }
    else
    {
        if ((ADC_INT & ADC_INT_EOC) == ADC_INT_EOC)
        {
            ADC->ADCON1.V32 &= ~ADC_ADCON1_ADIE_Msk;
        }
        if ((ADC_INT & ADC_INT_ADG) == ADC_INT_ADG)
        {
            ADC->ADCMPCON.V32 &= ~ADC_ADCMPCON_ADGIE_Msk;
        }
        if ((ADC_INT & ADC_INT_ADL) == ADC_INT_ADL)
        {
            ADC->ADCMPCON.V32 &= ~ADC_ADCMPCON_ADLIE_Msk;
        }
    }   
}

/**
  * @brief  Check whether the specified ADC flag is set or not
  * @param  ADC_Flag: specifie the flag to check.
  *          This parameter can be one of the following values:
  *            @arg ADC_FLAG_EOC
  *            @arg ADC_FLAG_ADG
  *            @arg ADC_FLAG_ADL  
  * @retval None
  */
SH_FlagStatus ADC_GetFlagStatus(uint16_t ADC_Flag)
{
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_ADC_GET_ONE_FLAG(ADC_Flag));

    if ((ADC->ADINTF.V32 & ADC_Flag) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }

    return bitStatus;
    
}

/**
  * @brief  Clear the ADC's pending flags.
  * @param  ADC_Flag: specifie the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg ADC_FLAG_EOC
  *            @arg ADC_FLAG_ADG
  *            @arg ADC_FLAG_ADL  
  * @retval None
  */
void ADC_ClearFlag(uint16_t ADC_Flag)
{
    /* Check the parameters */
    assert_param(IS_ADC_FLAG(ADC_Flag));

    ADC->ADINTF.V32 = (uint32_t)ADC_Flag << 16;
}




/**
  * @}
  */
  



/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


