/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_amoc.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_amoc.h"

/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup AMOC_MODULE AMOC 
  * @brief AMOC driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup AMOC_Exported_Functions Public Functions
  * @{
  */ 

/** @defgroup AMOC_Group1 COMP Initialization and Configuration
 *  @brief    COMP Initialization and Configuration
 *
@verbatim    
 ===============================================================================
                      COMP Initialization and Configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */


/**
  * @brief  Reset the COMPx module registers to their default reset values.
  * @param  COMPx: Where x can be 0 or 1 to select the COMP module.
  * @retval None
  */
void COMP_Reset(uint8_t COMPx)
{
    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));

    if (COMPx == COMP0)
    {
        AMOC->CMP0CON.V32 = (uint32_t)0x00;
        AMOC->CMPINTF.V32 = (uint32_t)0x10000;
    }  
    else if (COMPx == COMP1)
    {
        AMOC->CMP1CON.V32 = (uint32_t)0x00;
        AMOC->CMPINTF.V32 = (uint32_t)0x20000;
    }
}

/**
  * @brief  Fills each COMP_InitStruct member with its default value.
  * @param  COMP_InitStruct : pointer to a @ref COMP_InitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void COMP_StructInit(COMP_InitTypeDef* COMP_InitStruct)
{
    /* Initialize the COMP_NRefVoltageSource member */
    COMP_InitStruct->COMP_NRefVoltageSource = COMP_NRefVoltage_AVDD;

    /* Initialize the COMP_NRefVoltageLevel member */
    COMP_InitStruct->COMP_NRefVoltageLevel = COMP_NRefVoltageLevel_32;

    /* Initialize the COMP_NrefOutputCmd member */
    COMP_InitStruct->COMP_NrefOutputCmd = COMP_NrefOutput_Disable;
    
    /* Initialize the COMP_OutputCmd member */
    COMP_InitStruct->COMP_OutputCmd = COMP_Output_Disable;

    /* Initialize the COMP_SchmittVoltage member */
    COMP_InitStruct->COMP_SchmittVoltage = COMP_SchmittVoltage_None;

    /* Initialize the COMP_IntSelect member */
    COMP_InitStruct->COMP_IntSelect = COMP_INT_NONE;

    /* Initialize the COMP_NSelect member */
    COMP_InitStruct->COMP_NSelect = COMP_NSelect_NPin;

    /* Initialize the COMP_PSelect member */
    COMP_InitStruct->COMP_PSelect = COMP_PSelect_P0;

    /* Initialize the COMP_Filter member */
    COMP_InitStruct->COMP_Filter = COMP_Filter_1us;

    /* Initialize the COMP_SynPwmSource member */
    COMP_InitStruct->COMP_SynPwmSource = COMP_SYN_PWM_NONE;

    /* Initialize the COMP_SynPwmPoly member */
    COMP_InitStruct->COMP_SynPwmPoly = COMP_SYN_PWM_DUTY;
}

/**
  * @brief  Initialize the COMP module according to the specified parameters
  *         in COMP_InitStruct
  * @param  COMPx: where x can be 0 or 1 to select the COMP module.
  * @param  COMP_InitStruct: pointer to an COMP_InitTypeDef structure that contains 
  *         the configuration information for the specified COMP module.
  * @retval None
  */
void COMP_Init(uint8_t COMPx, COMP_InitTypeDef* COMP_InitStruct)
{
    uint32_t temp = 0;
    uint32_t tempAddress = 0;
    
    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));
    assert_param(IS_COMP_N_REF_VOLTAGE_SOURCE(COMP_InitStruct->COMP_NRefVoltageSource));
    assert_param(IS_COMP_N_REF_VOLTAGE_LEVEL(COMP_InitStruct->COMP_NRefVoltageLevel));
    assert_param(IS_COMP_OUTPUT_CMD(COMP_InitStruct->COMP_OutputCmd));
    assert_param(IS_COMP_SCHMITT_VOLTAGE(COMP_InitStruct->COMP_SchmittVoltage));
    assert_param(IS_COMP_NREF_OUTPUT_CMD(COMP_InitStruct->COMP_NrefOutputCmd));   
    assert_param(IS_COMP_N_SELECT(COMP_InitStruct->COMP_NSelect));
    assert_param(IS_COMP_FILTER(COMP_InitStruct->COMP_Filter));
    assert_param(IS_COMP_INT(COMP_InitStruct->COMP_IntSelect));
    assert_param(IS_CMP_SYN_PWM_SELECT(COMP_InitStruct->COMP_SynPwmSource));
    assert_param(IS_CMP_SYN_PWM_DUTY(COMP_InitStruct->COMP_SynPwmPoly));
    

    tempAddress = AMOC_BASE + COMPx;

    temp = *(__IO uint32_t *)tempAddress;

    /* Clear all bits except bit15(CMPxEN),bit22~23&bit27(CMP0_Auto_Rotation) */
    temp &= 0x08C08000;
    
    if (COMPx == COMP0)
    {
        assert_param(IS_COMP_0_P_SELECT(COMP_InitStruct->COMP_PSelect));
    }
    else
    {
        assert_param(IS_COMP_1_P_SELECT(COMP_InitStruct->COMP_PSelect));
    }
   
    temp |= COMP_InitStruct->COMP_PSelect \
            | COMP_InitStruct->COMP_Filter \
            | COMP_InitStruct->COMP_NSelect \
            | COMP_InitStruct->COMP_SchmittVoltage \
            | COMP_InitStruct->COMP_OutputCmd \
            | (COMP_InitStruct->COMP_NRefVoltageLevel << 16)\
            | COMP_InitStruct->COMP_NRefVoltageSource\
            | COMP_InitStruct->COMP_NrefOutputCmd\
            | COMP_InitStruct->COMP_IntSelect\
            | COMP_InitStruct->COMP_SynPwmSource\
            | COMP_InitStruct->COMP_SynPwmPoly;
    
    *(__IO uint32_t *)tempAddress = temp;
}

/**
  * @brief  Open or close the COMP Module.
  * @param  COMPx: where x can be 0 to 1 to select the COMP module.
  * @param  OnOffState: state of the COMP module.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void COMP_OnOff(uint8_t COMPx, SH_CmdState OnOffState)
{
    uint32_t tempAddress = 0;
    
    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));
    assert_param(IS_CMD_STATE(OnOffState));

    tempAddress = AMOC_BASE + COMPx;
    
    if (OnOffState != SH_OFF)
    {
        *(__IO uint32_t *)tempAddress |= (uint32_t)AMOC_CMP0CON_CMP0EN_Msk;
    }
    else
    {
        *(__IO uint32_t *)tempAddress &= (~(uint32_t)AMOC_CMP0CON_CMP0EN_Msk);
    }
}


/**
  * @brief  Return the output level (high or low) of the selected COMP module. 
  * @param  COMPx: where x can be 0 or 1 to select COMP module. 
  * @retval Return the selected COMP module output level: low or high.      
  */
SH_FlagStatus COMP_GetOutputLevel(uint8_t COMPx)
{
    uint8_t OutputLevel = 0;
    uint32_t temp = 0;
    uint32_t tempAddress = 0;

    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));

    tempAddress = AMOC_BASE + COMPx;
    temp = *(__IO uint32_t *)tempAddress;
    
    if ((temp & AMOC_CMP0CON_C0OUT_Msk) != (uint32_t)SH_RESET)
    {
        OutputLevel = SH_SET;
    }
    else
    {
        OutputLevel = SH_RESET;
    }

    return (SH_FlagStatus)OutputLevel;
}

/**
  * @}
  */
  

/** @defgroup AMOC_Group2 Comp Interrupt
 *  @brief    Comp Interrupt
 *  @verbatim   
 ===============================================================================
                               Comp Interrupt
 ===============================================================================

  * @endverbatim
  * @{
  */
  

/**
  * @brief  Enable or disable the specified COMP interrupts.
  * @param  COMPx: where x can be 0 or 1 to select the COMP module.
  * @param  COMP_INT: specifie the COMP interrupt source to be enabled or disabled.
  *          This parameter can be any combination of the following values:
  *            @arg COMP_INT_NONE
  *            @arg COMP_INT_RISING
  *            @arg COMP_INT_FALLING
  *            @arg COMP_INT_DOUBLE
  * @retval None
  */
void COMP_INTConfig(uint8_t COMPx, uint16_t COMP_INT)
{
    uint32_t tempAddress = 0;
    
    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));
    assert_param(IS_COMP_INT(COMP_INT));

    tempAddress = AMOC_BASE + COMPx;

    *(__IO uint32_t *)tempAddress &= ~((uint32_t)AMOC_CMP0CON_C0IES_Msk);
    *(__IO uint32_t *)tempAddress |= (uint32_t)COMP_INT;
}

/**
  * @brief  Check whether the specified flag is set or not.
  * @param  COMPx: where x can be 0 or 1 to select the COMP module.
  * @retval The new state of the Flag(SH_SET or SH_RESET).
  */
SH_FlagStatus COMP_GetFlagStatus(uint8_t COMPx)
{
    uint8_t pos = 0;
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));
    
    pos = COMPx >> 2;

    if ((AMOC->CMPINTF.V32 & (AMOC_CMPINTF_C0IF_Msk << pos)) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @brief  Clear the COMP's pending flag.
  * @param  COMPx: where x can be 0 or 1 to select the COMP module.
  * @retval None
  */
void COMP_ClearFlag(uint8_t COMPx)
{
    /* Check the parameters */
    assert_param(IS_COMP_ALL_PERIPH(COMPx));
    
    if (COMPx == COMP0)
    {
        AMOC->CMPINTF.V32 = (uint32_t)0x10000;
    }
    else
    {
        AMOC->CMPINTF.V32 = (uint32_t)0x20000;    
    }
}

/**
  * @}
  */
  

/** @defgroup AMOC_Group3 Comp0 Auto Rotation
 *  @brief    Comp0 Auto Rotation
 *  @verbatim   
 ===============================================================================
                               Comp0 Auto Rotation
 ===============================================================================

  * @endverbatim
  * @{
  */
  
/**
  * @brief  set CMP0 automatic rotation comparison
  * @param  AutoRotationCommand: enable or disable
  * @param  AutoRotationGap: select gap timer
  * @retval None
  */
void COMP0_SetAutoRotation(uint32_t AutoRotationCommand, uint32_t AutoRotationGap)
{
    /* Check the parameters */
    assert_param(IS_CMP0_AUTO_ROTATION_CMD(AutoRotationCommand));
    assert_param(IS_CMP0_AUTO_ROTATION_GAP(AutoRotationGap));

    AMOC->CMP0CON.V32 &= ~(AMOC_CMP0CON_CP0NOUTEN_Msk | AMOC_CMP0CON_CMPGAP_Msk);
    AMOC->CMP0CON.V32 |= (AutoRotationCommand | AutoRotationGap);
}

/**
  * @brief  Check whether the specified flag is set or not.
  * @retval The new state of the output(SH_SET or SH_RESET).
  */
SH_FlagStatus COMP0_GetAutoOutLevel(uint32_t C0OUTx)
{
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_CMP0_AUTO_OUTPUT_SELECT(C0OUTx));

    if ((AMOC->CMP0CON.V32 & C0OUTx) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @}
  */



/** @defgroup AMOC_Group4 OP configuration
 *  @brief    OP configuration
 *
@verbatim    
 ===============================================================================
                                OP configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */


/**
  * @brief  On or Off OP0 gain function.
  * @param  OnOffState: state of the OP0 gain function.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */  
void OP0_GainOnOff(SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));
    
    AMOC->OPCON.BIT.OP0GEN = (uint32_t)OnOffState;
}

/**
  * @brief  Config the OP0 gain factor.
  * @param  OP0_GainSelect: the OP0 gain factor.
  *          This parameter can be one of the following values:
  *            @arg OP0_GAIN_MUL_1
  *            @arg OP0_GAIN_MUL_4
  *            @arg OP0_GAIN_MUL_6
  *            @arg OP0_GAIN_MUL_8
  *            @arg OP0_GAIN_MUL_12
  *            @arg OP0_GAIN_MUL_16
  *            @arg OP0_GAIN_MUL_24
  *            @arg OP0_GAIN_MUL_32
  * @retval None
  */
void OP0_GainConfig(uint8_t OP0_GainSelect)
{
    /* Check the parameters */
    assert_param(IS_OP0_GAIN_MUL(OP0_GainSelect));
    
    AMOC->OPCON.V32 &= ~AMOC_OPCON_OP0GSEL_Msk;
    AMOC->OPCON.V32 |= OP0_GainSelect;    
}

/**
  * @brief  On or off the OP Module.
  * @param  OPx: where x can be 1 to 3 to select the OP module.
  * @param  OnOffState: state of the OP module.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void OP_OnOff(uint8_t OPx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_OP_ALL_PERIPH(OPx));
    assert_param(IS_CMD_STATE(OnOffState));    

    if (OnOffState != SH_OFF)
    {
        /* Enable the OP module */
        AMOC->OPCON.V32 |= OPx; 
    }
    else
    {
        /* Disable the OP module */
        AMOC->OPCON.V32 &= ~OPx;
    }
}





/**
  * @}
  */



/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


