/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_can.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_can.h"

/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup CAN_MODULE CAN
  * @brief CAN driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup CAN_Exported_Functions Public Functions
  * @{
  */ 

/**
  * @brief  Initializes the CANx structure CAN_InitTypeDef
  * @param  CAN_InitStruct: pointer to a @ref CAN_InitTypeDef structure that contains
  *         the configuration information for the specified CAN peripheral.
  * @retval None
  */
void CAN_StructInit(CAN_InitTypeDef* CAN_InitStruct)
{
    /*if can module clock is 42MHz, can bus baudrate is 500Kbps*/
    CAN_InitStruct->CAN_BaudRate.BIT.psc = 5;
    CAN_InitStruct->CAN_BaudRate.BIT.sjw = 1;
    CAN_InitStruct->CAN_BaudRate.BIT.tseg1 = 5;
    CAN_InitStruct->CAN_BaudRate.BIT.tseg2 = 4;
    
    CAN_InitStruct->CAN_Mode.BIT.enable = 1;
    CAN_InitStruct->CAN_Mode.BIT.receive_level = 0;
    CAN_InitStruct->CAN_Mode.BIT.send_level = 0;
    CAN_InitStruct->CAN_Mode.BIT.buff_lock = 0;
    CAN_InitStruct->CAN_Mode.BIT.tsync_enable = 0;
    CAN_InitStruct->CAN_Mode.BIT.data_dir = 0;
    CAN_InitStruct->CAN_Mode.BIT.listen_only = 0;
    CAN_InitStruct->CAN_Mode.BIT.ignore_ack = 0;
    CAN_InitStruct->CAN_Mode.BIT.loop_back = 0;
    CAN_InitStruct->CAN_Mode.BIT.internal = 0;
    CAN_InitStruct->CAN_Mode.BIT.diag_enable = 0;
    CAN_InitStruct->CAN_Mode.BIT.err_int_type = 0;
}

/**
  * @brief  Initializes the CANx peripheral according to the specified parameters 
  *         in the CAN_InitStruct.
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_InitStruct: pointer to a @ref CAN_InitTypeDef structure that contains
  *         the configuration information for the specified CAN peripheral.
  * @retval None
  */
void CAN_Init(CAN_TypeDef* CANx, const CAN_InitTypeDef* CAN_InitStruct)
{
    uint32_t* pv;
    int i;    

    /* Check the parameters */
    assert_param(IS_CAN_SJW(CAN_InitStruct->CAN_BaudRate.BIT.sjw));
    
    //clear CAN Registers
    pv = (uint32_t*)(void*)(CANx);
    for(i = 0; i < 0x200/4; i++)
    {
        *pv = 0;
        pv++;
    }
    
    CANx->CTIM.V32 = CAN_InitStruct->CAN_BaudRate.V32;
    CANx->CGCR.V32 = CAN_InitStruct->CAN_Mode.V32;
}

/**
  * @brief  Initializes the CANx ID Mask bit  
  *         
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  gmask: pointer to a @ref CAN_MaskTypeDef structure that contains general mask bits for(buffer0~13)
  * @param  bmask: pointer to a @ref CAN_MaskTypeDef structure that contains base mask bits for(buffer14)
  *         
  * @retval None
  */
void CAN_SetMask(CAN_TypeDef* CANx, const CAN_MaskTypeDef* gmask, const CAN_MaskTypeDef* bmask)
{
   CANx->GMSKB.BIT.GMXGM  = (gmask->idmask>>18)&0x7FF;
   CANx->GMSKB.BIT.XGM   = (gmask->idmask>>15)&0x7;
   CANx->GMSKX.BIT.XGM   = (gmask->idmask)&0x7FFF;       
   CANx->GMSKB.BIT.IDE  = gmask->idemask;
   CANx->GMSKX.BIT.XRTR = gmask->rtrmask;
   CANx->GMSKB.BIT.RTRSRR  = gmask->rtrmask;
    
   CANx->BMSKB.BIT.BMXBM  = (bmask->idmask>>18)&0x7FF;
   CANx->BMSKB.BIT.XBM   = (bmask->idmask>>15)&0x7;
   CANx->BMSKX.BIT.XBM   = (bmask->idmask)&0x7FFF;       
   CANx->BMSKB.BIT.IDE  = bmask->idemask;
   CANx->BMSKX.BIT.XRTR = bmask->rtrmask;
   CANx->BMSKB.BIT.RTRSRR  = bmask->rtrmask;
}

/**
  * @brief  Initiates and transmits a CAN frame.
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: number of CAN Buffer, where x can be 0 to 14 to select CAN buffer.
  * @param  CAN_Msg: pointer to a message object.
  * @param  eCmd:    Send command @ref E_CAN_CmdSend
  *            @arg eCANCmd_Send_Data
  *            @arg eCANCmd_Send_Wait_Remote
  *            @arg eCANCmd_Wait_Remote
  * @retval none
  */
void CAN_StartSendMsg(CAN_TypeDef* CANx, uint32_t CAN_Bufferx, const CAN_MsgTypeDef* CAN_Msg,E_CAN_CmdSend eCmd)
{
    CAN_BufTypeDef* pCanBuf = (CAN_BufTypeDef*)(void*)CANx;
    pCanBuf += CAN_Bufferx;
    
    pCanBuf->CNTSTAT.BIT.ST = CAN_ST_TX_NOT_ACTIVE;
    pCanBuf->ID0.V32 = 0;
    pCanBuf->ID1.V32 = 0;
    
    pCanBuf->CNTSTAT.BIT.DLC = CAN_Msg->len;
    pCanBuf->CNTSTAT.BIT.PRI = CAN_Msg->ePri;
    if(CANx->CGCR.BIT.DDIR == 0)
    {
        pCanBuf->DATA0.BIT.DATAH = CAN_Msg->data[0];
        pCanBuf->DATA0.BIT.DATAL = CAN_Msg->data[1];
        pCanBuf->DATA1.BIT.DATAH = CAN_Msg->data[2];
        pCanBuf->DATA1.BIT.DATAL = CAN_Msg->data[3];
        pCanBuf->DATA2.BIT.DATAH = CAN_Msg->data[4];
        pCanBuf->DATA2.BIT.DATAL = CAN_Msg->data[5];
        pCanBuf->DATA3.BIT.DATAH = CAN_Msg->data[6];
        pCanBuf->DATA3.BIT.DATAL = CAN_Msg->data[7];
    }
    else 
    {
        pCanBuf->DATA3.BIT.DATAL = CAN_Msg->data[0];
        pCanBuf->DATA3.BIT.DATAH = CAN_Msg->data[1];
        pCanBuf->DATA2.BIT.DATAL = CAN_Msg->data[2];
        pCanBuf->DATA2.BIT.DATAH = CAN_Msg->data[3];
        pCanBuf->DATA1.BIT.DATAL = CAN_Msg->data[4];
        pCanBuf->DATA1.BIT.DATAH = CAN_Msg->data[5];
        pCanBuf->DATA0.BIT.DATAL = CAN_Msg->data[6];
        pCanBuf->DATA0.BIT.DATAH = CAN_Msg->data[7];
    }
    
    if(CAN_Msg->ie)
    {            
        CANx->CICEN.V32 |= 1UL<<(CAN_Bufferx);
        CANx->CIEN.V32 |= 1UL<<CAN_Bufferx;
    }

    switch(CAN_Msg->eMsgType)
    {
        case eFrameStd:
            pCanBuf->ID1.BIT.ID = CAN_Msg->id & 0x7FF;
            break;
        case eFrameExt:
            pCanBuf->ID0.BIT.ID   = CAN_Msg->id & 0x7FFF;
            pCanBuf->ID1.BIT.IDEX = (CAN_Msg->id>>15) & 0x7;
            pCanBuf->ID1.BIT.ID   = (CAN_Msg->id>>18) & 0x7FF;
            pCanBuf->ID1.BIT.IDE  = 1;
            pCanBuf->ID1.BIT.SRR  = 1;
            break;
        case eFrameStdRt:
            pCanBuf->ID1.BIT.ID = CAN_Msg->id & 0x7FF;
            pCanBuf->ID1.BIT.SRR = 1;
            break;
        case eFrameExtRt:
            pCanBuf->ID0.BIT.ID   = CAN_Msg->id & 0x7FFF;
            pCanBuf->ID1.BIT.IDEX = (CAN_Msg->id>>15) & 0x7;
            pCanBuf->ID1.BIT.ID   = (CAN_Msg->id>>18) & 0x7FF;
            pCanBuf->ID1.BIT.SRR  = 1;
            pCanBuf->ID0.BIT.RTR  = 1;
            break;        
    }    
    if(eCmd == eCANCmd_Send_Data)
        pCanBuf->CNTSTAT.BIT.ST = CAN_ST_TX_ONCE;
    else if(eCmd == eCANCmd_Wait_Remote)
        pCanBuf->CNTSTAT.BIT.ST = CAN_ST_TX_RTR;
    else if(eCmd == eCANCmd_Send_Wait_Remote)
        pCanBuf->CNTSTAT.BIT.ST = CAN_ST_TX_ONCE_RTR;    
    
}

/**
  * @brief  Receive a CAN frame.
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: number of CAN Buffer, where x can be 0 to 14 to select CAN buffer.
  * @param  CAN_Msg: pointer to a message object.
  * @retval none
  */
void CAN_StartReceiveMsg(CAN_TypeDef* CANx, uint32_t CAN_Bufferx, const CAN_MsgTypeDef* CAN_Msg)
{
    CAN_BufTypeDef* pCanBuf = (CAN_BufTypeDef*)(void*)CANx;
    pCanBuf += CAN_Bufferx;
    
    pCanBuf->CNTSTAT.BIT.ST = CAN_ST_RX_NOT_ACTIVE;
    pCanBuf->ID0.V32 = 0;
    pCanBuf->ID1.V32 = 0;
    pCanBuf->CNTSTAT.BIT.DLC = CAN_Msg->len;
    pCanBuf->CNTSTAT.BIT.PRI = CAN_Msg->ePri;

    switch(CAN_Msg->eMsgType)
    {
        case eFrameStd:
            pCanBuf->ID1.BIT.ID = CAN_Msg->id & 0x7FF;
            break;
        case eFrameExt:
            pCanBuf->ID0.BIT.ID   = CAN_Msg->id & 0x7FFF;
            pCanBuf->ID1.BIT.IDEX = (CAN_Msg->id>>15) & 0x7;
            pCanBuf->ID1.BIT.ID   = (CAN_Msg->id>>18) & 0x7FF;
            pCanBuf->ID1.BIT.IDE  = 1;
            pCanBuf->ID1.BIT.SRR  = 1;
            break;
        case eFrameStdRt:
            pCanBuf->ID1.BIT.ID = CAN_Msg->id & 0x7FF;
            pCanBuf->ID1.BIT.SRR = 1;
            break;
        case eFrameExtRt:
            pCanBuf->ID0.BIT.ID   = CAN_Msg->id & 0x7FFF;
            pCanBuf->ID1.BIT.IDEX = (CAN_Msg->id>>15) & 0x7;
            pCanBuf->ID1.BIT.ID   = (CAN_Msg->id>>18) & 0x7FF;
            pCanBuf->ID1.BIT.SRR  = 1;
            pCanBuf->ID0.BIT.RTR  = 1;
            break;        
    }
    
    if(CAN_Msg->ie)
    {            
        CANx->CICEN.V32 |= 1UL<<(CAN_Bufferx);
        CANx->CIEN.V32 |= 1UL<<CAN_Bufferx;
    }
        
    pCanBuf->CNTSTAT.BIT.ST = CAN_ST_RX_READY;        
    
}

/**
  * @brief  Receive a CAN frame.
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: number of CAN Buffer, where x can be 0 to 14 to select CAN buffer.
  * @param  CAN_Msg: pointer to a message object.
  * @retval none
  */
void CAN_ReadReceivedMsg(CAN_TypeDef* CANx, uint32_t CAN_Bufferx, CAN_MsgTypeDef* CAN_Msg)
{
    CAN_BufTypeDef* pCanBuf = (CAN_BufTypeDef*)(void*)CANx;
    pCanBuf += CAN_Bufferx;
    
    CAN_Msg->len = pCanBuf->CNTSTAT.BIT.DLC ;
    CAN_Msg->ePri = pCanBuf->CNTSTAT.BIT.PRI;
    
    if(CANx->CGCR.BIT.DDIR == 0)
    {
        CAN_Msg->data[0] = pCanBuf->DATA0.BIT.DATAH;
        CAN_Msg->data[1] = pCanBuf->DATA0.BIT.DATAL;
        CAN_Msg->data[2] = pCanBuf->DATA1.BIT.DATAH;
        CAN_Msg->data[3] = pCanBuf->DATA1.BIT.DATAL;
        CAN_Msg->data[4] = pCanBuf->DATA2.BIT.DATAH;
        CAN_Msg->data[5] = pCanBuf->DATA2.BIT.DATAL;
        CAN_Msg->data[6] = pCanBuf->DATA3.BIT.DATAH;
        CAN_Msg->data[7] = pCanBuf->DATA3.BIT.DATAL;
    }
    else 
    {
        CAN_Msg->data[0] = pCanBuf->DATA3.BIT.DATAL;
        CAN_Msg->data[1] = pCanBuf->DATA3.BIT.DATAH;
        CAN_Msg->data[2] = pCanBuf->DATA2.BIT.DATAL;
        CAN_Msg->data[3] = pCanBuf->DATA2.BIT.DATAH;
        CAN_Msg->data[4] = pCanBuf->DATA1.BIT.DATAL;
        CAN_Msg->data[5] = pCanBuf->DATA1.BIT.DATAH;
        CAN_Msg->data[6] = pCanBuf->DATA0.BIT.DATAL;
        CAN_Msg->data[7] = pCanBuf->DATA0.BIT.DATAH;
    }
    if(pCanBuf->ID1.BIT.IDE)
    {
        CAN_Msg->id = (uint32_t)(pCanBuf->ID1.BIT.ID <<18)|((uint32_t)pCanBuf->ID1.BIT.IDEX<<15)| (pCanBuf->ID0.BIT.ID);
        if(pCanBuf->ID0.BIT.RTR)
            CAN_Msg->eMsgType = eFrameExtRt;
        else
            CAN_Msg->eMsgType = eFrameExt;
    }
    else
    {
        CAN_Msg->id = pCanBuf->ID1.BIT.ID;
        if(pCanBuf->ID1.BIT.SRR)
            CAN_Msg->eMsgType = eFrameStdRt;
        else
            CAN_Msg->eMsgType = eFrameStd;
    }

}

/**
  * @brief  Receive a CAN frame.
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: the index of CAN Buffer, where x can be 0 to 14 to select CAN buffer.
  * @retval none
  */
void CAN_ClearBuffer(CAN_TypeDef* CANx, uint32_t CAN_Bufferx)
{
    CAN_BufTypeDef* pCanBuf = (CAN_BufTypeDef*)(void*)CANx;
    pCanBuf += CAN_Bufferx;
    pCanBuf->CNTSTAT.V32 = 0;
    pCanBuf->BUF_TSTP=0;
    pCanBuf->ID0.V32 = 0;
    pCanBuf->ID1.V32 = 0;
    pCanBuf->DATA0.V32 = 0;
    pCanBuf->DATA1.V32 = 0;
    pCanBuf->DATA2.V32 = 0;
    pCanBuf->DATA3.V32 = 0;
}


/**
  * @brief  Check whether the specified CAN Buffer status is set or not
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: where x can be 0 to 14 to select the CAN peripheral.
  * @retval Buffer Status
  *            @arg CAN_ST_RX_NOT_ACTIVE
  *            @arg CAN_ST_BUSY
  *            @arg CAN_ST_RXREADY      
  *            @arg CAN_ST_RXFULL       
  *            @arg CAN_ST_RX_OVERRUN   
  *            @arg CAN_ST_TX_NOT_ACTIVE
  *            @arg CAN_ST_TX_ONCE      
  *            @arg CAN_ST_TX_RTR       
  *            @arg CAN_ST_TX_ONCE_RTR    
 */
uint32_t CAN_GetBufferStatus(CAN_TypeDef* CANx, uint32_t CAN_Bufferx)
{
    CAN_BufTypeDef* pCanBuf = (CAN_BufTypeDef*)(void*)CANx;
    pCanBuf += CAN_Bufferx;
    return pCanBuf->CNTSTAT.BIT.ST;    
}

/**
  * @brief  Enable or disable the Buffer Lock function.  
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  NewState: state of the CAN Buffer Lock.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void CAN_BufferLockCmd(CAN_TypeDef* CANx, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
    assert_param(IS_FUNCTION_STATE(NewState));
    
    if (NewState != SH_DISABLE)
    {
        CANx->CGCR.V32 |= CAN_CGCR_BUFFLOCK_Msk;
    }
    else
    {
        CANx->CGCR.V32 &= ~CAN_CGCR_BUFFLOCK_Msk;
    }       
}

/**
  * @brief  Returns the CANx Transmit Error Counter (TEC).
  * @param  CANx: where x can be 0 to to select the CAN peripheral.
  * @retval CAN Transmit Error Counter. 
  */
uint32_t CAN_GetTransmitErrorCounter(CAN_TypeDef* CANx)
{
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
    
    return CANx->CANEC.BIT.TEC;
}

/**
  * @brief  Returns the CANx Receive Error Counter (REC).
  * @note   In case of an error during reception, this counter is incremented 
  *         by 1 or by 8 depending on the error condition as defined by the CAN 
  *         standard. After every successful reception, the counter is 
  *         decremented by 1 or reset to 120 if its value was higher than 128. 
  *         When the counter value exceeds 127, the CAN controller enters the 
  *         error passive state.  
  * @param  CANx: where x can be 0 to to select the CAN peripheral.  
  * @retval CAN Receive Error Counter. 
  */
uint32_t CAN_GetReceiveErrorCounter(CAN_TypeDef* CANx)
{
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
    
    return CANx->CANEC.BIT.REC;
}


/**
  * @brief  Enable or disable the specified CAN interrupts.  
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: specifie the CAN buffer index.
 *          This parameter can be one of the following values:
  *            @arg CAN_INT_BUF0 
  *            @arg CAN_INT_BUF1 
  *            @arg CAN_INT_BUF2 
  *            @arg CAN_INT_BUF3 
  *            @arg CAN_INT_BUF4 
  *            @arg CAN_INT_BUF5 
  *            @arg CAN_INT_BUF6 
  *            @arg CAN_INT_BUF7 
  *            @arg CAN_INT_BUF8 
  *            @arg CAN_INT_BUF9 
  *            @arg CAN_INT_BUF10
  *            @arg CAN_INT_BUF11
  *            @arg CAN_INT_BUF12
  *            @arg CAN_INT_BUF13
  *            @arg CAN_INT_BUF14
  *            @arg CAN_INT_ERR  
  * @param  NewState: new state of ADC's interrupts.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void CAN_INTConfig(CAN_TypeDef* CANx, uint32_t CAN_Bufferx, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
//    assert_param(IS_CAN_INT(CAN_INT)); //always true
    assert_param(IS_FUNCTION_STATE(NewState));
    
    if (NewState != SH_DISABLE)
    {
        CANx->CIEN.V32 |= (1UL<<CAN_Bufferx);
    }
    else
    {
        CANx->CIEN.V32 &= ~(1UL<<CAN_Bufferx);
    }      
}

/**
  * @brief  Enable or disable the specified CAN interrupt Code.  
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: specifie the CAN buffer index.
 *          This parameter can be one of the following values:
  *            @arg CAN_INT_BUF0 
  *            @arg CAN_INT_BUF1 
  *            @arg CAN_INT_BUF2 
  *            @arg CAN_INT_BUF3 
  *            @arg CAN_INT_BUF4 
  *            @arg CAN_INT_BUF5 
  *            @arg CAN_INT_BUF6 
  *            @arg CAN_INT_BUF7 
  *            @arg CAN_INT_BUF8 
  *            @arg CAN_INT_BUF9 
  *            @arg CAN_INT_BUF10
  *            @arg CAN_INT_BUF11
  *            @arg CAN_INT_BUF12
  *            @arg CAN_INT_BUF13
  *            @arg CAN_INT_BUF14
  *            @arg CAN_INT_ERR  
  * @param  NewState: new state of ADC's interrupts.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void CAN_INTCodeConfig(CAN_TypeDef* CANx, uint32_t CAN_Bufferx, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
//    assert_param(IS_CAN_INT(CAN_INT)); //always true
    assert_param(IS_FUNCTION_STATE(NewState));
    
    if (NewState != SH_DISABLE)
    {
        CANx->CICEN.V32 |= (1UL<<CAN_Bufferx);
    }
    else
    {
        CANx->CICEN.V32 &= ~(1UL<<CAN_Bufferx);
    }      
}

/**
  * @brief  Check whether the specified CAN flag is set or not
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: specifie the CAN buffer index.
  *          This parameter can be one of the following values:
  *            @arg CAN_FLAG_BUF0 
  *            @arg CAN_FLAG_BUF1 
  *            @arg CAN_FLAG_BUF2 
  *            @arg CAN_FLAG_BUF3 
  *            @arg CAN_FLAG_BUF4 
  *            @arg CAN_FLAG_BUF5 
  *            @arg CAN_FLAG_BUF6 
  *            @arg CAN_FLAG_BUF7 
  *            @arg CAN_FLAG_BUF8 
  *            @arg CAN_FLAG_BUF9 
  *            @arg CAN_FLAG_BUF10
  *            @arg CAN_FLAG_BUF11
  *            @arg CAN_FLAG_BUF12
  *            @arg CAN_FLAG_BUF13
  *            @arg CAN_FLAG_BUF14
  *            @arg CAN_FLAG_ERR  
  * @retval  SH_FlagStatus INT reset flag
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus CAN_INTGetStatus(CAN_TypeDef* CANx, uint32_t CAN_Bufferx)
{
    SH_FlagStatus bitStatus;
   
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
   
    if ((CANx->CIPND.V32 & (1UL<<CAN_Bufferx)) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }

    return bitStatus;  
}

/**
  * @brief  Check whether the specified CAN flag is set or not
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Code: pointer to interrupt code buffer.
  * @retval  SH_FlagStatus INT reset flag
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus CAN_INTGetCode(CAN_TypeDef* CANx,uint32_t* CAN_Code)
{
    SH_FlagStatus bitStatus;
      
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
    bitStatus = SH_RESET;
    *CAN_Code = CANx->CSTPND.V32;
    if(CANx->CSTPND.BIT.IRQ)
    {
        if(CANx->CSTPND.BIT.IST != 0)
        { 
            *CAN_Code = CANx->CSTPND.BIT.IST-1;
            bitStatus = SH_SET;
        }
    }
    return bitStatus;
}

/**
  * @brief  Get CAN timer counter
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @retval timer counter value
  */
uint32_t CAN_GetTimerCounter(CAN_TypeDef* CANx)
{
    return CANx->CTMR;
}
/**
  * @brief  Get error code to diagnostic
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @retval Error code
  */
CAN_ErrorCode CAN_GetErrorCode(CAN_TypeDef* CANx)
{
    CAN_ErrorCode code;
    code.V32 = CANx->CEDIAG.V32;
       
    return code;
}

/**
  * @brief  Clear the CAN's pending flags.
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  CAN_Bufferx: specifie the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg CAN_FLAG_BUF0 
  *            @arg CAN_FLAG_BUF1 
  *            @arg CAN_FLAG_BUF2 
  *            @arg CAN_FLAG_BUF3 
  *            @arg CAN_FLAG_BUF4 
  *            @arg CAN_FLAG_BUF5 
  *            @arg CAN_FLAG_BUF6 
  *            @arg CAN_FLAG_BUF7 
  *            @arg CAN_FLAG_BUF8 
  *            @arg CAN_FLAG_BUF9 
  *            @arg CAN_FLAG_BUF10
  *            @arg CAN_FLAG_BUF11
  *            @arg CAN_FLAG_BUF12
  *            @arg CAN_FLAG_BUF13
  *            @arg CAN_FLAG_BUF14
  *            @arg CAN_FLAG_ERR
  * @retval None
  */
void CAN_ClearFlag(CAN_TypeDef* CANx, uint32_t CAN_Bufferx)
{
    /* Check the parameters */
    assert_param(IS_CAN_ALL_PERIPH(CANx));
//    assert_param(IS_CAN_GET_FLAG(CAN_Flag)); //always true

    CANx->CICLR.V32 = (1UL<<CAN_Bufferx);
}

/**
  * @brief  Turn on or turn off CAN module
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @param  OnOffState: state of the FIFOx peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void CAN_OnOff(CAN_TypeDef* CANx, SH_CmdState OnOffState)
{
    if(OnOffState == SH_ON)
        CANx->CGCR.BIT.CANEN = 1;
    else
        CANx->CGCR.BIT.CANEN = 0;
}

/**
  * @brief  reset CAN module
  * @param  CANx: where x can be 0 to select the CAN peripheral.
  * @retval none
  *
*/
void CAN_Reset(CAN_TypeDef* CANx)
{
    uint32_t* pv;
    int i;    
    RCC->RCCLOCK = 0x33CC;
    if(CANx == CAN0)
        RCC->APB0RSTR.BIT.CAN0RST = 1;
    RCC->RCCLOCK = 0;
    
    //clear CAN Registers
    pv = (uint32_t*)(void*)(CANx);
    for(i = 0; i < 0x200/4; i++)
    {
        *pv = 0;
        pv++;
    }
}

/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


