/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_fifo.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_fifo.h"

/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup FIFO_MODULE FIFO
  * @brief FIFO driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup FIFO_Exported_Functions Public Functions
  * @{
  */ 

/**
  * @brief  Reset the FIFO peripheral registers to their default reset values.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @retval None
  */
void FIFO_Reset(FIFO_TypeDef* FIFOx)
{
	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
	
    if (FIFOx == (FIFO_TypeDef*)FIFO0)
        RCC_APB0PeriphReset(RCC_APB0_FIFO0);
    else if (FIFOx == (FIFO_TypeDef*)FIFO1)
        RCC_APB0PeriphReset(RCC_APB0_FIFO1);
    else if (FIFOx == FIFO2)
        RCC_APB0PeriphReset(RCC_APB0_FIFO2);
}

/**
  * @brief  Reset the FIFO TX Module.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @retval None
  */
void FIFO_ResetTX(FIFO_TypeDef* FIFOx)
{
	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
	
    FIFOx->CR.V32 |= (1 << FIFO_CR_TXFRST_Pos);
}

/**
  * @brief  Reset the FIFO RX Module.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @retval None
  */
void FIFO_ResetRX(FIFO_TypeDef* FIFOx)
{
	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
    
    FIFOx->CR.V32 |= (1 << FIFO_CR_RXFRST_Pos);
}

/**
  * @brief  Initializes the FIFO peripheral according to the specified
  *         parameters in the FIFO_InitStruct.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @param  FIFO_InitStruct: pointer to a @ref FIFO_InitTypeDef structure
  *         that contains the configuration information for the FIFO peripheral.
  * @retval None
  */
void FIFO_Init(FIFO_TypeDef* FIFOx, FIFO_InitTypeDef* FIFO_InitStruct)
{

	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
    assert_param(IS_FIFO_TX_TRIGGER(FIFO_InitStruct->FIFO_TX_Trigger)); 
    assert_param(IS_FIFO_RX_TRIGGER(FIFO_InitStruct->FIFO_RX_Trigger));  
    assert_param(IS_FIFO_TX_BITWIDTH(FIFO_InitStruct->FIFO_TX_BitWidth));    
    assert_param(IS_FIFO_RX_BITWIDTH(FIFO_InitStruct->FIFO_RX_BitWidth)); 
    assert_param(IS_FIFO_TX_MAP(FIFO_InitStruct->FIFO_TX_Map)); 
    assert_param(IS_FIFO_RX_MAP(FIFO_InitStruct->FIFO_RX_Map)); 
    assert_param(IS_FIFO_RX_Mode(FIFO_InitStruct->FIFO_RX_Mode)); 
		
		
    FIFOx->CR.V32 &= (~((uint32_t)(FIFO_CR_RXMODE_Msk | FIFO_CR_RXMAP_Msk | FIFO_CR_TXMAP_Msk | FIFO_CR_RXRDBITW_Msk |FIFO_CR_TXWRBITW_Msk | FIFO_CR_RXTRIG_Msk | FIFO_CR_TXTRIG_Msk)));
    FIFOx->CR.V32 |= ((uint32_t) FIFO_InitStruct->FIFO_TX_Trigger
             |(uint32_t) FIFO_InitStruct->FIFO_RX_Trigger
             |(uint32_t) FIFO_InitStruct->FIFO_TX_BitWidth
             |(uint32_t) FIFO_InitStruct->FIFO_RX_BitWidth
             |(uint32_t) FIFO_InitStruct->FIFO_TX_Map
             |(uint32_t) FIFO_InitStruct->FIFO_RX_Map
             |(uint32_t) FIFO_InitStruct->FIFO_RX_Mode
             );

}

/**
  * @brief  Fills each FIFO_InitStruct member with its reset value.
  * @param  FIFO_InitStruct: pointer to a @ref FIFO_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void FIFO_StructInit(FIFO_InitTypeDef* FIFO_InitStruct)
{
    FIFO_InitStruct->FIFO_TX_Trigger = FIFO_TX_Trigger_Half;
    FIFO_InitStruct->FIFO_RX_Trigger = FIFO_RX_Trigger_Half;
    FIFO_InitStruct->FIFO_TX_BitWidth = FIFO_TX_BitWidth_8;
    FIFO_InitStruct->FIFO_RX_BitWidth = FIFO_RX_BitWidth_8;
    FIFO_InitStruct->FIFO_TX_Map = FIFO_TX_MAP_UART1;
    FIFO_InitStruct->FIFO_RX_Map = FIFO_RX_MAP_UART1;
    FIFO_InitStruct->FIFO_RX_Mode = FIFO_RX_Mode_8bits;
}

/**
  * @brief  On or off the specified FIFO peripheral.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @param  FIFO_Mode: TX mode or RX mode.
  * @param  OnOffState: state of the FIFOx peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void FIFO_OnOff(FIFO_TypeDef* FIFOx, uint32_t FIFO_Mode, SH_CmdState OnOffState)
{
	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));

    if (OnOffState != SH_OFF)
    {
        /* Enable the sources */
        if ((FIFO_Mode & FIFO_TX_EN) == FIFO_TX_EN)
        {
            FIFOx->CR.V32 |= FIFO_TX_EN;
        }
        if ((FIFO_Mode & FIFO_RX_EN) == FIFO_RX_EN)
        {
            FIFOx->CR.V32 |= FIFO_RX_EN;
        }					
    }
    else
    {
        /* Disable the sources */
        if ((FIFO_Mode & FIFO_TX_EN) == FIFO_TX_EN)
        {
            FIFOx->CR.V32 &= ~(FIFO_CR_TXFEN_Msk);
        }
        if ((FIFO_Mode & FIFO_RX_EN) == FIFO_RX_EN)
        {
            FIFOx->CR.V32 &= ~(FIFO_CR_RXFEN_Msk);
        }	
    }

}



/**
  * @brief  Enable or Disable specified FIFO interrupts.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @param  FIFO_INT: specifie the FIFO interrupts sources to be enabled or disabled.
  *          This parameter can be any combination of following values:
  *            @arg      FIFO_RXOVFIE_EN    
  *            @arg      FIFO_TXTRIGIE_EN   
  *            @arg      FIFO_RXTRIGIE_EN 
  * @param  NewState: new state of interrupt sources.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void FIFO_INTConfig(FIFO_TypeDef* FIFOx, uint32_t FIFO_INT, SH_FunctionalState NewState)
{
	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
    assert_param(IS_FIFO_INT(FIFO_INT));

    if (NewState != SH_DISABLE)
    {
        /* Enable the interrupt sources */
        if ((FIFO_INT & FIFO_RXTRIGIE_EN) == FIFO_RXTRIGIE_EN)
        {
            FIFOx->CR.V32 |= FIFO_RXTRIGIE_EN;
        }
        if ((FIFO_INT & FIFO_TXTRIGIE_EN) == FIFO_TXTRIGIE_EN)
        {
            FIFOx->CR.V32 |= FIFO_TXTRIGIE_EN;
        }	
        if ((FIFO_INT & FIFO_RXOVFIE_EN) == FIFO_RXOVFIE_EN)
        {
            FIFOx->CR.V32 |= FIFO_RXOVFIE_EN;
        }						
    }
    else
    {
        /* Disable the interrupt sources */
        if ((FIFO_INT & FIFO_RXTRIGIE_EN) == FIFO_RXTRIGIE_EN)
        {
            FIFOx->CR.V32 &= ~(FIFO_CR_RXTRIGIE_Msk);
        }
        if ((FIFO_INT & FIFO_TXTRIGIE_EN) == FIFO_TXTRIGIE_EN)
        {
            FIFOx->CR.V32 &= ~(FIFO_CR_TXTRIGIE_Msk);
        }	
				if ((FIFO_INT & FIFO_RXOVFIE_EN) == FIFO_RXOVFIE_EN)
        {
            FIFOx->CR.V32 &= ~(FIFO_CR_RXOVFIE_Msk);
        }		
    }
}


/**
  * @brief  Checks whether the specified FIFO flag is set or not.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @param  FIFO_Flag: specifies the FIFO flag to check.
  *          This parameter can be one of the following values:
  *            @arg     FIFO_TXFULL_FLAG
  *            @arg     FIFO_RXFULL_FLAG
  *            @arg     FIFO_TXEMPT_FLAG
  *            @arg     FIFO_RXEMPT_FLAG
  *            @arg     FIFO_RXOVF_FLAG
  *            @arg     FIFO_TXTRIGF_FLAG
  *            @arg     FIFO_RXTRIGF_FLAG
  * @retval The new state of FIFO_Flag (SH_SET or SH_RESET).
  */
SH_FlagStatus FIFO_GetFlagStatus(FIFO_TypeDef* FIFOx, uint32_t FIFO_Flag)
{
    SH_FlagStatus bitStatus;

	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
    assert_param(IS_FIFO_GET_ONE_FLAG(FIFO_Flag));

    if ((FIFOx->FR.V32 & FIFO_Flag) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    return bitStatus;
}

/**
  * @brief  Clears the FIFO's pending flags.
  * @param  FIFOx: where x can be 0 or 2 to select the FIFOx peripheral.
  * @param  FIFO_Flag: specifies the FIFO flags to clear.
  *          This parameter can be one of the following values:
  *            @arg     FIFO_RXOVF_FLAG  
  *            @arg     FIFO_TXTRIGF_FLAG
  *            @arg     FIFO_RXTRIGF_FLAG

  * @retval None
  */
void FIFO_ClearFlag(FIFO_TypeDef* FIFOx, uint32_t FIFO_Flag)
{
	/* Check the parameters */
    assert_param(IS_FIFO_ALL_PERIPH(FIFOx));
    assert_param(IS_FIFO_GET_ONE_FLAG(FIFO_Flag));
        
    FIFOx->FR.V32 = (uint32_t)FIFO_Flag << 16;
}



/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


