/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_macp.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provide MACP APIs
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                  How to use this driver
  *          ===================================================================
  *          
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_macp.h"
      
/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/* MACP  Module----------------------------------------------------------*/
/** @defgroup MACP_MODULE  MACP 
*  MACP Module: Include Cordic and SVPWM units
  * @{
  */ 
        
/** @defgroup  MACP_Group_Pub_Funcs  Public Functions
 *  @brief   MACP Public Functions
 *
  * @{
  */
  

/** @defgroup  MACP_Group_Cordic_Funcs  CORDIC Functions
 *  @brief   Cordic Public Functions
 *
  * @{
  */

/**
  *@code Example
       CORDIC_DataTypeDef CordicData;
       CORDIC_InitTypeDef InitCfg;
       CORDIC_StructInit(&InitCfg);
       CORDIC_Init(CORDIC0, &InitCfg);
       CordicData.X = IQ24(0.4);
       CordicData.Y = IQ24(-0.4);
       CordicData.Z = IQ24(0.3);       
       if(CORDIC_transform(CORDIC0,&CordicData) == SH_TRUE)
       {
            printf("X=%f  Y=%f  Z=%f\n",IQnToFloat(CordicData.X,24),(double)IQnToFloat(CordicData.Y,24),(double)IQnToFloat(CordicData.Z,24));
       }       
       CORDIC_Init(CORDIC1, &CORDIC_CFG);
       CordicData.X = IQ24(0.4);
       CordicData.Y = IQ24(-0.4);
       CordicData.Z = IQ24(0.3);       
       if(CORDIC_transform(CORDIC1,&CordicData) == SH_TRUE)
       {
            printf("X=%f  Y=%f  Z=%f\n",(double)IQnToFloat(CordicData.X,24),(double)IQnToFloat(CordicData.Y,24),IQnToFloat(CordicData.Z,24));
       }       
  *@endcode     
  *@brief   CORDIC Module Init
  *@param CORDICx  CORDIC device selection
  *     @arg @b CORDIC0 
  *     @arg @b CORDIC1 
  *@param InitCfg  Input CORDIC init values
  *@li BIT.KADJ
  *     @arg @b CORDIC_KADJ_NONE
  *     @arg @b CORDIC_KADJ_DIVK
  *@li BIT.XYMRS
  *     @arg @b CORDIC_XYMRS_NONE
  *     @arg @b CORDIC_XYMRS_DIV2
  *@li BIT.FORMAT
  *     @arg @b CORDIC_FMT_IQ26
  *     @arg @b CORDIC_FMT_IQ24
  *     @arg @b CORDIC_FMT_IQ15
  *@li BIT.MODE
  *     @arg @b CORDIC_MODE_ROTATE
  *     @arg @b CORDIC_MODE_VECTOR  
  *@retval  None
  */
void CORDIC_Init(CORDIC_TypeDef* CORDICx, const CORDIC_InitTypeDef* InitCfg)
{
    assert_param(IS_CORDIC_MODULE(CORDICx));
    assert_param(IS_CORDIC_KADJ_TYPE(InitCfg->KAdjust));
    assert_param(IS_CORDIC_XYMRS_TYPE(InitCfg->XYAmplitude));
    assert_param(IS_CORDIC_FMT_TYPE(InitCfg->Format));
    assert_param(IS_CORDIC_MODE_TYPE(InitCfg->TransformMode));
    
    /* Initialize CORDIC Module*/
    CORDICx->CSR.V32 = *((const uint32_t*)(const void*)InitCfg);
}

/**
  * @brief  Fills each InitStruct member with its default value.
  * @param  InitStruct : pointer to a CORDIC_InitTypeDef structure which will
  *   be initialized.
  * @retval None
  */
void CORDIC_StructInit(CORDIC_InitTypeDef* InitStruct)
{
    InitStruct->RunCtrl = SH_DISABLE;
	InitStruct->KAdjust = CORDIC_KADJ_NONE;				// no K factor adjust
	InitStruct->XYAmplitude = CORDIC_XYMRS_NONE;		// no amplitude adjust
	InitStruct->Format = CORDIC_FMT_IQ24;			// default: IQ24
	InitStruct->TransformMode = CORDIC_MODE_ROTATE;	// default: Rotate Mode
	InitStruct->IteTimes = 25;			// N+1,q24
	InitStruct->IteAlgorithm = CORDIC_ARITH_CIRCULAR;		// default:Circular Algorithm
}

/**@brief  Deinitializes the CORDICx registers to their default reset
  *@param CORDICx  CORDIC device selection
  *     @arg @b CORDIC0 
  *     @arg @b CORDIC1 
  *@retval None
  */
void CORDIC_Reset(CORDIC_TypeDef* CORDICx)
{
    assert_param(IS_CORDIC_MODULE(CORDICx));
    
    /* Initialize CORDIC Module*/
    CORDICx->CSR.V32 = 0;
    CORDICx->OPRDX = 0;
    CORDICx->OPRDY = 0;
    CORDICx->OPRDZ = 0;
}


/**
  *@brief   CORDIC transform
  *@details
  *\n        === Rotate Mode ===
  *\n        X' = K[Xcos(Z) - Ysin(Z)]  
  *\n        Y' = K[Xsin(Z) + Ycos(Z)]
  *\n        Z' = 0
  *\n        K = 1.6467602579
  *\n        === Vector Mode ===
  *\n        X' = K * sqrt(X^2 + Y^2)
  *\n        Y' = 0
  *\n        Z' = Z + arctan(Y/X)
  *\n        K = 1.6467602579  
  *@param CORDICx  CORDIC device selection
  *     @arg @b CORDIC0 
  *     @arg @b CORDIC1 
  *@param CordicData  input data to transform then get result from it
  *@retval  SH_Bool transform result is legal or not
  *    @arg @b SH_TRUE  result is right
  *    @arg @b SH_FALSE operation overflow
  */
SH_Bool CORDIC_Transform(CORDIC_TypeDef* CORDICx, CORDIC_DataTypeDef* CordicData)
{
    assert_param(IS_CORDIC_MODULE(CORDICx));
    
    /* set CORDIC parameters */
    CORDICx->OPRDX = (uint32_t)CordicData->X;
    CORDICx->OPRDY = (uint32_t)CordicData->Y;
    CORDICx->OPRDZ = (uint32_t)CordicData->Z;
    
    /* start transform */
    CORDIC_START(CORDICx);    
    
    /* wait transform done */
    while(CORDIC_IS_BUSY(CORDICx));
    
    /* get result */
    CordicData->X = (int32_t)CORDICx->OPRDX;
    CordicData->Y = (int32_t)CORDICx->OPRDY;
    CordicData->Z = (int32_t)CORDICx->OPRDZ;    
    
    /* return transform error flag */
    return (CORDICx->CSR.BIT.OVF ? SH_FALSE : SH_TRUE);
}


/**
  * @code   Example
       CORDIC_DataTypeDef CordicData;
       CORDIC_InitTypeDef InitCfg;
       CORDIC_StructInit(&InitCfg);       
       CORDIC_Init(CORDIC0, &InitCfg);
       CordicData.X = IQ24(0.4);
       CordicData.Y = IQ24(-0.4);
       CordicData.Z = IQ24(0.3);       
       CORDIC_Run(CORDIC0,&CordicData);
       
       while(CORDIC_IS_BUSY(CORDIC0));  
        
       if(CORDIC_GetResult(CORDIC0,&CordicData))
       {
            printf("X=%f  Y=%f  Z=%f\n",IQnToFloat(CordicData.X,24),IQnToFloat(CordicData.Y,24),IQnToFloat(CordicData.Z,24));
       }

  * @endcode 
  *@brief  start CORDIC transform then return immediately
  *@param CORDICx  CORDIC device selection
  *     @arg @b CORDIC0 
  *     @arg @b CORDIC1 
  *@param CordicData  input data to transform then get result from it
  *@retval   None
  */ 
void CORDIC_Run(CORDIC_TypeDef* CORDICx,CORDIC_DataTypeDef* CordicData)
{
    assert_param(IS_CORDIC_MODULE(CORDICx));
    
    /* set CORDIC parameters */
    CORDICx->OPRDX = (uint32_t)CordicData->X;
    CORDICx->OPRDY = (uint32_t)CordicData->Y;
    CORDICx->OPRDZ = (uint32_t)CordicData->Z;
    
    /* start transform */
    CORDIC_START(CORDICx);    
}

/**
  *@brief  get CORDIC result
  *@param CORDICx  CORDIC device selection
  *     @arg @b CORDIC0 
  *     @arg @b CORDIC1 
  *@param CordicData  output the transform result
  *@retval  SH_Bool transform result is legal or not
  *    @arg @b SH_TRUE  result is right
  *    @arg @b SH_FALSE operation overflow
  */
SH_Bool CORDIC_GetResult(CORDIC_TypeDef* CORDICx, CORDIC_DataTypeDef* CordicData)
{
    assert_param(IS_CORDIC_MODULE(CORDICx));
        
    /* get transform result */
    CordicData->X = (int32_t)CORDICx->OPRDX;
    CordicData->Y = (int32_t)CORDICx->OPRDY;
    CordicData->Z = (int32_t)CORDICx->OPRDZ;    

    /* return transform error flag */
    return (CORDICx->CSR.BIT.OVF ? SH_FALSE : SH_TRUE);
}
   

/**
  * @}
  */


/** @defgroup  MACP_Group_SVPWM_Funcs  SVPWM Functions
 *  @brief   SVPWM Public Functions
 *
  * @{
  */

/**
  * @code   Example
       SVPWM_DataTypeDef SVPWMData;
       SVPWMData.UAlpha = IQ24(0.3);
       SVPWMData.UBeta  = IQ24(0.2);
       SVPWMData.SVType = SVPWM_SVTYPE_7;
       SVPWMData.SVIQN  = 24;
       SVPWM_Transform(&SVPWMData);
       printf("Ta:%X Tb:%X Tc:%X Sector:%X\n",SVPWMData.Ta,SVPWMData.Tb,SVPWMData.Tc,SVPWMData.Sector);
  *@endcode 
  *@brief Initial SVPWM Module
  *@param SVPWMData  Input SVPWM Alpha and Beta value then get result from Ta,Tb,Tc,Sector.
  *@retval None
  */
void SVPWM_Transform(SVPWM_DataTypeDef* SVPWMData)
{
    assert_param(IS_SVPWM_SVTYPE_TYPE(SVPWMData->SVType));
    assert_param(SVPWMData->SVIQN < 32);
    assert_param(IS_SVPWM_SATEN(SVPWMData->SATEN));
    
    /* set parameters */
    MACP->SVIQN.V32 = 0;  //clear config
    MACP->SVUALPHA = SVPWMData->UAlpha;
    MACP->SVUBETA  = SVPWMData->UBeta;
    MACP->SVIQN.V32 = (uint32_t)(SVPWMData->SVType | SVPWMData->SVIQN | SVPWMData->SATEN);
    /* start transform */
    SVPWM_START();
    
    /* wait transform done */
    while(SVPWM_IS_BUSY());
    
    /* get ransform result */
    SVPWMData->Ta = MACP->SVTA;
    SVPWMData->Tb = MACP->SVTB;
    SVPWMData->Tc = MACP->SVTC;
    SVPWMData->Sector = (uint8_t)MACP->SVSECTOR;    
    SVPWMData->FlagOverAL = (uint8_t)(MACP->SVIQN.BIT.OVERAL); 
    SVPWMData->FlagOverAH = (uint8_t)(MACP->SVIQN.BIT.OVERAH);  
    SVPWMData->FlagOverBL = (uint8_t)(MACP->SVIQN.BIT.OVERBL);  
    SVPWMData->FlagOverBH = (uint8_t)(MACP->SVIQN.BIT.OVERBH);  
    SVPWMData->FlagOverCL = (uint8_t)(MACP->SVIQN.BIT.OVERCL);  
    SVPWMData->FlagOverCH = (uint8_t)(MACP->SVIQN.BIT.OVERCH);   
}


/**
  * @}
  */


/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


