/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_mcm.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_mcm.h"
#include "sh3xf2xx_sb0_rcc.h"

/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup MCM_MODULE MCM 
  * @brief MCM driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup MCM_Exported_Functions Public Functions
  * @{
  */ 

/** @defgroup MCM_Group1 Initialization and Configuration
 *  @brief    Initialization and Configuration
 *
@verbatim    
 ===============================================================================
                      Initialization and Configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */


/**
  * @brief  Reset the MCM peripheral registers to their default reset values.
  * @retval None
  */
void MCM_Reset(void)
{
    RCC_AHBPeriphReset(RCC_AHB_MCM);
}

/**
  * @brief  On or off the specified MCM peripheral.
  * @param  OnOffState: state of the MCM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void MCM_OnOff(SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));

    MCM_REG_UNLOCK;

    if (OnOffState != SH_OFF)
    {
        MCM->PWMOE |= MCM_PWMOE_PWMOE_Msk;
    }
    else
    {
        MCM->PWMOE &= ~MCM_PWMOE_PWMOE_Msk;
    } 

    MCM_REG_LOCK;
}

/**
  * @brief  Fills each MCM_TimeBaseInitStruct member with its default value.
  * @param  InitStruct : pointer to a @ref MCM_TimeBaseInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void MCM_TimeBaseStructInit(MCM_TimeBaseInitTypeDef* InitStruct)
{
    InitStruct->CounterMode = MCM_CounterMode_Center;
    InitStruct->PWMOutMode = MCM_PWMOutMode_Complementary;
    InitStruct->PWMSymmetry = MCM_PWM_Symmetry;
    InitStruct->DutyActivePoint = MCM_DutyActivePoint_Zero;
    InitStruct->AdcCmpActivePoint = MCM_ADC_CMP_Active_Zero;
    InitStruct->PosTPS = MCM_INT_EVENT_FLAG_DIV_1;
    InitStruct->Prescaler = 0x00;
    InitStruct->Period = 4200;
}

/**
  * @brief  Initializes the MCM Time Base Unit peripheral according to 
  *         the specified parameters in the MCM_TimeBaseInitStruct.
  * @param  InitCfg: pointer to a @ref MCM_TimeBaseInitTypeDef structure
  *         that contains the configuration information for the specified MCM peripheral.
  * @retval None
  */
void MCM_TimeBaseInit(MCM_TimeBaseInitTypeDef* InitCfg)
{
    /* Check the parameters */
    assert_param(IS_MCM_COUNTER_MODE(InitCfg->CounterMode));
    assert_param(IS_MCM_PWM_OUT_MODE(InitCfg->PWMOutMode));
    assert_param(IS_MCM_PWM_SYMMETRY(InitCfg->PWMSymmetry));
    assert_param(IS_MCM_DUTY_ACTIVE_POINT(InitCfg->DutyActivePoint));
    assert_param(IS_MCM_ADC_CMP_ACTIVE(InitCfg->AdcCmpActivePoint)); 
    assert_param(IS_MCM_INT_EVENT_FLAG_DIV(InitCfg->PosTPS));

    MCM_REG_UNLOCK;
    /* Reset the PWMCON1 bits */
    MCM->PWMCON1.V32 &= ~(MCM_PWMCON1_PTMOD_Msk | MCM_PWMCON1_PWMSYM_Msk  \
                         | MCM_PWMCON1_POUTMOD_Msk | MCM_PWMCON1_POSTPS_Msk); 
    MCM->PWMCON1.V32 |= (InitCfg->CounterMode | InitCfg->PWMSymmetry   \
                        | InitCfg->PosTPS | InitCfg->PWMOutMode);
    
    
    /* Reset the PWMCON2 bits */
    MCM->PWMCON2.V32 &= ~(MCM_PWMCON2_DILDEN_Msk | MCM_PWMCON2_CILDEN_Msk  \
                         | MCM_PWMCON2_ZDLDEN_Msk | MCM_PWMCON2_PDLDEN_Msk \
                         | MCM_PWMCON2_ZCMLDEN_Msk | MCM_PWMCON2_PCMLDEN_Msk); 
    MCM->PWMCON2.V32 |= (InitCfg->DutyActivePoint | InitCfg->AdcCmpActivePoint);

    /* Set MCM Prescaler and Period register value */
    MCM->PWMPSQ = InitCfg->Prescaler;
    MCM->PWMP = InitCfg->Period;

    MCM_REG_LOCK;
}

/**
  * @brief  Fills each MCM_PWMxPWMx1StructInit member with its default value.
  * @param  InitStruct : pointer to a @ref MCM_PWMxPWMx1InitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void MCM_PWMxPWMx1StructInit(MCM_PWMxPWMx1InitTypeDef* InitStruct)
{
    InitStruct->DutyArea = MCM_DutyArea_1;
    InitStruct->PWMxDutyPolarity = MCM_DutyPolarity_High;
    InitStruct->PWMx1DutyPolarity = MCM_DutyPolarity_Low;
    InitStruct->PWMxDutyValue = 840;
    InitStruct->PWMx1DutyValue = 840;
    InitStruct->DTx0Value = 84;
    InitStruct->DTx1Value = 84;
}


/**
  * @brief  Initialize the MCM PWM0&PMW01 function according to 
  *         the specified parameters in the MCM_PWMxPWMx1InitTypeDef InitCfg.
  * @param  InitCfg: pointer to a @ref MCM_PWMxPWMx1InitTypeDef structure
  *         that contains the configuration information for the PWM0&PWM01 function.
  * @retval None
  */
void MCM_PWM0PWM01Init(MCM_PWMxPWMx1InitTypeDef* InitCfg)
{
    uint32_t temp = 0;

    assert_param(IS_MCM_DUTY_AREA(InitCfg->DutyArea));
    assert_param(IS_MCM_DUTY_POLARITY(InitCfg->PWMxDutyPolarity));
    assert_param(IS_MCM_DUTY_POLARITY(InitCfg->PWMx1DutyPolarity));

    MCM_REG_UNLOCK;

    temp = MCM->PWMCON1.V32;
    
    /* Reset PDCON0/PWM0S/PWM01S bits */
    temp &= ~(MCM_PWMCON1_PDCON0_Msk | MCM_PWMCON1_PWM0S_Msk | MCM_PWMCON1_PWM01S_Msk);

    temp |= (uint32_t)InitCfg->DutyArea;
    temp |= (uint32_t)(InitCfg->PWMxDutyPolarity & (uint16_t)0x0001);
    temp |= (uint32_t)((InitCfg->PWMx1DutyPolarity & (uint16_t)0x0001) << 3);

    MCM->PWMCON1.V32 = temp;

    MCM->PWM0D = InitCfg->PWMxDutyValue;
    MCM->PWM01D = InitCfg->PWMx1DutyValue;
    
    MCM->PWMDT00 = InitCfg->DTx0Value;
    MCM->PWMDT01 = InitCfg->DTx1Value;

    MCM_REG_LOCK;
}

/**
  * @brief  Initialize the MCM PWM1&PMW11 function according to 
  *         the specified parameters in the MCM_PWMxPWMx1InitTypeDef InitCfg.
  * @param  InitCfg: pointer to a @ref MCM_PWMxPWMx1InitTypeDef structure
  *         that contains the configuration information for the PWM1&PWM11 function.
  * @retval None
  */
void MCM_PWM1PWM11Init(MCM_PWMxPWMx1InitTypeDef* InitCfg)
{
    uint32_t temp = 0;

    assert_param(IS_MCM_DUTY_AREA(InitCfg->DutyArea));
    assert_param(IS_MCM_DUTY_POLARITY(InitCfg->PWMxDutyPolarity));
    assert_param(IS_MCM_DUTY_POLARITY(InitCfg->PWMx1DutyPolarity));

    MCM_REG_UNLOCK;

    temp = MCM->PWMCON1.V32;
    
    /* Reset PDCON0/PWM0S/PWM01S bits */
    temp &= ~(MCM_PWMCON1_PDCON1_Msk | MCM_PWMCON1_PWM1S_Msk | MCM_PWMCON1_PWM11S_Msk);

    temp |= (uint32_t)(InitCfg->DutyArea << 1);
    temp |= (uint32_t)((InitCfg->PWMxDutyPolarity & (uint16_t)0x0001) << 1);
    temp |= (uint32_t)((InitCfg->PWMx1DutyPolarity & (uint16_t)0x0001) << 4);

    MCM->PWMCON1.V32 = temp;

    MCM->PWM1D = InitCfg->PWMxDutyValue;
    MCM->PWM11D = InitCfg->PWMx1DutyValue;
    
    MCM->PWMDT10 = InitCfg->DTx0Value;
    MCM->PWMDT11 = InitCfg->DTx1Value;

    MCM_REG_LOCK;
}

/**
  * @brief  Initialize the MCM PWM2&PMW21 function according to 
  *         the specified parameters in the MCM_PWMxPWMx1InitTypeDef InitCfg.
  * @param  InitCfg: pointer to a @ref MCM_PWMxPWMx1InitTypeDef structure
  *         that contains the configuration information for the PWM2&PWM21 function.
  * @retval None
  */
void MCM_PWM2PWM21Init(MCM_PWMxPWMx1InitTypeDef* InitCfg)
{
    uint32_t temp = 0;

    assert_param(IS_MCM_DUTY_AREA(InitCfg->DutyArea));
    assert_param(IS_MCM_DUTY_POLARITY(InitCfg->PWMxDutyPolarity));
    assert_param(IS_MCM_DUTY_POLARITY(InitCfg->PWMx1DutyPolarity));

    MCM_REG_UNLOCK;

    temp = MCM->PWMCON1.V32;
    
    /* Reset PDCON0/PWM0S/PWM01S bits */
    temp &= ~(MCM_PWMCON1_PDCON2_Msk | MCM_PWMCON1_PWM2S_Msk | MCM_PWMCON1_PWM21S_Msk);

    temp |= (uint32_t)(InitCfg->DutyArea << 2);
    temp |= (uint32_t)((InitCfg->PWMxDutyPolarity & (uint16_t)0x0001) << 2);
    temp |= (uint32_t)((InitCfg->PWMx1DutyPolarity & (uint16_t)0x0001) << 5);

    MCM->PWMCON1.V32 = temp;

    MCM->PWM2D = InitCfg->PWMxDutyValue;
    MCM->PWM21D = InitCfg->PWMx1DutyValue;
    
    MCM->PWMDT20 = InitCfg->DTx0Value;
    MCM->PWMDT21 = InitCfg->DTx1Value;

    MCM_REG_LOCK;
}

/**
  * @}
  */
  

/** @defgroup MCM_Group2 Timer Base
 *  @brief    Timer Base
 *  @verbatim   
 ===============================================================================
                               Timer Base
 ===============================================================================

  * @endverbatim
  * @{
  */
  
/**
  * @brief enable write or reload the MCM's registers.
  * @param  NewState: new state of the MCM's registers write or reload.
  *          This parameter can be one of the following values:
  *            @arg MCM_REG_WRITE_EN
  *            @arg MCM_REG_RELOAD_EN
  * @retval None
  */
void MCM_RegWritReloadLock(MCM_REG_LOCK_RELOAD_State NewState)
{
    /* Check the parameters */
    assert_param(IS_MCM_REG_STATE(NewState));

    if (NewState != MCM_REG_WRITE_EN)
    {
        /* Enable MCM registers protection */
        MCM->PWMRLDEN = 0xAA;
    }
    else
    {
        /* Disable MCM registers protection */
        MCM->PWMRLDEN = 0x55;
    }
}  


/**
  * @brief  Sets the MCM Counter Register value
  * @param  Counter: specifies the Counter register new value.
  * @retval None
  */
void MCM_SetCounter(uint16_t Counter)
{
    MCM_REG_UNLOCK;

    MCM->PWMC = Counter;    

    MCM_REG_LOCK;
}

/**
  * @brief  Sets the MCM Prescaler Register value
  * @param  Prescaler: specifies the Prescaler register new value.
  * @retval None
  */
void MCM_SetPrescaler(uint16_t Prescaler)
{
    MCM_REG_UNLOCK;

    MCM->PWMPSQ = Prescaler;    

    MCM_REG_LOCK;
}

/**
  * @brief  Sets the MCM Period Register value
  * @param  Period: specifies the Period register new value.
  * @retval None
  */
void MCM_SetPeriod(uint16_t Period)
{
    MCM_REG_UNLOCK;

    MCM->PWMP = Period;
    
    MCM_REG_LOCK;
}

/**
  * @brief  Get the MCM Counter Register value
  * @retval The Counter Register value.
  */
uint16_t MCM_GetCounter(void)
{
    return (uint16_t)MCM->PWMC;    
}

/**
  * @brief  Get the MCM Prescaler Register value
  * @retval The Prescaler Register value.
  */
uint16_t MCM_GetPrescaler(void)
{
    return (uint16_t)MCM->PWMPSQ;    
}

/**
  * @brief  Get the MCM Period Register value
  * @retval The Period Register value.
  */
uint16_t MCM_GetPeriod(void)
{
    return (uint16_t)MCM->PWMP;    
}

/**
  * @brief  Set the MCM PWM0 Duty Register value
  * @param  Duty0: specifies the PWM0 Duty Register new value.
  * @retval None
  */
void MCM_SetPWMDuty0(uint16_t Duty0)
{
    MCM_REG_UNLOCK;

    MCM->PWM0D = Duty0;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM01 Duty Register value
  * @param  Duty01: specifies the PWM01 Duty Register new value.
  * @retval None
  */
void MCM_SetPWMDuty01(uint16_t Duty01)
{
    MCM_REG_UNLOCK;

    MCM->PWM01D = Duty01;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM1 Duty Register value
  * @param  Duty1: specifies the PWM1 Duty Register new value.
  * @retval None
  */
void MCM_SetPWMDuty1(uint16_t Duty1)
{
    MCM_REG_UNLOCK;

    MCM->PWM1D = Duty1;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM11 Duty Register value
  * @param  Duty11: specifies the PWM11 Duty Register new value.
  * @retval None
  */
void MCM_SetPWMDuty11(uint16_t Duty11)
{
    MCM_REG_UNLOCK;

    MCM->PWM11D = Duty11;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM2 Duty Register value
  * @param  Duty2: specifies the PWM2 Duty Register new value.
  * @retval None
  */
void MCM_SetPWMDuty2(uint16_t Duty2)
{
    MCM_REG_UNLOCK;

    MCM->PWM2D = Duty2;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM01 Duty Register value
  * @param  Duty21: specifies the PWM21 Duty Register new value.
  * @retval None
  */
void MCM_SetPWMDuty21(uint16_t Duty21)
{
    MCM_REG_UNLOCK;

    MCM->PWM21D = Duty21;

    MCM_REG_LOCK;
}

/**
  * @brief  Get the MCM PWM0 Duty Register value.
  * @retval The PWM0 Duty Register value.
  */
uint16_t MCM_GetPWMDuty0(void)
{
    return (uint16_t)MCM->PWM0D;
}

/**
  * @brief  Set the MCM PWM01 Duty Register value.
  * @retval The PWM01 Duty Register value.
  */
uint16_t MCM_GetPWMDuty01(void)
{
    return (uint16_t)MCM->PWM01D;
}

/**
  * @brief  Get the MCM PWM1 Duty Register value.
  * @retval The PWM1 Duty Register value.
  */
uint16_t MCM_GetPWMDuty1(void)
{
    return (uint16_t)MCM->PWM1D;
}

/**
  * @brief  Set the MCM PWM11 Duty Register value.
  * @retval The PWM11 Duty Register value.
  */
uint16_t MCM_GetPWMDuty11(void)
{
    return (uint16_t)MCM->PWM11D;
}

/**
  * @brief  Get the MCM PWM2 Duty Register value.
  * @retval The PWM2 Duty Register value.
  */
uint16_t MCM_GetPWMDuty2(void)
{
    return (uint16_t)MCM->PWM2D;
}

/**
  * @brief  Set the MCM PWM21 Duty Register value.
  * @retval The PWM21 Duty Register value.
  */
uint16_t MCM_GetPWMDuty21(void)
{
    return (uint16_t)MCM->PWM21D;
}

/**
  * @brief  configure interrupt or event flag division parameter.
  * @param  MCM_Division: specifies the value of division parameter.
  *          This parameter can be one of the following values:
  *            @arg MCM_INT_EVENT_FLAG_DIV_1
  *            @arg MCM_INT_EVENT_FLAG_DIV_2
  *            @arg MCM_INT_EVENT_FLAG_DIV_3
  *            @arg MCM_INT_EVENT_FLAG_DIV_4
  *            @arg MCM_INT_EVENT_FLAG_DIV_5
  *            @arg MCM_INT_EVENT_FLAG_DIV_6
  *            @arg MCM_INT_EVENT_FLAG_DIV_7
  *            @arg MCM_INT_EVENT_FLAG_DIV_8  
  * @retval None
  */
void MCM_IntEventFlagDivConfig(uint16_t MCM_Division)
{
    /* Check the parameters */
    assert_param(IS_MCM_INT_EVENT_FLAG_DIV(MCM_Division));

    MCM_REG_UNLOCK;

    MCM->PWMCON1.V32 &= ~MCM_PWMCON1_POSTPS_Msk;
    MCM->PWMCON1.V32 |= MCM_Division;

    MCM_REG_LOCK;
}

/**
  * @brief  Enable or disable the MCM Duty, Deadtime and Event auto reload.
  * @param  MCM_DUTY_Active: specifie the auto reload register to be enabled or disabled.
  *          This parameter can be any combination of the following values:
  *            @arg MCM_DutyActivePoint_Now
  *            @arg MCM_DutyActivePoint_Zero
  *            @arg MCM_DutyActivePoint_Period
  * @param  NewState: new state of synchronous function.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE  
  * @retval None
  */
void MCM_DutyEventActiveConfig(uint16_t MCM_DUTY_Active, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_MCM_DUTY_ACTIVE_POINT(MCM_DUTY_Active));
    assert_param(IS_FUNCTION_STATE(NewState));

    MCM_REG_UNLOCK;

    if (NewState != SH_DISABLE)
    {
        MCM->PWMCON2.V32 |= MCM_DUTY_Active;
    }
    else
    {
        MCM->PWMCON2.V32 &= ~MCM_DUTY_Active;
    }

    MCM_REG_LOCK;
}


/**
  * @brief  Set the MCM PWM0 module dead time register value.  
  * @param  PWM0DT: specifie the PWM0&PWM01 rising dead time register value.
  * @param  PWM01DT: specifie the  PWM0&PWM01 falling dead time register value.
  * @retval None
  */
void MCM_PWM0DeadTimeConfig(uint16_t PWM0DT,uint16_t PWM01DT)
{
    MCM_REG_UNLOCK;

    MCM->PWMDT00 = PWM0DT;
    MCM->PWMDT01 = PWM01DT;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM1 module dead time register value.  
  * @param  PWM1DT: specifie the PWM1 port dead time register value.
  * @param  PWM11DT: specifie the PWM11 port dead time register value.
  * @retval None
  */
void MCM_PWM1DeadTimeConfig(uint16_t PWM1DT,uint16_t PWM11DT)
{
    MCM_REG_UNLOCK;

    MCM->PWMDT10 = PWM1DT;
    MCM->PWMDT11 = PWM11DT;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM PWM2 module dead time register value.  
  * @param  PWM2DT: specifie the PWM2 port dead time register value.
  * @param  PWM21DT: specifie the PWM21 port dead time register value.
  * @retval None
  */
void MCM_PWM2DeadTimeConfig(uint16_t PWM2DT,uint16_t PWM21DT)
{
    MCM_REG_UNLOCK;

    MCM->PWMDT20 = PWM2DT;
    MCM->PWMDT21 = PWM21DT;

    MCM_REG_LOCK;
}


/**
  * @}
  */
  
/** @defgroup MCM_Group3 MCM Interrupt
 *  @brief    MCM Interrupt
 *
@verbatim   
 ===============================================================================
                               MCM Interrupt
 ===============================================================================

@endverbatim
  * @{
  */

/**
  * @brief  Enable or Disable specified MCM interrupts.
  * @param  MCM_INT: specifie the MCM interrupts sources to be enabled or disabled.
  *          This parameter can be any combination of following values:
  *            @arg MCM_INT_PWM0_UP
  *            @arg MCM_INT_PWM0_DOWN
  *            @arg MCM_INT_PWM1_UP
  *            @arg MCM_INT_PWM1_DOWN
  *            @arg MCM_INT_PWM2_UP
  *            @arg MCM_INT_PWM2_DOWN
  *            @arg MCM_INT_ZM
  *            @arg MCM_INT_PM
  *            @arg MCM_INT_FLT
  *            @arg MCM_INT_FLTCMP0
  *            @arg MCM_INT_FLTCMP1
  *            @arg MCM_INT_OUT
  *            @arg MCM_INT_OSC 
  * @param  NewState: new state of interrupt sources.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void MCM_INTConfig(uint16_t MCM_INT, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_MCM_INT(MCM_INT));
    assert_param(IS_FUNCTION_STATE(NewState));  

    MCM_REG_UNLOCK;

    if (NewState != SH_DISABLE)
    {
        /* Enable the interrupt sources */
        MCM->PWMINTEN.V32 |= MCM_INT;
    }
    else
    {
        /* Disable the interrupt sources */
        MCM->PWMINTEN.V32 &= ~MCM_INT;
    }

    MCM_REG_LOCK;
}

/**
  * @brief  Check whether the specified MCM flag is set or not
  * @param  MCM_Flag: specifie the flag to check.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLAG_PWM0_UP
  *            @arg MCM_FLAG_PWM0_DOWN
  *            @arg MCM_FLAG_PWM1_UP
  *            @arg MCM_FLAG_PWM1_DOWN
  *            @arg MCM_FLAG_PWM2_UP
  *            @arg MCM_FLAG_PWM2_DOWN
  *            @arg MCM_FLAG_ZM
  *            @arg MCM_FLAG_PM
  *            @arg MCM_FLAG_FLT
  *            @arg MCM_FLAG_FLTCMP0
  *            @arg MCM_FLAG_FLTCMP1  
  *            @arg MCM_FLAG_OUT
  *            @arg MCM_FLAG_OSC
  *            @arg MCM_FLAG_SC0STAT
  *            @arg MCM_FLAG_SC1STAT
  *            @arg MCM_FLAG_SC2STAT  
  * @retval None
  */
SH_FlagStatus MCM_GetFlagStatus(uint16_t MCM_Flag)
{
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_MCM_GET_ONE_FLAG(MCM_Flag));

    if ((MCM->PWMINTF.V32 & MCM_Flag) != (uint16_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @brief  Clear the MCM's pending flags.  
  * @param  MCM_Flag: specifie the flag bit to clear.
  *          This parameter can be any combination of the following values:
  *            @arg MCM_FLAG_PWM0_UP
  *            @arg MCM_FLAG_PWM0_DOWN
  *            @arg MCM_FLAG_PWM1_UP
  *            @arg MCM_FLAG_PWM1_DOWN
  *            @arg MCM_FLAG_PWM2_UP
  *            @arg MCM_FLAG_PWM2_DOWN
  *            @arg MCM_FLAG_ZM
  *            @arg MCM_FLAG_PM
  *            @arg MCM_FLAG_FLT
  *            @arg MCM_FLAG_FLTCMP0
  *            @arg MCM_FLAG_FLTCMP1  
  *            @arg MCM_FLAG_OUT
  *            @arg MCM_FLAG_OSC
  *            @arg MCM_FLAG_SC0STAT
  *            @arg MCM_FLAG_SC1STAT
  *            @arg MCM_FLAG_SC2STAT 
  * @retval None
  */
void MCM_ClearFlag(uint16_t MCM_Flag)
{
    /* Check the parameters */
    assert_param(IS_MCM_GET_ONE_FLAG(MCM_Flag));

    MCM->PWMINTF.V32 = (uint32_t)MCM_Flag << 16;
}


/**
  * @}
  */
  
/** @defgroup MCM_Group4 MCM Trigger ADC
 *  @brief    MCM Trigger ADC
 *
@verbatim   
 ===============================================================================
                               MCM Trigger ADC
 ===============================================================================

@endverbatim
  * @{
  */

/**
  * @brief  Enable or disable the MCM ADC compare register auto reload.
  * @param  MCM_CMP_Active: specifie the auto reload register to be enabled or disabled.
  *          This parameter can be any combination of the following values:
  *            @arg MCM_ADC_CMP_Active_Now
  *            @arg MCM_ADC_CMP_Active_Zero
  *            @arg MCM_ADC_CMP_Active_Period
  * @param  NewState: new state of synchronous function.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE  
  * @retval None
  */
void MCM_CompareEventActiveConfig(uint16_t MCM_CMP_Active, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_MCM_ADC_CMP_ACTIVE(MCM_CMP_Active));
    assert_param(IS_FUNCTION_STATE(NewState));

    MCM_REG_UNLOCK;

    if (NewState != SH_DISABLE)
    {
        MCM->PWMCON2.V32 |= MCM_CMP_Active;
    }
    else
    {
        MCM->PWMCON2.V32 &= ~MCM_CMP_Active;
    }

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM Trigger ADC Register 1 value.  
  * @param  Value1: specifie Trigger ADC Register 1 value.
  *          This parameter can be 0 to 0xFFFF.
  * @retval None
  */
void MCM_SetADCTriggerValue1(uint16_t Value1)
{
    MCM_REG_UNLOCK;

    MCM->PWMCMP1 = Value1;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM Trigger ADC Register 2 value.  
  * @param  Value2: specifie Trigger ADC Register 2 value.
  *          This parameter can be 0 to 0xFFFF.  
  * @retval None
  */
void MCM_SetADCTriggerValue2(uint16_t Value2)
{
    MCM_REG_UNLOCK;

    MCM->PWMCMP2 = Value2;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM Trigger ADC Register 3 value.  
  * @param  Value3: specifie Trigger ADC Register 3 value.
  *          This parameter can be 0 to 0xFFFF.  
  * @retval None
  */
void MCM_SetADCTriggerValue3(uint16_t Value3)
{
    MCM_REG_UNLOCK;

    MCM->PWMCMP3 = Value3;

    MCM_REG_LOCK;
}

/**
  * @brief  Set the MCM Trigger ADC Register 4 value.  
  * @param  Value4: specifie Trigger ADC Register 4 value.
  *          This parameter can be 0 to 0xFFFF.  
  * @retval None
  */
void MCM_SetADCTriggerValue4(uint16_t Value4)
{
    MCM_REG_UNLOCK;

    MCM->PWMCMP4 = Value4;

    MCM_REG_LOCK;
}


/**
  * @brief  Configure the MCM ADC trigger 1 sources.  
  * @param  MCM_ADCTrigger1: specifie the ADC trigger 1 source.
  *          This parameter can be one of the following values:
  *            @arg MCM_TRIGGER_CMP1_None
  *            @arg MCM_TRIGGER_CMP1_RisingFalling
  *            @arg MCM_TRIGGER_CMP1_Rising
  *            @arg MCM_TRIGGER_CMP1_Falling  
  * @retval None
  */
void MCM_ADCTrigger1Config(uint16_t MCM_ADCTrigger1)
{
    /* Check the parameters */
    assert_param(IS_MCM_ADC_TRIGGER1_SOURCE(MCM_ADCTrigger1));

    MCM_REG_UNLOCK;

    MCM->PWMCON2.V32 &= ~MCM_PWMCON2_CMP1_Msk;
    MCM->PWMCON2.V32 |= MCM_ADCTrigger1;

    MCM_REG_LOCK;
}

/**
  * @brief  Configure the MCM ADC trigger 2 sources.  
  * @param  MCM_ADCTrigger2: specifie the ADC trigger 2 source.
  *          This parameter can be one of the following values:
  *            @arg MCM_TRIGGER_CMP2_None
  *            @arg MCM_TRIGGER_CMP2_RisingFalling
  *            @arg MCM_TRIGGER_CMP2_Rising
  *            @arg MCM_TRIGGER_CMP2_Falling  
  * @retval None
  */
void MCM_ADCTrigger2Config(uint16_t MCM_ADCTrigger2)
{
    /* Check the parameters */
    assert_param(IS_MCM_ADC_TRIGGER2_SOURCE(MCM_ADCTrigger2));

    MCM_REG_UNLOCK;

    MCM->PWMCON2.V32 &= ~MCM_PWMCON2_CMP2_Msk;
    MCM->PWMCON2.V32 |= MCM_ADCTrigger2;

    MCM_REG_LOCK;
}

/**
  * @brief  Configure the MCM ADC trigger 3 sources.  
  * @param  MCM_ADCTrigger3: specifie the ADC trigger 3 source.
  *          This parameter can be one of the following values:
  *            @arg MCM_TRIGGER_CMP3_None
  *            @arg MCM_TRIGGER_CMP3_RisingFalling
  *            @arg MCM_TRIGGER_CMP3_Rising
  *            @arg MCM_TRIGGER_CMP3_Falling  
  * @retval None
  */
void MCM_ADCTrigger3Config(uint16_t MCM_ADCTrigger3)
{
    /* Check the parameters */
    assert_param(IS_MCM_ADC_TRIGGER3_SOURCE(MCM_ADCTrigger3));

    MCM_REG_UNLOCK;

    MCM->PWMCON2.V32 &= ~MCM_PWMCON2_CMP3_Msk;
    MCM->PWMCON2.V32 |= MCM_ADCTrigger3;

    MCM_REG_LOCK;
}

/**
  * @brief  Configure the MCM ADC trigger 4 sources.  
  * @param  MCM_ADCTrigger4: specifie the ADC trigger 4 source.
  *          This parameter can be one of the following values:
  *            @arg MCM_TRIGGER_CMP4_None
  *            @arg MCM_TRIGGER_CMP4_RisingFalling
  *            @arg MCM_TRIGGER_CMP4_Rising
  *            @arg MCM_TRIGGER_CMP4_Falling  
  * @retval None
  */
void MCM_ADCTrigger4Config(uint16_t MCM_ADCTrigger4)
{
    /* Check the parameters */
    assert_param(IS_MCM_ADC_TRIGGER4_SOURCE(MCM_ADCTrigger4));

    MCM_REG_UNLOCK;

    MCM->PWMCON2.V32 &= ~MCM_PWMCON2_CMP4_Msk;
    MCM->PWMCON2.V32 |= MCM_ADCTrigger4;

    MCM_REG_LOCK;
}

/**
  * @}
  */
  
/** @defgroup MCM_Group5 PWM MANUAL MODE
 *  @brief    PWM MANUAL MODE
 *
@verbatim   
 ===============================================================================
                               PWM MANUAL MODE
 ===============================================================================

@endverbatim
  * @{
  */

/**
  * @brief  Configure the MCM manual PWM Out Register active timing.
  * @param  MCM_ActiveTiming: specifie the timing of the manual register active.
  *          This parameter can be one of the following values:
  *            @arg MCM_ManualActiveTiming_Now
  *            @arg MCM_ManualActiveTiming_PeriodUpdate  
  * @retval None
  */
void MCM_ManualPWMOutRegSync(uint16_t MCM_ActiveTiming)
{    
    /* Check the parameters */
    assert_param(IS_MCM_MANUAL_ACTIVE_TIMING(MCM_ActiveTiming));

    MCM_REG_UNLOCK;

    MCM->PWMCON2.V32 &= ~MCM_PWMCON2_OSYNC_Msk;
    MCM->PWMCON2.V32 |= MCM_ActiveTiming;

    MCM_REG_LOCK;
}

/**
  * @brief  Turn On or turn off manual output mode of the specified PWM port.
  * @param  MCM_PWMx: specifie PWM port to be enabled or disabled.
  *          This parameter can be any combination of following values:
  *            @arg MCM_MANUAL_PWM0
  *            @arg MCM_MANUAL_PWM1
  *            @arg MCM_MANUAL_PWM2
  *            @arg MCM_MANUAL_PWM01
  *            @arg MCM_MANUAL_PWM11
  *            @arg MCM_MANUAL_PWM21
  * @param  OnOffState: state of synchronous function.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void MCM_ManualPWMOutOnOff(uint8_t MCM_PWMx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_MCM_MANUAL_PWM(MCM_PWMx));
    assert_param(IS_CMD_STATE(OnOffState)); 

    MCM_REG_UNLOCK;

    if (OnOffState != SH_OFF)
    {
        /* Enable manual output mode of specified PWM port */
        MCM->PMANUALCON1.V32 |= MCM_PWMx; 
    }
    else
    {
        /* Disable manual output mode of specified PWM port */
        MCM->PMANUALCON1.V32 &= ~MCM_PWMx;
    }

    MCM_REG_LOCK;
}

/**
  * @brief  Set the specified PWM port output level through the PWM Mannual Register.
  * @param  MCM_PWMx: specifie PWM port to high or low.
  *          This parameter can be any combination of following values:
  *            @arg MCM_MANUAL_PWM0
  *            @arg MCM_MANUAL_PWM1
  *            @arg MCM_MANUAL_PWM2
  *            @arg MCM_MANUAL_PWM01
  *            @arg MCM_MANUAL_PWM11
  *            @arg MCM_MANUAL_PWM21
  * @param  NewState: high or low.
  *          This parameter can be any combination of following values:
  *            @arg PwmManualOutLow
  *            @arg PwmManualOutHigh
  * @retval None
  */
void MCM_ManualPWMOutLevel(uint8_t MCM_PWMx, MCM_MANUAL_OUT_LEVEL_State NewState)
{
    /* Check the parameters */
    assert_param(IS_MCM_MANUAL_PWM(MCM_PWMx));
    assert_param(IS_MCM_OUTPUT_LEVEL(NewState));

    MCM_REG_UNLOCK;
    
    if (PwmManualOutHigh == NewState)
    {
        /* Set the specified PWM port out high */
        MCM->PMANUALCON2.V32 |= MCM_PWMx;
    }
    else
    {
        /* Set the specified PWM port out low */
        MCM->PMANUALCON2.V32 &= ~MCM_PWMx;        
    }

    MCM_REG_LOCK;
}

/**
  * @}
  */
  
/** @defgroup MCM_Group6 PWM PROTECT
 *  @brief    PWM PROTECT
 *
@verbatim   
 ===============================================================================
                               PWM PROTECT
 ===============================================================================

@endverbatim
  * @{
  */

/**
  * @brief  Enable or disable FLT module registers write protection.
  * @param  NewState: new state of FLT write protection.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void MCM_FLTWriteLock(SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTION_STATE(NewState));
    
    if (NewState != SH_DISABLE)
    {
        /* Enable the FLT register write protection */
        MCM->FLTWEN = 0x0000;
    }
    else
    {
        /* Disable the FLT register write protection */
        MCM->FLTWEN = 0x33CC;
    }
}

/**
  * @brief  Enable or disable Fault Detect Module(FLT).  
  * @param  MCM_FLT: specifie which FLT module to be enable or disable.
  *          This parameter can be any combination of the following values:
  *            @arg MCM_FLT2_Pin
  *            @arg MCM_FLT1_COMP1
  *            @arg MCM_FLT0_COMP0
  * @param  OnOffState: state of FLT module.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void MCM_FLTOnOff(uint32_t MCM_FLT, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_MCM_FLT_INPUT(MCM_FLT));
    assert_param(IS_CMD_STATE(OnOffState));

    MCM_FLT_REG_UNLOCK;

    if (OnOffState != SH_OFF)
    {
        MCM->FLTCON.V32 |= MCM_FLT;
    }
    else
    {
        MCM->FLTCON.V32 &= ~MCM_FLT;
    }

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Configure filter and active level of the MCM FLT0 module. 
  * @param  MCM_FLT0DetectMode: specifie the FLT0 detect mode to set.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT0_DETECT_Latch
  *            @arg MCM_FLT0_DETECT_Each 
  * @param  MCM_FLT0ActiveLevel: specifie the FLT0 active level.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT0_ActiveLevel_High
  *            @arg MCM_FLT0_ActiveLevel_Low
  * @retval None
  */
void MCM_FLT0Config(uint32_t MCM_FLT0DetectMode, uint32_t MCM_FLT0ActiveLevel)
{
    /* Check the parameters */    
    assert_param(IS_MCM_FLT0_DETECT_MODE(MCM_FLT0DetectMode));
    assert_param(IS_MCM_FLT0_ACTIVE_LEVEL(MCM_FLT0ActiveLevel));

    MCM_FLT_REG_UNLOCK;

    MCM->FLTCON.V32 &= ~(MCM_FLTCON_FLT0M_Msk | MCM_FLTCON_FLT0S_Msk);
    MCM->FLTCON.V32 |= (MCM_FLT0DetectMode | MCM_FLT0ActiveLevel);

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Configure filter and active level of the MCM FLT1 module. 
  * @param  MCM_FLT1DetectMode: specifie the FLT1 detect mode to set.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT1_DETECT_Latch
  *            @arg MCM_FLT1_DETECT_Each 
  * @param  MCM_FLT1ActiveLevel: specifie the FLT1 active level.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT1_ActiveLevel_High
  *            @arg MCM_FLT1_ActiveLevel_Low
  * @retval None
  */
void MCM_FLT1Config(uint32_t MCM_FLT1DetectMode, uint32_t MCM_FLT1ActiveLevel)
{
    /* Check the parameters */    
    assert_param(IS_MCM_FLT1_DETECT_MODE(MCM_FLT1DetectMode));
    assert_param(IS_MCM_FLT1_ACTIVE_LEVEL(MCM_FLT1ActiveLevel));

    MCM_FLT_REG_UNLOCK;

    MCM->FLTCON.V32 &= ~(MCM_FLTCON_FLT1M_Msk | MCM_FLTCON_FLT1S_Msk);
    MCM->FLTCON.V32 |= (MCM_FLT1DetectMode | MCM_FLT1ActiveLevel);

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Configure filter and active level of the MCM FLT2 module. 
  * @param  MCM_FLT2DetectMode: specifie the FLT2 detect mode to set.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT2_DETECT_Latch
  *            @arg MCM_FLT2_DETECT_Each 
  * @param  MCM_FLT2ActiveLevel: specifie the FLT2 active level.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT2_ActiveLevel_High
  *            @arg MCM_FLT2_ActiveLevel_Low
  * @param  MCM_FLT2Filter: specifie the FLT2 filter value.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT2_Filter_None         
  *            @arg MCM_FLT2_Filter_0dot5us      
  *            @arg MCM_FLT2_Filter_1us          
  *            @arg MCM_FLT2_Filter_1dot5us      
  *            @arg MCM_FLT2_Filter_2us          
  *            @arg MCM_FLT2_Filter_3us          
  *            @arg MCM_FLT2_Filter_4us          
  *            @arg MCM_FLT2_Filter_6us          
  *            @arg MCM_FLT2_Filter_8us          
  *            @arg MCM_FLT2_Filter_10us         
  *            @arg MCM_FLT2_Filter_12us         
  *            @arg MCM_FLT2_Filter_14us         
  *            @arg MCM_FLT2_Filter_16us         
  *            @arg MCM_FLT2_Filter_20us         
  *            @arg MCM_FLT2_Filter_24us         
  *            @arg MCM_FLT2_Filter_32us  
  * @retval None
  */
void MCM_FLT2Config(uint32_t MCM_FLT2DetectMode, uint32_t MCM_FLT2ActiveLevel, uint32_t MCM_FLT2Filter)
{
    /* Check the parameters */    
    assert_param(IS_MCM_FLT2_DETECT_MODE(MCM_FLT2DetectMode));
    assert_param(IS_MCM_FLT2_ACTIVE_LEVEL(MCM_FLT2ActiveLevel));
    assert_param(IS_MCM_FLT2_FILTER(MCM_FLT2Filter));

    MCM_FLT_REG_UNLOCK;

    MCM->FLTCON.V32 &= ~(MCM_FLTCON_FLT2M_Msk | MCM_FLTCON_FLT2S_Msk | MCM_FLTCON_FLT2DEB_Msk);
    MCM->FLTCON.V32 |= (MCM_FLT2DetectMode | MCM_FLT2ActiveLevel | MCM_FLT2Filter);

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Configure the MCM FLT mode and output pin status. 
  * @param  MCM_PWMxPinStatus: specifie the output status of the FLT PWMx pin.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT_PWMx_PinStatus_Hiz
  *            @arg MCM_FLT_PWMx_PinStatus_Low
  *            @arg MCM_FLT_PWMx_PinStatus_High  
  * @param  MCM_PWMx1PinStatus: specifie the output status of the FLT PWMx1 pin.
  *          This parameter can be one of the following values:
  *            @arg MCM_FLT_PWMx1_PinStatus_Hiz
  *            @arg MCM_FLT_PWMx1_PinStatus_Low
  *            @arg MCM_FLT_PWMx1_PinStatus_High   
  * @retval None
  */
void MCM_FLTOutLevelConfig(uint32_t MCM_PWMxPinStatus, uint32_t MCM_PWMx1PinStatus)
{
    /* Check the parameters */
    assert_param(IS_MCM_FLT_PWMx_PIN_STATUS(MCM_PWMxPinStatus));
    assert_param(IS_MCM_FLT_PWMx1_PIN_STATUS(MCM_PWMx1PinStatus));

    MCM_FLT_REG_UNLOCK;

    MCM->FLTCON.V32 &= ~(MCM_FLTCON_FOUT0_Msk | MCM_FLTCON_FOUT1_Msk);
    MCM->FLTCON.V32 |= (MCM_PWMxPinStatus | MCM_PWMx1PinStatus);    

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Check whether the specified MCM FLT status flag is set or not  
  * @param  MCM_FLTx_STAT: where x can be  0 ~ 2
  * @retval None
  */
SH_FlagStatus MCM_GetFLTStatus(uint32_t MCM_FLTx_STAT)
{
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_MCM_FLT_STATUS(MCM_FLTx_STAT));

    if ((MCM->FLTCON.V32 & MCM_FLTx_STAT) != SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @brief  Clear the MCM's FLT pending flag.
  * @note   This function only can be called in Latch mode of FLT,because this bit will be
  *         set by hardware in Each Mode.
  * @param  FltClearStat: where can be FLT0~2
  * @retval None
  */
void MCM_ClearFLTStatusFlag(uint32_t FltClearStat)
{
    
    assert_param(IS_MCM_FLT_CLEAR_STATUS(FltClearStat));
    
    MCM_FLT_REG_UNLOCK;

    MCM->FLTCON.V32 |= FltClearStat;

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Open or close oscillator stop detection.
  * @param  OnOffState: state of oscillator stop detection.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void MCM_OSCStopDetectOnOff(SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));

    MCM_FLT_REG_UNLOCK;

    if (OnOffState != SH_OFF)
    {
        MCM->POSTDCR.V32 = MCM_POSTDCR_OSTDEN_Msk;
    }
    else
    {
        MCM->POSTDCR.V32 = ~MCM_POSTDCR_OSTDEN_Msk;
    }

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Open or close the MCM short circuit protection function.  
  * @param  OnOffState: state of the short circuit protection.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF  
  * @retval None
  */
void MCM_ShortCircuitProtectOnOff(SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));

    MCM_FLT_REG_UNLOCK;

    if (OnOffState != SH_OFF)
    {
        MCM->POSCR.V32 |= MCM_POSCR_OLSEN_Msk;
    }
    else
    {
        MCM->POSCR.V32 &= ~MCM_POSCR_OLSEN_Msk;
    }

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Configure the MCM short circuit output level.  
  * @param  MCM_PWMxPin: specifie the PWM Pin of the short circuit function.
  *          This parameter can be any combination of the following values:
  *            @arg MCM_ShortCircuit_PWM0Pin     
  *            @arg MCM_ShortCircuit_PWM1Pin         
  *            @arg MCM_ShortCircuit_PWM2Pin        
  *            @arg MCM_ShortCircuit_PWM01Pin    
  *            @arg MCM_ShortCircuit_PWM11Pin       
  *            @arg MCM_ShortCircuit_PWM21Pin       
  * @param  MCM_OutputLevel: specifie the output level of the short circuit function.
  *          This parameter can be one of the following values:
  *            @arg MCM_ShortCircuit_Level_Low
  *            @arg MCM_ShortCircuit_Level_High
  * @retval None
  */
void MCM_ShortCircuitOutPutLevel(uint8_t MCM_PWMxPin, uint8_t MCM_OutputLevel)
{
    /* Check the parameters */
    assert_param(IS_MCM_SHORT_CIRCUIT_PIN(MCM_PWMxPin));
    assert_param(IS_MCM_SHORT_CIRCUIT_LEVEL(MCM_OutputLevel));

    MCM_FLT_REG_UNLOCK;

    if (MCM_OutputLevel != MCM_ShortCircuit_Level_Low)
    {
        MCM->POSCR.V32 |= MCM_PWMxPin;
    }
    else
    {
        MCM->POSCR.V32 &= ~MCM_PWMxPin;
    }

    MCM_FLT_REG_LOCK;
}

/**
  * @brief  Fills each MCM_SCInitStruct member with its default value.
  * @param  InitStruct : pointer to a @ref MCM_SCInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void MCM_SCStructInit(MCM_SCInitTypeDef* InitStruct)
{
    InitStruct->SC_InputSource = MCM_SC_InputSource_DISABLE;
    InitStruct->SC_ProtectPWM0 = MCM_SC_Protect_PWM0_ENABLE;
    InitStruct->SC_ProtectPWM1 = MCM_SC_Protect_PWM1_ENABLE;
    InitStruct->SC_ProtectPWM2 = MCM_SC_Protect_PWM2_ENABLE;
    InitStruct->SC_InputLevel = MCM_SC_InputLevel_High;
    InitStruct->SC_ProtectTime = MCM_SC_ProtectTime_2400CLK;
    InitStruct->SC_Filter = MCM_SC_Filter_160CLK;
}

/**
  * @brief  Initialize the MCM Stoppage Check(SC) module 3 according to 
  *         the specified parameters in the MCM_SCInitStruct.  
  * @param  InitCfg: pointer to a @ref MCM_SCInitTypeDef structure
  *         that contains the configuration information for the Stoppage Check(SC) module 3.
  * @retval None
  */
void MCM_SC0Init(MCM_SCInitTypeDef* InitCfg)
{
    uint16_t temp = 0;
    
    MCM_REG_UNLOCK;

    /* Reset SCTIME/SCDEB/SCS/ bits */
    MCM->SC0CON.V32 &= (uint16_t)0xC000;

    temp = InitCfg->SC_InputSource;
    temp |= InitCfg->SC_InputLevel;
    temp |= InitCfg->SC_ProtectPWM0;
    temp |= InitCfg->SC_ProtectPWM1;
    temp |= InitCfg->SC_ProtectPWM2;
    temp |= InitCfg->SC_Filter;
    temp |= InitCfg->SC_ProtectTime;
    MCM->SC0CON.V32 |= temp;

    MCM_REG_LOCK;
}

/**
  * @brief  Initialize the MCM Stoppage Check(SC) module 1 according to 
  *         the specified parameters in the MCM_SCInitStruct.  
  * @param  InitCfg: pointer to a @ref MCM_SCInitTypeDef structure
  *         that contains the configuration information for the Stoppage Check(SC) module 1.
  * @retval None
  */
void MCM_SC1Init(MCM_SCInitTypeDef* InitCfg)
{
    uint16_t temp = 0;
    
    MCM_REG_UNLOCK;

    /* Reset SCTIME/SCDEB/SCS/ bits */
    MCM->SC1CON.V32 &= (uint16_t)0xC000;

    temp = InitCfg->SC_InputSource;
    temp |= InitCfg->SC_InputLevel;
    temp |= InitCfg->SC_ProtectPWM0;
    temp |= InitCfg->SC_ProtectPWM1;
    temp |= InitCfg->SC_ProtectPWM2;
    temp |= InitCfg->SC_Filter;
    temp |= InitCfg->SC_ProtectTime;
    MCM->SC1CON.V32 |= temp;

    MCM_REG_LOCK;
}

/**
  * @brief  Initialize the MCM Stoppage Check(SC) module 2 according to 
  *         the specified parameters in the MCM_SCInitStruct.  
  * @param  InitCfg: pointer to a @ref MCM_SCInitTypeDef structure
  *         that contains the configuration information for the Stoppage Check(SC) module 2.
  * @retval None
  */
void MCM_SC2Init(MCM_SCInitTypeDef* InitCfg)
{
    uint16_t temp = 0;

    MCM_REG_UNLOCK;

    /* Reset SCTIME/SCDEB/SCS/ bits */
    MCM->SC2CON.V32 &= (uint16_t)0xC000;

    temp = InitCfg->SC_InputSource;
    temp |= InitCfg->SC_InputLevel;
    temp |= InitCfg->SC_ProtectPWM0;
    temp |= InitCfg->SC_ProtectPWM1;
    temp |= InitCfg->SC_ProtectPWM2;
    temp |= InitCfg->SC_Filter;
    temp |= InitCfg->SC_ProtectTime;
    MCM->SC2CON.V32 |= temp;

    MCM_REG_LOCK;
}

/**
  * @brief  Check whether the specified MCM SC status flag is set or not  
  * @param  MCM_SCx_STAT: where x can be  0 ~ 2
  * @retval None
  */ 
SH_FlagStatus MCM_GetSCxStatus(uint32_t MCM_SCx_STAT)
{ 
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_MCM_SCxSTAT(MCM_SCx_STAT));
    
    if ((MCM->PWMINTF.V32 & MCM_SCx_STAT) != SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;    
}

/**
  * @brief  clear the specified MCM SC status flag  
  * @param  MCM_SCx_STAT: where x can be  0 ~ 2
  * @retval None
  */
void MCM_ClearSCxStatus(uint32_t MCM_SCx_STAT)
{
    /* Check the parameters */
    assert_param(IS_MCM_SCxSTAT(MCM_SCx_STAT));

    MCM->PWMINTF.V32 |= (MCM_SCx_STAT << 16);
}

/**
  * @}
  */
  
/** @defgroup MCM_Group7 Saturation and Phase Shift
 *  @brief    Saturation and Phase Shift
 *
@verbatim   
 ===============================================================================
                               Saturation and Phase Shift
 ===============================================================================

@endverbatim
  * @{
  */

/**
  * @brief  Set the MCM Duty saturation minimum and maximum Register value.
  * @param  MCM_PWMMin: specifie the minimum value of the duty saturation.
  *          This parameter can be 0 to 0xFFFF.
  * @param  MCM_PWMMax: specifie the maximum value of the duty saturation.
  *          This parameter can be 0 to 0xFFFF.  
  * @retval None
  */
void MCM_DutySaturationMinMaxValue(uint16_t MCM_PWMMin, uint16_t MCM_PWMMax)
{
    MCM_REG_UNLOCK;

    MCM->PWMDMIN = MCM_PWMMin;
    MCM->PWMDMAX = MCM_PWMMax;

    MCM_REG_LOCK;
}

/**
  * @brief  Configure the MCM Duty Register Change or not with the duty saturation function.
  * @param  MCM_MinSelect: specifie the minimum register to set.
  *          This parameter can be one of the following values:
  *            @arg MCM_MinSelect_None
  *            @arg MCM_MinSelect_ToMin
  *            @arg MCM_MinSelect_ToZero
  * @param  MCM_MaxSelect: specifie the maximum register to set.
  *          This parameter can be one of the following values:
  *            @arg MCM_MaxSelect_None
  *            @arg MCM_MaxSelect_ToMax
  *            @arg MCM_MaxSelect_ToPeriod   
  * @retval None
  */
void MCM_DutySaturationConfig(uint16_t MCM_MinSelect, uint16_t MCM_MaxSelect)
{
    /* Check the parameters */
    assert_param(IS_MCM_MIN_SELECT(MCM_MinSelect));
    assert_param(IS_MCM_MAX_SELECT(MCM_MaxSelect));

    MCM_REG_UNLOCK;

    MCM->PSCON.V32 &= ~(MCM_PSCON_MINSELECT_Msk | MCM_PSCON_MAXSELECT_Msk);
    MCM->PSCON.V32 |= MCM_MinSelect | MCM_MaxSelect;

    MCM_REG_LOCK;
}

/**
  * @brief  run the MCM's Duty Saturation function.  
  * @retval None
  */
void MCM_DutySaturationRun(void)
{
    MCM_REG_UNLOCK;

    /* Enable the duty saturation function */
    MCM->PSCON.V32 |= MCM_PSCON_STATRUN_Msk;

    MCM_REG_LOCK;
}

/**
  * @brief  configure the MCM's phase shift function. 
  * @param  MCM_Shift1: specifie the Phase Shift Register 1 value.
  *          This parameter can be 0 to 0xFFFF.
  * @param  MCM_Shift2: specifie the Phase Shift Register 2 value.
  *          This parameter can be 0 to 0xFFFF.  
  * @retval None
  */
void MCM_PhaseShiftConfig(uint16_t MCM_Shift1, uint16_t MCM_Shift2)
{

    MCM_REG_UNLOCK;

    MCM->PWMDCMP1 = MCM_Shift1;
    MCM->PWMDCMP2 = MCM_Shift2;

    MCM_REG_LOCK;
}

/**
  * @brief  run the MCM's Phase Shift function.  
  * @retval None
  */
void MCM_PhaseShiftRun(void)
{
    MCM_REG_UNLOCK;

    /* Enable the Phase Shift function */
    MCM->PSCON.V32 |= MCM_PSCON_SHIFTRUN_Msk;

    MCM_REG_LOCK;
}

/**
  * @brief  run MCM's Phase Shift & Saturation function at the same time 
  * @retval None
  */
void MCM_PhaseShiftSatRun(void)
{
    MCM_REG_UNLOCK;

    /* Enable the Phase Shift function */
    MCM->PSCON.V32 |= (MCM_PSCON_SHIFTRUN_Msk | MCM_PSCON_STATRUN_Msk);

    MCM_REG_LOCK;
}


/**
  * @brief  Check whether the phase shift over  
  * @retval None
  */
SH_FlagStatus MCM_GetPhaseShiftOver(void)
{
    SH_FlagStatus bitStatus;

    if (MCM->PSCON.BIT.OVER != SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @brief  get the phase shift sector  
  * @retval None
  */
uint8_t MCM_GetPhaseShiftSector(void)
{  
    return (uint8_t)MCM->PSCON.BIT.SECTOR;
}

/**
  * @}
  */ 
  
/**
  * @}
  */ 


/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


