/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_pca.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_pca.h"

/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup PCA_MODULE PCA 
  * @brief PCA driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/** @defgroup PCA_Public_Functions Public Functions
  * @{
  */ 



/**
  * @brief  Reset the PCAx peripheral registers to their default reset values.
  * @param  PCAx: Where x can be 0 or 1 to select the PCA peripheral.
  * @retval None
  */
void PCA_Reset(PCA_TypeDef* PCAx)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
    if (PCAx == PCA0)
        RCC_APB0PeriphReset(RCC_APB0_PCA0);
		
		else if (PCAx == PCA1)
        RCC_APB0PeriphReset(RCC_APB0_PCA1);
}


/**
  * @brief  Enables or disables the PCA's registers protection.
  * @param  PCAx: where x can be 0 or 1 to select the PCAx peripheral.
  * @param  NewState: new state of the PCA's registers protection.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE:
*            @arg SH_DISABLE:
  * @retval None
  */
void PCA_RegisterWriteLock(PCA_TypeDef* PCAx, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_FUNCTION_STATE(NewState));

    if (NewState != SH_DISABLE)
    {
        /* Enable PCA registers protection */
        PCAx->LCKR = 0x33CC;
    }
    else
    {
        /* Disable PCA registers protection */
        PCAx->LCKR = 0x0;
    }
}

/**
  * @brief  On or off the specified PCA peripheral.
  * @param  PCAx: where x can be 0 or 1 to select the PCAx peripheral.
  * @param  OnOffState: state of the PCAx peripheral.
  *          This parameter can be one of the following values:
*            @arg SH_ON: PCA on
*            @arg SH_OFF: PCA off
  * @retval None
  */
void PCA_OnOff(PCA_TypeDef* PCAx, SH_CmdState OnOffState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_CMD_STATE(OnOffState));

  PCA_REG_UNLOCK(PCAx);

	if (OnOffState != SH_OFF)
	{
		PCAx->CR |= PCA_CR_PEN_Msk;
	}
	else
	{
		PCAx->CR &= ~PCA_CR_PEN_Msk;
	} 

   PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Fills each PCA_TimeBaseInitStruct member with its default value.
  * @param  PCA_TimeBaseInitStruct : pointer to a @ref PCA_TimeBaseInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_TimeBaseStructInit(PCA_TimeBaseInitTypeDef* PCA_TimeBaseInitStruct)
{
    PCA_TimeBaseInitStruct->PCAMode = CCMR_SM_FREQ;
    PCA_TimeBaseInitStruct->CounterMode = CFGR_SDEN_UP;
    PCA_TimeBaseInitStruct->InitialLevel = CCMR_TCP_HIGHLEVEL;
    PCA_TimeBaseInitStruct->Prescaler = 0x00;
    PCA_TimeBaseInitStruct->Period = 0x0000FFFF;
    PCA_TimeBaseInitStruct->Count = 0x0;
		PCA_TimeBaseInitStruct->InitA_Data		= 0x8000; 
		PCA_TimeBaseInitStruct->ReloadA_Data	= 0x8000; 
		PCA_TimeBaseInitStruct->InitB_Data		= 0x8000; 
		PCA_TimeBaseInitStruct->ReloadB_Data	= 0x8000; 
		PCA_TimeBaseInitStruct->InitC_Data		= 0x8000; 
		PCA_TimeBaseInitStruct->ReloadC_Data	= 0x8000;

}

/**
  * @brief  Initializes the PCAx Time Base Unit peripheral according to 
  *         the specified parameters in the PCA_TimeBaseInitStruct.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_TimeBaseInitStruct: pointer to a @ref PCA_TimeBaseInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_TimeBaseInit(PCA_TypeDef* PCAx, PCA_TimeBaseInitTypeDef* PCA_TimeBaseInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_TimeBaseInitStruct->PCAMode));

    PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	PCAx->CFGR.V32 |= PCA_TimeBaseInitStruct->CounterMode;
	
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk); 
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk); 
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk); 

	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_TimeBaseInitStruct->PCAMode
										|PCA_TimeBaseInitStruct->InitialLevel
										|CCMR_CEN_EN);
	
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_TimeBaseInitStruct->PCAMode
										|PCA_TimeBaseInitStruct->InitialLevel
										|CCMR_CEN_EN);
	
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR2.V32 |=	 ( PCA_TimeBaseInitStruct->PCAMode
										|PCA_TimeBaseInitStruct->InitialLevel
										|CCMR_CEN_EN);

  /* Set PCA Prescaler ,Period and count register value */
	PCAx->PSC = PCA_TimeBaseInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_TimeBaseInitStruct->Period;
	PCAx->CNT.V32 = PCA_TimeBaseInitStruct->Count;
	
	/* Set initial data and reload data */
	PCAx->CCR0.BIT.CCR0L = PCA_TimeBaseInitStruct->InitA_Data;
	PCAx->CCR0.BIT.CCR0H = PCA_TimeBaseInitStruct->ReloadA_Data;
	PCAx->CCR1.BIT.CCR1L = PCA_TimeBaseInitStruct->InitB_Data;
	PCAx->CCR1.BIT.CCR1H = PCA_TimeBaseInitStruct->ReloadB_Data;
	PCAx->CCR2.BIT.CCR2L = PCA_TimeBaseInitStruct->InitC_Data;
	PCAx->CCR2.BIT.CCR2H = PCA_TimeBaseInitStruct->ReloadC_Data;
	

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Fills each PCA_CaptureInitStruct member with its default value.
  * @param  PCA_CaptureInitStruct : pointer to a @ref PCA_CaptureInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_CaptureStructInit(PCA_CaptureInitTypeDef* PCA_CaptureInitStruct)
{
	PCA_CaptureInitStruct->PCAMode = CCMR_SM_CAPTURE;
	PCA_CaptureInitStruct->Prescaler = 0x00;
	PCA_CaptureInitStruct->Count = 0x0;
	PCA_CaptureInitStruct->CounterMode = CFGR_SDEN_UP;
	PCA_CaptureInitStruct->TriggerLevel = CCMR_FS_UP_DOWN_CAPTURE;
	PCA_CaptureInitStruct->InputFilter = CCMR_ICF_320PCLK;
	PCA_CaptureInitStruct->CaptureClear = CCMR_CC_EN;
	PCA_CaptureInitStruct->CaptureSelect = CCMR_CSSEL_PCAxA;	
	PCA_CaptureInitStruct->Period = 0xFFF;
}


/**
  * @brief  Initializes the PCAx Capture Unit peripheral according to 
  *         the specified parameters in the PCA_CaptureInitStruct.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_CaptureInitStruct: pointer to a @ref PCA_CaptureInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_CaptureInit(PCA_TypeDef* PCAx, PCA_CaptureInitTypeDef* PCA_CaptureInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_CaptureInitStruct->PCAMode));
	assert_param(IS_PCA_CAPTURE_FILTER(PCA_CaptureInitStruct->InputFilter));
	assert_param(IS_PCA_CAPTURE_SELECT(PCA_CaptureInitStruct->CaptureSelect));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	PCAx->CFGR.V32 |= PCA_CaptureInitStruct->CounterMode;
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_ICF_Msk | PCA_CCMR0_SM_Msk | PCA_CCMR0_FS_Msk | PCA_CCMR0_TCP_Msk | PCA_CCMR0_CSSEL_Msk |  PCA_CCMR0_CC_Msk | PCA_CCMR0_CEN_Msk); 
	/* Set the PCA mode, the trigger level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_CaptureInitStruct->PCAMode
										|PCA_CaptureInitStruct->TriggerLevel
										|PCA_CaptureInitStruct->InputFilter
										|PCA_CaptureInitStruct->CaptureClear
										|PCA_CaptureInitStruct->CaptureSelect
										|CCMR_CEN_EN);
	/* Reset the  bits */
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_ICF_Msk | PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk | PCA_CCMR1_CSSEL_Msk| PCA_CCMR1_CC_Msk | PCA_CCMR1_CEN_Msk); 
	/* Set the PCA mode, the trigger level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_CaptureInitStruct->PCAMode
										|PCA_CaptureInitStruct->TriggerLevel
										|PCA_CaptureInitStruct->InputFilter
										|PCA_CaptureInitStruct->CaptureClear
										|PCA_CaptureInitStruct->CaptureSelect
										|CCMR_CEN_EN);
	/* Reset the  bits */
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_ICF_Msk | PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk | PCA_CCMR2_CSSEL_Msk| PCA_CCMR2_CC_Msk | PCA_CCMR2_CEN_Msk); 
	/* Set the PCA mode, the trigger level of output and enable bit */
	PCAx->CCMR2.V32 |=	 ( PCA_CaptureInitStruct->PCAMode
										|PCA_CaptureInitStruct->TriggerLevel
										|PCA_CaptureInitStruct->InputFilter
										|PCA_CaptureInitStruct->CaptureClear
										|PCA_CaptureInitStruct->CaptureSelect
									  |CCMR_CEN_EN);

  /* Set PCA Prescaler and Period register value */
	PCAx->PSC = PCA_CaptureInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_CaptureInitStruct->Period;
	PCAx->CNT.V32 = PCA_CaptureInitStruct->Count;
	
  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Fills each PCA_CMPInitStruct member with its default value.
  * @param  PCA_CMPInitStruct : pointer to a @ref PCA_CMPInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_CMPStructInit(PCA_CMPInitTypeDef* PCA_CMPInitStruct)
{
	PCA_CMPInitStruct->PCAMode = CCMR_SM_CMP;
	PCA_CMPInitStruct->Prescaler = 0x00;
	PCA_CMPInitStruct->Count = 0x00;
	PCA_CMPInitStruct->CounterMode = CFGR_SDEN_UP;
	PCA_CMPInitStruct->CMPMode = CCMR_FS_CONTINUE_CMP;
	PCA_CMPInitStruct->InitialLevel = CCMR_TCP_HIGHLEVEL;
	
	PCA_CMPInitStruct->Period = 0xFF;
	PCA_CMPInitStruct->CMPA_Value = 0x80;
	PCA_CMPInitStruct->CMPB_Value = 0x40;
	PCA_CMPInitStruct->CMPC_Value = 0x20;
}


/**
  * @brief  Initializes the PCAx Compare Unit peripheral according to 
  *         the specified parameters in the PCA_CMPInitStruct.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_CMPInitStruct: pointer to a @ref PCA_CMPInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_CMPInit(PCA_TypeDef* PCAx, PCA_CMPInitTypeDef* PCA_CMPInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_CMPInitStruct->PCAMode));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	PCAx->CFGR.V32 |= PCA_CMPInitStruct->CounterMode;
	
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk | PCA_CCMR0_CEN_Msk); 
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_CMPInitStruct->PCAMode
										|PCA_CMPInitStruct->InitialLevel
										|PCA_CMPInitStruct->CMPMode
										|CCMR_CEN_EN);
	
	/* Reset the  bits */
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk | PCA_CCMR1_CEN_Msk); 
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_CMPInitStruct->PCAMode
										|PCA_CMPInitStruct->InitialLevel
										|PCA_CMPInitStruct->CMPMode
										|CCMR_CEN_EN);
	
	/* Reset the  bits */
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk | PCA_CCMR2_CEN_Msk); 
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR2.V32 |=	 ( PCA_CMPInitStruct->PCAMode
										|PCA_CMPInitStruct->InitialLevel
										|PCA_CMPInitStruct->CMPMode
										|CCMR_CEN_EN);

  /* Set Compare register value */
	PCAx->PSC = PCA_CMPInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_CMPInitStruct->Period;
	PCAx->CNT.V32 = PCA_CMPInitStruct->Count;

	/* Set initial data and reload data */
	PCAx->CCR0.V32 = PCA_CMPInitStruct->CMPA_Value;
	PCAx->CCR1.V32 = PCA_CMPInitStruct->CMPB_Value;
	PCAx->CCR2.V32 = PCA_CMPInitStruct->CMPC_Value; 

  PCA_REG_LOCK(PCAx);
}




/**
  * @brief  Fills each PCA_PWMInitStruct member with its default value.
  * @param  PCA_PWMInitStruct : pointer to a @ref PCA_PWMInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_PWMStructInit(PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */
	PCA_PWMInitStruct->PCAMode = CCMR_SM_PWM;
	PCA_PWMInitStruct->PWMMode = CCMR_FS_PWM3;
	PCA_PWMInitStruct->InitialLevel = CCMR_TCP_DUTYLOW;
	
	/* Set the counter mode*/
	if((PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1) || (PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM2))
		PCA_PWMInitStruct->CounterMode = CFGR_SDEN_UP;
	else
		PCA_PWMInitStruct->CounterMode = CFGR_SDEN_CENTRAL;
	
	PCA_PWMInitStruct->Prescaler = 0x00;
	PCA_PWMInitStruct->Count = 0x00;
		
	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		/* Set initial data and reload data */
		PCA_PWMInitStruct->Period = 0x0000FFFF;
		PCA_PWMInitStruct->InitA_Data		= 0x800; 
		PCA_PWMInitStruct->ReloadA_Data	= 0x800; 
		PCA_PWMInitStruct->InitB_Data		= 0x800; 
		PCA_PWMInitStruct->ReloadB_Data	= 0x800; 
		PCA_PWMInitStruct->InitC_Data		= 0x800; 
		PCA_PWMInitStruct->ReloadC_Data	= 0x800;
	}
	else
	{
	  /* Set PWM Duty data*/
		PCA_PWMInitStruct->Period = 0xFF;
		PCA_PWMInitStruct->DutyA_Value = 0x80;
		PCA_PWMInitStruct->DutyB_Value = 0x40;
		PCA_PWMInitStruct->DutyC_Value = 0x20;
	}
}

/**
  * @brief  Initializes the PCAx Time Base Unit peripheral according to 
  *         the specified parameters in the PCA_TimeBaseInitStruct.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_PWMInit(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));
	
  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;
	
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk | PCA_CCMR0_CEN_Msk); 
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);

	/* Reset the  bits */
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk | PCA_CCMR1_CEN_Msk); 
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);
										
	/* Reset the  bits */
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk | PCA_CCMR2_CEN_Msk);
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */	
	PCAx->CCMR2.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);

  /* Set PCA Prescaler and Period register value */
	PCAx->PSC = PCA_PWMInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_PWMInitStruct->Period;
	PCAx->CNT.V32 = PCA_PWMInitStruct->Count;

	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->PR.V32 = 0x0000FFFF;
		PCAx->CCR0.BIT.CCR0L = PCA_PWMInitStruct->InitA_Data;
		PCAx->CCR0.BIT.CCR0H = PCA_PWMInitStruct->ReloadA_Data;
		PCAx->CCR1.BIT.CCR1L = PCA_PWMInitStruct->InitB_Data;
		PCAx->CCR1.BIT.CCR1H = PCA_PWMInitStruct->ReloadB_Data;
		PCAx->CCR2.BIT.CCR2L = PCA_PWMInitStruct->InitC_Data;
		PCAx->CCR2.BIT.CCR2H = PCA_PWMInitStruct->ReloadC_Data;
	}
	else
	{
		PCAx->PR.V32 = PCA_PWMInitStruct->Period;
		PCAx->CCR0.V32 = PCA_PWMInitStruct->DutyA_Value;
		PCAx->CCR1.V32 = PCA_PWMInitStruct->DutyB_Value;
		PCAx->CCR2.V32 = PCA_PWMInitStruct->DutyC_Value; 
	}
	
  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Initialize the PCAxA function according to 
  *         the specified parameters in the PCA_PWMInitStruct.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the PWM0 function.
  * @retval None
  */
void PCA_PCAxAPWMConfig(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;

 	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->CCR0.BIT.CCR0L = PCA_PWMInitStruct->InitA_Data;
		PCAx->CCR0.BIT.CCR0H = PCA_PWMInitStruct->ReloadA_Data;
	}
	else
	{
		PCAx->CCR0.V32 = PCA_PWMInitStruct->DutyA_Value;
	}
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk); 
	PCAx->CCMR0.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);	
  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Initialize the PCAxB function according to 
  *         the specified parameters in the PCA_PWMInitStruct.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the PWM1 function.
  * @retval None
  */
void PCA_PCAxBPWMConfig(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;
	
	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->CCR1.BIT.CCR1L = PCA_PWMInitStruct->InitB_Data;
		PCAx->CCR1.BIT.CCR1H = PCA_PWMInitStruct->ReloadB_Data;
	}
	else
	{
		PCAx->CCR1.V32 = PCA_PWMInitStruct->DutyB_Value;
	}
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk); 
	PCAx->CCMR1.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);	

  PCA_REG_LOCK(PCAx);

}



/**
  * @brief  Initialize the PCAxC function according to 
  *         the specified parameters in the PCA_PWMInitStruct.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the PWM2 function.
  * @retval None
  */
void PCA_PCAxCPWMConfig(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;
	
	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->CCR2.BIT.CCR2L = PCA_PWMInitStruct->InitC_Data;
		PCAx->CCR2.BIT.CCR2H = PCA_PWMInitStruct->ReloadC_Data;
	}
	else
	{
		PCAx->CCR2.V32 = PCA_PWMInitStruct->DutyC_Value;
	}
	
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk); 
	PCAx->CCMR2.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);	
	
  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Sets the PCAx Counter Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  Counter: specifies the Counter register new value.
  * @retval None
  */
void PCA_SetCounter(PCA_TypeDef* PCAx, uint32_t Counter)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CNT.V32 = Counter;	

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Sets the PCAx Prescaler Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  Prescaler: specifies the Prescaler register new value.
  * @retval None
  */
void PCA_SetPrescaler(PCA_TypeDef* PCAx, uint32_t Prescaler)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->PSC = Prescaler;	

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Sets the PCAx Period Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  Period: specifies the Period register new value.
  * @retval None
  */
void PCA_SetPeriod(PCA_TypeDef* PCAx, uint32_t Period)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->PR.V32 = Period;
    
  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Get the PCAx Counter Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @retval The Counter Register value.
  */
uint32_t PCA_GetCounter(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CNT.V32;	
}

/**
  * @brief  Get the PCAx Prescaler Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @retval The Prescaler Register value.
  */
uint32_t PCA_GetPrescaler(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->PSC;	
}

/**
  * @brief  Get the PCAx Period Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @retval The Period Register value.
  */
uint32_t PCA_GetPeriod(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->PR.V32;	
}


/**
  * @brief  Set the PCAxA Duty Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  Duty: specifies the PCAxA Duty Register new value.
  * @retval None
  */
void PCA_SetCCR0(PCA_TypeDef* PCAx, uint32_t Duty)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CCR0.V32 = Duty;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAxB Duty Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  Duty: specifies the PCAxB Duty Register new value.
  * @retval None
  */
void PCA_SetCCR1(PCA_TypeDef* PCAx, uint32_t Duty)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CCR1.V32 = Duty;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAxC Duty Register value
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  Duty: specifies the PCAxC Duty Register new value.
  * @retval None
  */
void PCA_SetCCR2(PCA_TypeDef* PCAx, uint32_t Duty)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CCR2.V32 = Duty;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Get the PCAxA Duty Register value.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @retval The PCAxA Duty Register value.
  */
uint32_t PCA_GetCCR0(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CCR0.V32;
}

/**
  * @brief  Set the PCAxB Duty Register value.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @retval The PCAxB Duty Register value.
  */
uint32_t PCA_GetCCR1(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CCR1.V32;
}

/**
  * @brief  Get the PCAxC Duty Register value.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @retval The PCAxC Duty Register value.
  */
uint32_t PCA_GetCCR2(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CCR2.V32;
}



/**
  * @brief  Enable or Disable specified PCA interrupts.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  PCA_INT: specifie the PCA interrupts sources to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCA_INT_CIE: PCAx Counter/timer overflow interrupt enable
*            @arg PCA_INT_PIE: PCAx Counter/timer period interrupt enable
*            @arg PCA_INT_CCIE0: PCAxA Capture/Compare Interrupt Control
*            @arg PCA_INT_CCIE1: PCAxB Capture/Compare Interrupt Control
*            @arg PCA_INT_CCIE2: PCAxC Capture/Compare Interrupt Control

  * @param  NewState: new state of interrupt sources.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE: Be enable
*            @arg SH_DISABLE: Be disable
  * @retval None
  */
void PCA_INTConfig(PCA_TypeDef* PCAx, uint32_t PCA_INT, SH_FunctionalState NewState)
{
	/* Check the parameters */
    assert_param(IS_PCA_ALL_PERIPH(PCAx));
    assert_param(IS_PCA_INT(PCA_INT));
		assert_param(IS_FUNCTION_STATE(NewState));  

    PCA_REG_UNLOCK(PCAx);

    if (NewState != SH_DISABLE)
    {
        /* Enable the interrupt sources */
        if ((PCA_INT & PCA_INT_CIE) == PCA_INT_CIE)
        {
            PCAx->CFGR.V32 |= CFGR_CIE_EN;
        }
        if ((PCA_INT & PCA_INT_PIE) == PCA_INT_PIE)
        {
            PCAx->CFGR.V32 |= CFGR_PIE_EN;
        }	
        if ((PCA_INT & PCA_INT_CCIE0) == PCA_INT_CCIE0)
        {
            PCAx->CCMR0.V32 |= CCMR_CCIE_EN;
        }	
        if ((PCA_INT & PCA_INT_CCIE1) == PCA_INT_CCIE1)
        {
            PCAx->CCMR1.V32 |= CCMR_CCIE_EN;
        }		
        if ((PCA_INT & PCA_INT_CCIE2) == PCA_INT_CCIE2)
        {
            PCAx->CCMR2.V32 |= CCMR_CCIE_EN;
        }						
    }
    else
    {
        /* Disable the interrupt sources */
        if ((PCA_INT & PCA_INT_CIE) == PCA_INT_CIE)
        {
            PCAx->CFGR.V32 &= ~(PCA_CFGR_CIE_Msk);
        }
        if ((PCA_INT & PCA_INT_PIE) == PCA_INT_PIE)
        {
            PCAx->CFGR.V32 &= ~(PCA_CFGR_PIE_Msk);
        }	
				if ((PCA_INT & PCA_INT_CCIE0) == PCA_INT_CCIE0)
        {
            PCAx->CCMR0.V32 &= ~(PCA_CCMR0_CCIE_Msk);
        }	
        if ((PCA_INT & PCA_INT_CCIE1) == PCA_INT_CCIE1)
        {
            PCAx->CCMR1.V32 &= ~(PCA_CCMR1_CCIE_Msk);
        }		
        if ((PCA_INT & PCA_INT_CCIE2) == PCA_INT_CCIE2)
        {
            PCAx->CCMR2.V32 &= ~(PCA_CCMR2_CCIE_Msk);
        }		
    }
    PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the specified of PCA update moment.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  PCA_Update: specifie the update moment of PACx period/CCR2/CCR1/CCR0 to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCA_PUE:PACx period register update control bit
*            @arg PCA_CC0UE:PACxA Module update control bit
*            @arg PCA_CC1UE:PACxB Module update control bit
*            @arg PCA_CC2UE:PACxC Module update control bit

  * @param  NewState: new state of the update moment.
  *          This parameter can be one of the following values:
*            @arg PCA_UPDATE_NEXT_CIRCLE: be valid in the next cycle
*            @arg PCA_UPDATE_NOW: be valid immediately at the moment
  * @retval None
  */
void PCA_UpdateConfig(PCA_TypeDef* PCAx, uint32_t PCA_Update, SH_FunctionalState NewState)
{
	/* Check the parameters */
    assert_param(IS_PCA_ALL_PERIPH(PCAx));
    assert_param(IS_PCA_UPDATE(PCA_Update));

    PCA_REG_UNLOCK(PCAx);

    if (NewState != PCA_UPDATE_NEXT_CIRCLE)
    {
        if ((PCA_Update & PCA_PUE) == PCA_PUE)
        {
            PCAx->CFGR.V32 |= CFGR_PUE_EN;
        }
        if ((PCA_Update & PCA_CC0UE) == PCA_CC0UE)
        {
            PCAx->CFGR.V32 |= CFGR_CC0UE_EN;
        }	
        if ((PCA_Update & PCA_CC1UE) == PCA_CC1UE)
        {
            PCAx->CFGR.V32 |= CFGR_CC1UE_EN;
        }	
        if ((PCA_Update & PCA_CC2UE) == PCA_CC2UE)
        {
            PCAx->CFGR.V32 |= CFGR_CC2UE_EN;
        }		
				
    }
    else
    {
        if ((PCA_Update & PCA_PUE) == PCA_PUE)
        {
            PCAx->CFGR.V32 &= ~(PCA_CFGR_PUE_Msk);
        }
        if ((PCA_Update & PCA_CC0UE) == PCA_CC0UE)
        {
            PCAx->CFGR.V32 &= ~(PCA_CFGR_CC0UE_Msk);
        }	
				if ((PCA_Update & PCA_CC1UE) == PCA_CC1UE)
        {
            PCAx->CFGR.V32 &= ~(PCA_CFGR_CC1UE_Msk);
        }	
        if ((PCA_Update & PCA_CC2UE) == PCA_CC2UE)
        {
            PCAx->CFGR.V32 &= ~(PCA_CFGR_CC2UE_Msk);
        }		
	
    }
    PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Check whether the specified PCA flag is set or not
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  PCA_Flag: specifie the flag to check.
  *          This parameter can be one of the following values:
  *            @arg PCA_FLAG_CC0IF:PCAxA Capture/Compare Flag
  *            @arg PCA_FLAG_CC1IF:PCAxB Capture/Compare Flag
  *            @arg PCA_FLAG_CC2IF:PCAxC Capture/Compare Flag
  *            @arg PCA_FLAG_FLTIF:Fault Detection Interrupt Flag
  *            @arg PCA_FLAG_PIF:PCAx time base periodic matching interrupt flag bit (only valid in center aligned mode)
  *            @arg PCA_FLAG_CIF:PCA counter overflow interrupt flag bit

  * @retval None
  */
SH_FlagStatus PCA_GetFlagStatus(PCA_TypeDef* PCAx, uint32_t PCA_Flag)
{
    SH_FlagStatus bitStatus;
    
	/* Check the parameters */
    assert_param(IS_PCA_ALL_PERIPH(PCAx));
    assert_param(IS_PCA_GET_ONE_FLAG(PCA_Flag));

    if ((PCAx->SR.V32 & PCA_Flag) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @brief  Clear the PCAx's pending flags.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_Flag: specifie the flag bit to clear.
  *          This parameter can be any combination of the following values:
*            @arg PCA_FLAG_CC0IF:PCAxA Capture/Compare Flag
*            @arg PCA_FLAG_CC1IF:PCAxB Capture/Compare Flag
*            @arg PCA_FLAG_CC2IF:PCAxC Capture/Compare Flag
*            @arg PCA_FLAG_FLTIF:Fault Detection Interrupt Flag
*            @arg PCA_FLAG_PIF:PCAx time base periodic matching interrupt flag bit (only valid in center aligned mode)
*            @arg PCA_FLAG_CIF:PCAx counter overflow interrupt flag bit

  * @retval None
  */
void PCA_ClearFlag(PCA_TypeDef* PCAx, uint32_t PCA_Flag)
{
	/* Check the parameters */
    assert_param(IS_PCA_ALL_PERIPH(PCAx));
    assert_param(IS_PCA_GET_ONE_FLAG(PCA_Flag));

		PCA_REG_UNLOCK(PCAx);
		/* Disable the interrupt sources */
    if ((PCA_Flag & PCA_FLAG_CC0IF) == PCA_FLAG_CC0IF)
		{
				PCAx->SR.V32 |= PCA_SR_CC0IFC_Msk ;
		}
    if ((PCA_Flag & PCA_FLAG_CC1IF) == PCA_FLAG_CC1IF)
		{
				PCAx->SR.V32 |= PCA_SR_CC1IFC_Msk ;
		}	
    if ((PCA_Flag & PCA_FLAG_CC2IF) == PCA_FLAG_CC2IF)
		{
				PCAx->SR.V32 |= PCA_SR_CC2IFC_Msk ;
		}	
    if ((PCA_Flag & PCA_FLAG_FLTIF) == PCA_FLAG_FLTIF)
		{
				PCAx->SR.V32 |= PCA_SR_FLTIFC_Msk ;
		}	
    if ((PCA_Flag & PCA_FLAG_PIF) == PCA_FLAG_PIF)
		{
				PCAx->SR.V32 |= PCA_SR_PIFC_Msk ;
		}		
    if ((PCA_Flag & PCA_FLAG_CIF) == PCA_FLAG_CIF)
		{
				PCAx->SR.V32 |= PCA_SR_CIFC_Msk ;
		}			
		PCA_REG_LOCK(PCAx);

}


/**
  * @brief  Enable force manual output mode of the specified PWM port.
  * @param  PCAx: where x can be 0 or 1 to select the PCA peripheral.
  * @param  PCAxPin: specifie PWM port to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCAxA_FORCE:PCAxA force output setting bit
*            @arg PCAxB_FORCE:PCAxB force output setting bit
*            @arg PCAxC_FORCE:PCAxC force output setting bit
  * @retval None
  */
void PCA_SetForceOut(PCA_TypeDef* PCAx, uint8_t PCAxPin)
{
	/* Check the parameters */
    assert_param(IS_PCA_ALL_PERIPH(PCAx));
    assert_param(IS_PCA_FORCE_PIN(PCAxPin));

    PCA_REG_UNLOCK(PCAx);

    /* Enable manual output mode of specified PCA port */
    PCAx->FORCE.V32 |= (1<<PCAxPin); 

    PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Set the PCAx Trigger ADC Register value.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  Value: specifie Trigger ADC Register value.
  *          This parameter can be 0 to 0xFFFFFFFF.
  * @retval None
  */
void PCA_SetADCTriggerValue(PCA_TypeDef* PCAx, uint32_t Value)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->ADTR = Value;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAx clock source.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  Source: specifie the clock source.
  *            @arg CFGR_CPS_PCLK: PCLK
  *            @arg CFGR_CPS_ECI: PCAxECI Falling edge
  *            @arg CFGR_CPS_RC128K: RC128K
  * @retval None
  */
void PCA_SetPCAClock(PCA_TypeDef* PCAx, uint32_t Source)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CFGR.V32 &= ~(PCA_CFGR_CPS_Msk);
	PCAx->CFGR.V32 |= Source;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the filter time, when the PCAx clock source is PCAxECI.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  Filter: specifie the clock source.
*            @arg CFGR_ECF_NONE: No filter
*            @arg CFGR_ECF_8PCLK: Filter time constant is 8*PCLK
*            @arg CFGR_ECF_16PCLK: Filter time constant is 16*PCLK
*            @arg CFGR_ECF_32PCLK: Filter time constant is 32*PCLK
  * @retval None
  */
void PCA_SetPCAxECIFilter(PCA_TypeDef* PCAx, uint32_t Filter)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_ECI_FILTER(Filter));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CFGR.V32 &= ~(PCA_CFGR_ECF_Msk);
	PCAx->CFGR.V32 |= Filter;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the synchronization mode of PCAx.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  SYN_Source: specifie the synchronization source.
*            @arg CFGR_SYN_NONE: No synchronization
*            @arg CFGR_SYN_PCA: Be set or cleared by the other PCA synchronously.
*            @arg CFGR_SYN_MCM: Be set or cleared by MCM synchronously.
  * @retval None
  */
void PCA_SetSYNSource(PCA_TypeDef* PCAx, uint32_t SYN_Source)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CFGR.V32 &= ~(PCA_CFGR_SYN_Msk);
	PCAx->CFGR.V32 |= SYN_Source;

  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Configure the PCAx ADC trigger timing.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_ADCTrigger: PCAx trigger ADC selection
*            @arg PCA_ADC_TRIGGER_None:No trigger
*            @arg PCA_ADC_TRIGGER_Up: Trigger ADC when counter is incremented
*            @arg PCA_ADC_TRIGGER_Down: Trigger ADC when counter is decremented
*            @arg PCA_ADC_TRIGGER_UpandDown: Trigger ADC when counter is incremented or decremented
  * @retval None
  */
void PCA_ADCTriggerConfig(PCA_TypeDef* PCAx, uint32_t PCA_ADCTrigger)
{
	uint32_t temp = 0;
	
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	temp = PCAx->CFGR.V32;
	
	temp &= ~(PCA_CFGR_ADTSEL_Msk);
	temp |= PCA_ADCTrigger;
	
	PCAx->CFGR.V32 = temp;

  PCA_REG_LOCK(PCAx);
}




/**
  * @brief  Fills each PCA_FLTInitStruct member with its default value.
  * @param  PCA_FLTInitStruct : pointer to a @ref PCA_FLTInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_FLTStructInit(PCA_FLTInitTypeDef* PCA_FLTInitStruct)
{
    PCA_FLTInitStruct->FLT_Mode = FLTCR_FLTM_TIME_MODE;
    PCA_FLTInitStruct->FLT_Int = FLTCR_FLTIE_DIS;
    PCA_FLTInitStruct->FLT_OutputState = FLTCR_FOUT_HIZ;
    PCA_FLTInitStruct->FLT_Syn = FLTCR_FLTSYN_EN;
	
    PCA_FLTInitStruct->FLT1_InputSel = FLTCR_FLT1SEL_CMP1;
    PCA_FLTInitStruct->FLT1_OnOff = FLTCR_FLT1EN_ON;
	
    PCA_FLTInitStruct->FLT2_Filter = FLTCR_FLT2DEB_8US;
    PCA_FLTInitStruct->FLT2_ActiveLevel = FLTCR_FLT2S_HIGH_LEVEL;
    PCA_FLTInitStruct->FLT2_OnOff = FLTCR_FLT2EN_ON;
}


/**
  * @brief  Initialize the PCAx Fault module according to 
  *         the specified parameters in the PCA_FLTInitStruct.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_FLTInitStruct: pointer to a @ref PCA_FLTInitTypeDef structure
  *         that contains the configuration information for the Fault module.
  * @retval None
  */
void PCA_FLTInit(PCA_TypeDef* PCAx, PCA_FLTInitTypeDef* PCA_FLTInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_FLT_REG_UNLOCK(PCAx);
	PCAx->FLTCR.V32 &= ~(PCA_FLTCR_FLT2DEB_Msk 
											| PCA_FLTCR_FLT2S_Msk 
											| PCA_FLTCR_FLT1SEL_Msk 
											| PCA_FLTCR_FLTM_Msk 
											| PCA_FLTCR_FLTIE_Msk 
											| PCA_FLTCR_FOUT_Msk 
											| PCA_FLTCR_FLTSYN_Msk);

	PCAx->FLTCR.V32 = (PCA_FLTInitStruct->FLT2_Filter 
										|PCA_FLTInitStruct->FLT2_ActiveLevel
										|PCA_FLTInitStruct->FLT1_InputSel
										|PCA_FLTInitStruct->FLT_Mode
										|PCA_FLTInitStruct->FLT_Int
										|PCA_FLTInitStruct->FLT_OutputState
										|PCA_FLTInitStruct->FLT_Syn);

  PCA_FLT_REG_LOCK(PCAx);
}


/**
  * @brief  Enable or disable FLT module registers write protection.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  NewState: new state of FLT write protection.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE: Enable the FLT register write protection
*            @arg SH_DISABLE: Disable the FLT register write protection
  * @retval None
  */
void PCA_FLTWriteLock(PCA_TypeDef* PCAx, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_FUNCTION_STATE(NewState));
	
	if (NewState != SH_DISABLE)
	{
        /* Enable the FLT register write protection */
		PCAx->FLTWPR = 0x33CC;
	}
	else
	{
        /* Disable the FLT register write protection */
		PCAx->FLTWPR = 0x0000;
	}
}


/**
  * @brief  Configure the PCAx FLT mode and output pin status.  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_FLT_STATE_Mode: specifie the FLT detect mode to set.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTM_LOCK_MODE: Latch mode
*            @arg FLTCR_FLTM_TIME_MODE: Sequential mode
  * @param  PCA_FLT_Int: specifie the control bit of the FLT Interrupt.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTIE_DIS: Fault detection interrupt disables.
*            @arg FLTCR_FLTIE_EN:  Fault detection interrupt enables.
  * @param  PCA_FLT_OutputLevel: specifie the output status of the FLT pin.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FOUT_HIZ:  Outputs high resistance state
*            @arg FLTCR_FOUT_LOW:  Outputs low level
*            @arg FLTCR_FOUT_HIGH: Outputs high level
  * @param  PCA_FLT_Syn: specifie the output status of the FLT PWMx1 pin.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTSYN_DIS:Fault detection synchronous disables
*            @arg FLTCR_FLTSYN_EN: Fault detection synchronous enables
  * @retval None
  */
void PCA_FLTConfig(PCA_TypeDef* PCAx, uint32_t PCA_FLT_STATE_Mode, uint32_t PCA_FLT_Int, uint32_t PCA_FLT_OutputLevel, uint32_t PCA_FLT_Syn)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_FLT_STATE_MODE(PCA_FLT_STATE_Mode));
	assert_param(IS_PCA_FLT_OUTPUT_LEVEL(PCA_FLT_OutputLevel));
	assert_param(IS_PCA_FLT_INT(PCA_FLT_Int));
	assert_param(IS_PCA_FLT_SYS(PCA_FLT_Syn));
	

  PCA_FLT_REG_UNLOCK(PCAx);
	PCAx->FLTCR.V32 &= ~(PCA_FLTCR_FLTM_Msk | PCA_FLTCR_FLTIE_Msk | PCA_FLTCR_FOUT_Msk | PCA_FLTCR_FLTSYN_Msk);	
	PCAx->FLTCR.V32 |= (PCA_FLT_STATE_Mode | PCA_FLT_Int | PCA_FLT_OutputLevel | PCA_FLT_Syn);	
  PCA_FLT_REG_LOCK(PCAx);
}

/**
  * @brief  Configure input source of the PCAx FLT1 module.   
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_FLTSource: specifie the FLT1 input sources.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLT1SEL_CMP0: CMP0 is used as input source of PCA fault detection 1, high level valid
*            @arg FLTCR_FLT1SEL_CMP1: CMP1 is used as input source of PCA fault detection 1, high level valid
  * @retval None
  */
void PCA_FLT1Config(PCA_TypeDef* PCAx, uint32_t PCA_FLTSource)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_FLT1_SOURCE(PCA_FLTSource));

  PCA_FLT_REG_UNLOCK(PCAx);

	/* Reset FLT1SEL bits */
	PCAx->FLTCR.V32 &= ~(PCA_FLTCR_FLT1SEL_Msk);
	PCAx->FLTCR.V32 |= PCA_FLTSource;

  PCA_FLT_REG_LOCK(PCAx);

}

/**
  * @brief  Configure filter and active level of the PCAx FLT2 module.   
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  PCA_FLT2Filter: specifie the FLT2 filter value.
  *          This parameter can be one of the following values:    
*            @arg FLTCR_FLT2DEB_0US:	filter time = 0us
*            @arg FLTCR_FLT2DEB_0P5US:filter time = 0.5us	
*            @arg FLTCR_FLT2DEB_1US:	filter time = 1us	
*            @arg FLTCR_FLT2DEB_1P5US:filter time = 1.5us	
*            @arg FLTCR_FLT2DEB_2US:	filter time = 2us	
*            @arg FLTCR_FLT2DEB_3US:	filter time = 3us
*            @arg FLTCR_FLT2DEB_4US:	filter time = 4us		
*            @arg FLTCR_FLT2DEB_6US:	filter time = 6us		
*            @arg FLTCR_FLT2DEB_8US:	filter time = 8us		
*            @arg FLTCR_FLT2DEB_10US:	filter time = 10us	
*            @arg FLTCR_FLT2DEB_12US:	filter time = 12us	
*            @arg FLTCR_FLT2DEB_14US:	filter time = 14us
*            @arg FLTCR_FLT2DEB_16US:	filter time = 16us	
*            @arg FLTCR_FLT2DEB_20US:	filter time = 20us	
*            @arg FLTCR_FLT2DEB_24US:	filter time = 24us	
*            @arg FLTCR_FLT2DEB_32US: filter time = 32us
  * @param  PCA_FLT2ActiveLevel: specifie the FLT2 active level.
  *          This parameter can be one of the following values:
  *            @arg FLTCR_FLT2S_HIGH_LEVEL: Fault detection 2 FLT input is valid on high level
  *            @arg FLTCR_FLT2S_LOW_LEVEL: Fault detection 2 FLT input is valid on low level
  * @retval None
  */
void PCA_FLT2Config(PCA_TypeDef* PCAx, uint32_t PCA_FLT2Filter, uint32_t PCA_FLT2ActiveLevel)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_FLT2_FILTER(PCA_FLT2Filter));	
	assert_param(IS_PCA_FLT2_ACTIVE_LEVEL(PCA_FLT2ActiveLevel));

  PCA_FLT_REG_UNLOCK(PCAx);

	PCAx->FLTCR.V32 &= ~(PCA_FLTCR_FLT2DEB_Msk | PCA_FLTCR_FLT2S_Msk);
	PCAx->FLTCR.V32 |= (PCA_FLT2Filter | PCA_FLT2ActiveLevel);

  PCA_FLT_REG_LOCK(PCAx);
}


/**
  * @brief  Clear the PCAx's FLT pending flag.
  * @note   This function only can be called in Latch mode of FLT,because this bit will be
  *         set by hardware in Each Mode.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @retval None
  */
void PCA_ClearFLTStatusFlag(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_FLT_REG_UNLOCK(PCAx);

	PCAx->FLTCR.V32 &= ~PCA_FLTCR_FLTSTAT_Msk;

  PCA_FLT_REG_LOCK(PCAx);
}

/**
  * @brief  Check whether the specified PCA FLT status flag is set or not  
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @retval None
  */
SH_FlagStatus PCA_GetFLTStatus(PCA_TypeDef* PCAx)
{
	SH_FlagStatus bitStatus;
	
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	if ((PCAx->FLTCR.V32 & PCA_FLTCR_FLTSTAT_Msk) != SH_RESET)
	{
		bitStatus = SH_SET;
	}
	else
	{
		bitStatus = SH_RESET;
	}
	
	return bitStatus;
}



/**
  * @brief  Enable or disable the PCAx's Fault module.
  * @param  PCAx: where x can be  0 or 1 to select the PCA peripheral.
  * @param  FLTx: specifie which FLTx module to be selected.
  *          This parameter can be any combination of the following values:
*            @arg PCA_FLT1: Fault detection 1 function
*            @arg PCA_FLT2: Fault detection 2 function
  * @param  OnOffState: state of the fault module.
  *          This parameter can be one of the following values:
*            @arg SH_ON: Enable the function
*            @arg SH_OFF: Disable the function
  * @retval None
  */
void PCA_FLTOnOff(PCA_TypeDef* PCAx, uint32_t FLTx, SH_CmdState OnOffState)
{    
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_CMD_STATE(OnOffState));  

  PCA_FLT_REG_UNLOCK(PCAx);

	if (OnOffState != SH_OFF)
	{
		if ((FLTx & PCA_FLT1) == PCA_FLT1)
		{
			PCAx->FLTCR.V32 |= PCA_FLTCR_FLT1EN_Msk;
		}
		if ((FLTx & PCA_FLT2) == PCA_FLT2)
		{
			PCAx->FLTCR.V32 |= PCA_FLTCR_FLT2EN_Msk;
		}
	}
	else
	{
		if ((FLTx & PCA_FLT1) == PCA_FLT1)
		{
			PCAx->FLTCR.V32 &= ~PCA_FLTCR_FLT1EN_Msk;
		}
		if ((FLTx & PCA_FLT2) == PCA_FLT2)
		{
			PCAx->FLTCR.V32 &= ~PCA_FLTCR_FLT2EN_Msk;
		}
	}

  PCA_FLT_REG_LOCK(PCAx);
}


/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/

