/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_rcc.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides reset and clock module's APIs
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *          
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_rcc.h"
#ifndef HSI_CLK
#define HSI_CLK 8000000
#endif
#ifndef HSE_CLK
#define HSE_CLK 8000000
#endif
      
/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/*  reset and clock control module-----------------------------------------------------------*/
/** @defgroup RCC_MODULE  RCC 
   *  reset and clock control module(RCC)
  * @{
  */ 
        
/** @defgroup  RCC_Group_Pub_Funcs  Public Functions
 *  @brief   RCC public functions
 *
  * @{
  */
/**
  *@code   Example    
  *     RCC_ClocksTypeDef rccClocks;
  *     RCC_GetClocksFreq(&rccClocks);
   *@endcode
   */
/**
  * @brief  get current system's clock frequency
  * @param  Clocks: output clock frequency
  * @retval None
  */
void RCC_GetClocksFreq(RCC_Clocks_TypeDef* Clocks)
{
    uint32_t sysClk=0;
    if(RCC->CR.BIT.SW == RCC_SYS_SRC_HSI)
    {/*  clock source is HSI */
          sysClk = HSI_CLK;
    }
    else if(RCC->CR.BIT.SW == RCC_SYS_SRC_HSE)
    {/*  clock source is HSE */
          sysClk = HSE_CLK;
    }
    else if(RCC->CR.BIT.SW == RCC_SYS_SRC_PLL)
    {/*  clock source is PLL */
          if(RCC->CFGR.BIT.PLLSRC == PLL_SRC_HSI)  
          {
                sysClk = HSI_CLK;
          }
          else if(RCC->CFGR.BIT.PLLSRC == PLL_SRC_HSE)
          {
                sysClk = HSE_CLK;
          }
          else
          {
              assert_param(0);
          }
          sysClk /= (RCC->CFGR.BIT.PLLXTPRE+1);
          sysClk *= (RCC->CFGR.BIT.PLLF+15);
          sysClk /= (RCC->CFGR.BIT.PLLK+1);
          /* fixed divider 2 */
          sysClk >>=1; 
    }
    else
    {
        assert_param(0);
    }
    
     Clocks->sysFreq = sysClk;
     sysClk >>= RCC->CFGR.BIT.HPRE;
     Clocks->hclkFreq = sysClk;
     Clocks->pclkFreq = sysClk >> RCC->CFGR.BIT.PPRE;
}
  
/**
  * @brief    restore default clocks
  * @retval None
  */
void RCC_Reset(void)
{
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    /* switch to  HSI*/
    RCC->CR.BIT.SW = RCC_SYS_SRC_HSI;
    
    /* wait switch ready */
    while(RCC->CR.BIT.SW != RCC_SYS_SRC_HSI);
    
    /*  close HSE and PLL */
    RCC->CR.V32 = 0;
    
    /*  clear  CFGR  */
    RCC->CFGR.V32 =  0; 

    /*  clear interrupt enable bits */
    RCC->CIENR.V32 = 0;
    
    /* clear interrupt flags */
    RCC->CICLR.V32 = 0xFFFFFFFF;

    /* clear clock gates */
    RCC->AHBENR.V32  = 0;
    RCC->APB0ENR.V32 = 0;

    /* clear reset flags */
    RCC->RSTCLR.V32  = 0xFFFFFFFF;

    /* clear RC Trim */
    RCC->HSICAL.V32  = 0;
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief    configure HSE (external clock)
  * @param OnOffState   open or close the HSE
  *     @arg @b SH_ON    open HSE
  *     @arg @b SH_OFF   close HSE
  * @retval None
  */
void RCC_HSEOnOff(SH_CmdState OnOffState)
{
    assert_param(IS_CMD_STATE(OnOffState));
       
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CR.BIT.HSEON = OnOffState;

   /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief    wait for HSE ready
  * @param TimeOut   timeout ticks
  * @retval SH_ErrorStatus 
  *      @arg @b SH_SUCCESS  HSE is ready 
  *      @arg @b SH_ERROR     timeout
  */
SH_ErrorStatus RCC_WaitForHSEReady(uint32_t TimeOut)
{
    assert_param(TimeOut > 0);
    do{
        if(RCC->CR.BIT.HSERDY)
            break;
        TimeOut--;
    }while(TimeOut > 0);
    return (TimeOut==0?SH_ERROR:SH_SUCCESS);
}

/**
  * @code   Example : HSI increase 0.25%
       RCC_AdjustHSITrimValue(RCC_HSITRIM_I25);    
  * @endcode 
  *@brief   adjust HSI frequency
  *@param HSITrimValue  trim value
  *     @arg @b RCC_HSITRIM_I00  ( + 0.0% )
  *     @arg @b RCC_HSITRIM_I25  ( + 0.25% )
  *     @arg @b RCC_HSITRIM_I50  ( + 0.50% )
  *     @arg @b RCC_HSITRIM_I75  ( + 0.75% )
  *     @arg @b RCC_HSITRIM_D100 (- 1% )
  *     @arg @b RCC_HSITRIM_D75  (- 0.75%  )
  *     @arg @b RCC_HSITRIM_D50  (- 0.50% )
  *     @arg @b RCC_HSITRIM_D25  (- 0.25% )
  *@retval  Trim reference value
  */
uint32_t RCC_AdjustHSITrimValue(RCC_HSITRIM_Type HSITrimValue)
{
    assert_param(IS_HSITRIM(HSITrimValue));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    /* set trim value*/
    RCC->HSICAL.BIT.HSITRIM = HSITrimValue;
    
    /* start  */
    RCC->HSICAL.BIT.TRIMRUN = SH_SET;
    
    while(RCC->HSICAL.BIT.TRIMRUN == SH_SET);
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();    
	
    return RCC->HSICAL.BIT.TRIMREF;
}

/**
  * @brief  Fill the RCC_PLL_InitStruct member with its default value.
  * @param  PLLInit : pointer to a @ref RCC_PLL_InitTypeDef structure which will be initialized.
  * @retval None
  */
void RCC_PLLStructInit(RCC_PLL_InitTypeDef* PLLInit)
{
    /* Reset GPIO init structure parameters values */
    PLLInit->xpreDiv = PLL_XTPRE_DIV2;
    PLLInit->pllSRC  = PLL_SRC_HSI;
    PLLInit->pllMul  = 42;			
    PLLInit->pllDiv  = 1;		// (8/2)*42/1/2=84MHz
}

/**
  * @code   Example
       // config PLLHSI is 8MHz and PLL's output is 100MHz
       // 1. PLL source is HSI     ;can be PLL_SRC_HSI or PLL_SRC_HSE
       // 2. PLL pre-divider is 1  ;can be PLL_XTPRE_DIV1 or PLL_XTPRE_DIV2
       // 3. PLL F = 25            ;can be 15~78
       // 4. PLL K = 1             ;can be 1~16
       // 5. PLL output = HSI(8MHz)/pre-divider(1) * F(25)/K(1)/Fixed(2) = 100MHz
       RCC_PllInitTypeDef pllInit;
       pllInit.pllSRC    = PLL_SRC_HSI;  
       pllInit.xpreDiv   = PLL_XTPRE_DIV1; 
       pllInit.pllMul    = 25; 
       pllInit.pllDiv    = 1;  
  * @endcode 
  */
/**
  * @brief             configure PLL
  * @param PLLInit  PLL parameters @ref RCC_PLL_InitTypeDef
  * @retval None
  */
void RCC_PLLConfig(RCC_PLL_InitTypeDef* PLLInit)
{
    assert_param(IS_PLL_XTPRE(PLLInit->xpreDiv));
    assert_param(IS_PLL_SRC(PLLInit->pllSRC));
    assert_param((PLLInit->pllMul >= 15 && PLLInit->pllMul <= 78));
    assert_param((PLLInit->pllDiv >= 1 && PLLInit->pllDiv <= 16));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->CFGR.BIT.PLLXTPRE = PLLInit->xpreDiv;
    RCC->CFGR.BIT.PLLSRC   = PLLInit->pllSRC;
    RCC->CFGR.BIT.PLLF     = PLL_PLLF(PLLInit->pllMul);
    RCC->CFGR.BIT.PLLK     = PLL_PLLK(PLLInit->pllDiv);
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();           
}

/**
  *@brief   open or close PLL
  *@param OnOffState  open or close PLL
  *     @arg @b SH_ON   open PLL
  *     @arg @b SH_OFF  close PLL
  *@retval None
  */
void RCC_PLLOnOff(SH_CmdState OnOffState)
{
    assert_param(IS_CMD_STATE(OnOffState));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->CR.BIT.PLLON = OnOffState;
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();        
}

/**
  * @brief    wait for PLL ready
  * @param    TimeOut   timeout ticks
  * @retval   SH_ErrorStatus 
  *      @arg @b SH_SUCCESS  PLL is ready 
  *      @arg @b SH_ERROR     timeout
  */
SH_ErrorStatus RCC_WaitForPLLReady(uint32_t TimeOut)
{
    assert_param(TimeOut > 0);
    do{
        if(RCC->CR.BIT.PLLRDY)
            break;
        TimeOut--;
    }while(TimeOut > 0);
    return (TimeOut==0?SH_ERROR:SH_SUCCESS);
}

/**
  *@brief    configure system clock
  *@param SysClkSrc  assign system clock source
  *     @arg @b RCC_SYS_SRC_HSI   
  *     @arg @b RCC_SYS_SRC_HSE  
  *     @arg @b RCC_SYS_SRC_PLL   
  * @retval None
  */
void RCC_SysClkConfig(RCC_SysSource_Type SysClkSrc)
{    
    volatile uint8_t tmpSysClk;
    assert_param(IS_SYS_SOURCE(SysClkSrc));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CR.BIT.SW = SysClkSrc;
    __DSB();
    
    /* wait switching ready */
    do{
        tmpSysClk = RCC->CR.BIT.SWS;
    }
    while(RCC->CR.BIT.SW != tmpSysClk );
 
    /* Lock RCC registers */
    RCC_REGS_LOCK();            
}

/**
  * @brief    get current system clock source 
  * @retval RCC_SysSource_Type current system clock source 
  *     @arg @b RCC_SYS_SRC_HSI  
  *     @arg @b RCC_SYS_SRC_HSE 
  *     @arg @b RCC_SYS_SRC_PLL  
  */
RCC_SysSource_Type RCC_GetSysClkSource(void)
{
    return ((RCC_SysSource_Type)RCC->CR.BIT.SWS);
}

/**
  *@brief    configure HCLK
  *@param  HCLK_DIV  system clock's divider
  *     @arg @b  RCC_HCLK_DIV1   system clock / 1
  *     @arg @b  RCC_HCLK_DIV2   system clock / 2
  *     @arg @b  RCC_HCLK_DIV4   system clock / 4
  *     @arg @b  RCC_HCLK_DIV8   system clock / 8
  *     @arg @b  RCC_HCLK_DIV16  system clock / 16
  *     @arg @b  RCC_HCLK_DIV32  system clock / 32
  *@retval  None
  */
void RCC_HCLKConfig(RCC_HCLK_DIV_Type HCLK_DIV)
{
    assert_param(IS_HCLK_DIV(HCLK_DIV));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CFGR.BIT.HPRE = HCLK_DIV;
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();               
}

/**
  *@brief    configure PCLK
  *@param  PCLK_DIV  HCLK's divider
  *     @arg @b  RCC_PCLK_DIV1   HCLK / 1
  *     @arg @b  RCC_PCLK_DIV2   HCLK / 2
  *     @arg @b  RCC_PCLK_DIV4   HCLK / 4
  *     @arg @b  RCC_PCLK_DIV8   HCLK / 8
  *     @arg @b  RCC_PCLK_DIV16  HCLK / 16
  *@retval  None
  */
void RCC_PCLKConfig(RCC_PCLK_DIV_Type PCLK_DIV)
{
    assert_param(IS_PCLK_DIV(PCLK_DIV));
   
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CFGR.BIT.PPRE = PCLK_DIV;
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();               
}



/**
 * @brief    configure interrupt enable or disable
  *@param  INT_SRC  interrupt source: can be one or more.
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_INT_HSERDY: HSE Ready Interrupt Enable
  *     @arg @b  RCC_INT_PLLRDY: PLL Ready Interrupt Enable   
  *@param  NewState enable or disable interrupt
  *     @arg @b  SH_ENABLE  
  *     @arg @b  SH_DISABLE
  *@retval  None
   */
void RCC_INTConfig(uint32_t INT_SRC, SH_FunctionalState NewState)
{
    assert_param(IS_RCC_INTSRCS(INT_SRC));
    assert_param(IS_FUNCTION_STATE(NewState));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    if(NewState == SH_ENABLE)
        RCC->CIENR.V32 |= INT_SRC;
    else
        RCC->CIENR.V32 &= ~INT_SRC;      
   
    /* Lock RCC registers */
    RCC_REGS_LOCK();         

}


/**
 * @brief    Get RCC interrupt flag
  *@param  INTFlag  interrupt flag
  *   This parameter can be one of following values:
  *     @arg @b  RCC_INT_HSERDY:  HSE ready interrupt flag
  *     @arg @b  RCC_INT_PLLRDY   PLL ready interrupt flag
  *     @arg @b  RCC_INT_CSMHSE:  HSE clock abnormal flag
  *     @arg @b  RCC_INT_CSMPLL:  PLL clock abnormal flag
  *@retval  SH_FlagStatus RCC Interrupt status
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus  RCC_GetINTStatus(RCC_INT_Type INTFlag)
{
    assert_param(IS_RCC_INTFLAG(INTFlag));
    
    return ((SH_FlagStatus)((RCC->CISTR.V32 & INTFlag) ? SH_SET : SH_RESET));
}


/**
 * @brief  clear RCC interrupt flag
  *@param  INTFlag  interrupt source, can select more than one
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_INT_HSERDY   HSE ready interrupt flag
  *     @arg @b  RCC_INT_PLLRDY   PLL ready interrupt flag
  *     @arg @b  RCC_INT_CSMHSE   CSM  abnormal flag
  *     @arg @b  RCC_INT_CSMPLL   CSM  abnormal flag
  *@retval  None
   */
void  RCC_ClearINTStatus(uint32_t INTFlag)
{
    assert_param(IS_RCC_INTFLAGS(INTFlag));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    /* CSMC bit for CSMHSE & CSMPLL */
    if((INTFlag & RCC_INT_CSMHSE) == RCC_INT_CSMHSE)
        INTFlag |= RCC_INT_CSMPLL;        
    
    RCC->CICLR.V32 = INTFlag;    
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();            
}

    
/**
  * @code   Example : open ADC1 and ADC2 module's clock
        RCC_AHBPeriphClockOnOff(RCC_AHB_ADC1 | RCC_AHB_ADC2, SH_ON);
  * @endcode 
  */
/**
  * @brief     open or close AHB modules' clock gate
  * @param  AHBModules:  AHB module bits @ref RCC_AHB_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_AHB_GPIO: for GPIO Module: Includes Config and IO
  *     @arg @b  RCC_AHB_MCM:  for MCM Module
  *     @arg @b  RCC_AHB_SYSCFG: for System Config Module
  *     @arg @b  RCC_AHB_MACP: for MACP Module
  *     @arg @b  RCC_AHB_CRC:  for CRC Module
  *@param OnOffState  open or close related clock gate
  *     @arg @b  SH_ON  open related clock gate
  *     @arg @b  SH_OFF  close related clock gate
  * @retval   None
   */ 
void  RCC_AHBPeriphClockOnOff(uint32_t AHBModules,SH_CmdState OnOffState)
{
    assert_param(IS_AHB_MODULES(AHBModules));
    assert_param(IS_CMD_STATE(OnOffState));
    
    /* Unlock RCC registers */
	RCC_REGS_UNLOCK();

    if(OnOffState == SH_ON)
    {
        RCC->AHBENR.V32 |= AHBModules;	
    }
    else
    {   
        uint32_t tmpV = RCC->AHBENR.V32;
        tmpV &= ~AHBModules;
        RCC->AHBENR.V32 = tmpV;	
    }

    /* Lock RCC registers */
	RCC_REGS_LOCK();   
}


/**
  * @brief  open or close APB0 modules' clock gate
  * @param  APB0Modules:  APB0 module bits @ref RCC_APB0_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_APB0_TIM7
  *     @arg @b  RCC_APB0_TIM8
  *     @arg @b  RCC_APB0_QEI
  *     @arg @b  RCC_APB0_UART0
  *     @arg @b  RCC_APB0_UART1
  *     @arg @b  RCC_APB0_UART2
  *     @arg @b  RCC_APB0_SPI0
  *     @arg @b  RCC_APB0_TWI0
  *     @arg @b  RCC_APB0_WWDT
  *     @arg @b  RCC_APB0_AMOC
  *     @arg @b  RCC_APB0_PCA0
  *     @arg @b  RCC_APB0_PCA1
  *     @arg @b  RCC_APB0_CAN0
  *     @arg @b  RCC_APB0_FIFO0
  *     @arg @b  RCC_APB0_FIFO1
  *     @arg @b  RCC_APB0_FIFO2
  *@param OnOffState  open or close related clock gate
  *     @arg @b  SH_ON  open related clock gate
  *     @arg @b  SH_OFF  close related clock gate
  * @retval  None
  */
void  RCC_APB0PeriphClockOnOff(uint32_t APB0Modules,SH_CmdState OnOffState)
{
    assert_param(IS_APB0_MODULES(APB0Modules));
    assert_param(IS_CMD_STATE(OnOffState));
    
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();

    if(OnOffState == SH_ON)
    {
        RCC->APB0ENR.V32 |= APB0Modules;	
    }
    else
    {
        RCC->APB0ENR.V32 &= ~APB0Modules;	
    }

   /* Lock RCC registers */
	RCC_REGS_LOCK();   
}

/**
  * @code   Example : reset CRC and MACP module
        RCC_AHBPeriphReset(RCC_AHB_CRC|RCC_AHB_MACP);
  * @endcode 
  */
/**
  * @brief     reset AHB modules
  * @param  AHBModules: AHB module bits @ref RCC_AHB_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_AHB_GPIO: for GPIO Module: Includes Config and IO
  *     @arg @b  RCC_AHB_MCM:  for MCM Module
  *     @arg @b  RCC_AHB_SYSCFG: for System Config Module
  *     @arg @b  RCC_AHB_MACP: for MACP Module
  *     @arg @b  RCC_AHB_CRC:  for CRC Module
  * @retval   None
   */ 
void RCC_AHBPeriphReset(uint32_t AHBModules)
{
    assert_param(IS_AHB_MODULES(AHBModules));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->AHBRSTR.V32 = AHBModules;

    /* Lock RCC registers */
    RCC_REGS_LOCK();   
}


/**
  * @brief    reset APB0 modules
  * @param  APB0Modules:  APB0 module bits @ref RCC_APB0_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_APB0_TIM7
  *     @arg @b  RCC_APB0_TIM8
  *     @arg @b  RCC_APB0_QEI
  *     @arg @b  RCC_APB0_UART0
  *     @arg @b  RCC_APB0_UART1
  *     @arg @b  RCC_APB0_UART2
  *     @arg @b  RCC_APB0_SPI0
  *     @arg @b  RCC_APB0_TWI0
  *     @arg @b  RCC_APB0_WWDT
  *     @arg @b  RCC_APB0_AMOC
  *     @arg @b  RCC_APB0_PCA0
  *     @arg @b  RCC_APB0_PCA1
  *     @arg @b  RCC_APB0_CAN0
  *     @arg @b  RCC_APB0_FIFO0
  *     @arg @b  RCC_APB0_FIFO1
  *     @arg @b  RCC_APB0_FIFO2
  * @retval  None
  */
void RCC_APB0PeriphReset(uint32_t APB0Modules)
{
    assert_param(IS_APB0_MODULES(APB0Modules));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->APB0RSTR.V32 = APB0Modules;

    /* Lock RCC registers */
    RCC_REGS_LOCK();       
}

/**
  * @brief     Get RCC reset flag
  * @param  ResetFlag:   RCC reset flag mask
  *     @arg @b  RCC_RST_PIN
  *     @arg @b  RCC_RST_LVR
  *     @arg @b  RCC_RST_POWERON
  *     @arg @b  RCC_RST_SOFTWARE
  *     @arg @b  RCC_RST_IWDT
  *     @arg @b  RCC_RST_WWDT
  * @retval  SH_FlagStatus RCC reset flag
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus RCC_GetResetFlag(RCC_RESET_Type ResetFlag)
{
    assert_param(IS_RCC_RESET_Type(ResetFlag));

    return ((RCC->RSTSTR.V32 & ResetFlag) ? SH_SET : SH_RESET);
}


/**
  * @brief  clear RCC reset flags
  * @param  ResetFlags:   RCC reset flag clear bits
  *     @arg @b  RCC_RST_PIN
  *     @arg @b  RCC_RST_LVR
  *     @arg @b  RCC_RST_POWERON
  *     @arg @b  RCC_RST_SOFTWARE
  *     @arg @b  RCC_RST_IWDT
  *     @arg @b  RCC_RST_WWDT
   * @retval None
  */
void RCC_ClearResetFlag(uint8_t ResetFlags)
{
    assert_param(IS_RCC_RESET_Types(ResetFlags));

    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();

    RCC->RSTCLR.V32 = ResetFlags;
        
   /* Lock RCC registers */
	RCC_REGS_LOCK();           
}


/**
  * @brief  Enable or Disable CSM function
  * @param  OnOffState: New CSM function status
  *     @arg @b SH_ON
  *     @arg @b SH_OFF
  * @retval None
  */
void RCC_CSMOnOff(SH_CmdState OnOffState)
{
    if(OnOffState == SH_ON)
        RCC->CR.BIT.CSMON = 1;
    else
        RCC->CR.BIT.CSMON = 0;
}

/**
  * @brief  Get CSM flag status
  * @param  CSM_Flag: CSM source
  *     @arg @b  RCC_INT_CSMHSE
  *     @arg @b  RCC_INT_CSMPLL
  * @retval  SH_FlagStatus CSM flag status
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus RCC_CSMGetFlagStatus(uint32_t CSM_Flag)
{
    assert_param(CSM_Flag == RCC_INT_CSMHSE || CSM_Flag == RCC_INT_CSMPLL);
    
    if( RCC->CISTR.V32 & CSM_Flag)
        return SH_SET;
    else
        return SH_RESET;
    
}

/**
  * @brief  clear CSM flags
  * @retval None
  */
void RCC_CSMClearFlag(void)
{
    RCC->CICLR.BIT.CSMC = 1;
}


/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


