/**
  ******************************************************************************
  * @file    sh3xf2xx_sb0_uart.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sb0_uart.h"
#include "sh3xf2xx_sb0_rcc.h"

/** @addtogroup SH3xF2xx_sb0_StdLib_Driver
  * @{
  */

/** @defgroup UART_MODULE UART
  * @brief UART driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup  UART_Public_Functions Public Functions
 *  @brief   UART Public Functions
 *
  * @{
  */

/** @defgroup UART_Group1 Initialization and Configuration
 *  @brief    Initialization and Configuration
 *
@verbatim    
 ===============================================================================
                      Initialization and Configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */

/**
  * @brief  Reset the UARTx peripheral registers to their default reset values.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @retval None
  */
void UART_Reset(UART_TypeDef* UARTx)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
        
    if (UARTx == UART0)
    {
        RCC_APB0PeriphReset(RCC_APB0_UART0);
    }
    else if (UARTx == UART1)
    {
        RCC_APB0PeriphReset(RCC_APB0_UART1);
    }
    else
    {
        if (UARTx == UART2)
        {
            RCC_APB0PeriphReset(RCC_APB0_UART2);
        }
    }
}

/**
  * @brief  Fills each UART_InitStruct member with its default value.
  * @param  UART_InitStruct : pointer to a @ref UART_InitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void UART_StructInit(UART_InitTypeDef* UART_InitStruct)
{
    UART_InitStruct->UART_Mode = UART_Mode_1;
    UART_InitStruct->UART_BaudRate = 9600;
    UART_InitStruct->UART_DataLength = UART_DataLength_8Bit;
    UART_InitStruct->UART_StopBits = UART_StopBits_1;
    UART_InitStruct->UART_Parity = UART_Parity_None;
    UART_InitStruct->UART_TX_FIFO_Func = UART_TX_Standard;
    UART_InitStruct->UART_RX_FIFO_Func = UART_RX_Standard;
    UART_InitStruct->UART_Enable = UART_Enable_Rx | UART_Enable_Tx;
}

/**
  * @brief  Initialize the UART peripheral according to the specified parameters
  *         in the UART_InitStruct.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_InitStruct: pointer to a @ref UART_InitTypeDef structure that contains
  *         the configuration information for the specified UART peripheral.
  * @retval None
  */
void UART_Init(UART_TypeDef* UARTx,UART_InitTypeDef* UART_InitStruct)
{
    uint32_t temp = 0x00;
    uint32_t baudRate = 0;
    RCC_Clocks_TypeDef RCC_ClocksStatus;

    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_MODE(UART_InitStruct->UART_Mode));

    /* Set UART mode */
    UARTx->CR.V32 &= ~UART_CR_SM_Msk;
    UARTx->CR.V32 |= UART_InitStruct->UART_Mode;

    /* UART baud rate configuration */
    if ((UART_InitStruct->UART_Mode == UART_Mode_1) \
        || (UART_InitStruct->UART_Mode == UART_Mode_3))
    {
        RCC_GetClocksFreq(&RCC_ClocksStatus);
        baudRate = RCC_ClocksStatus.pclkFreq / 16 / UART_InitStruct->UART_BaudRate - 1;
        temp = RCC_ClocksStatus.pclkFreq / UART_InitStruct->UART_BaudRate \
                 - 16 * (baudRate + 1);
        baudRate += temp << 16;
    }

    if (UART_InitStruct->UART_Mode == UART_Mode_0)
    {
        assert_param(IS_UART_MODE_0_BAUDRATE(UART_InitStruct->UART_BaudRate));
        assert_param(IS_UART_MODE_01_DATALENGTH(UART_InitStruct->UART_DataLength));

        /* Set the UART baud rate */
        UARTx->CR.V32 &= ~(UART_CR_SMOD0_Msk);
        UARTx->CR.V32 |= ((UART_InitStruct->UART_BaudRate >> 23) & UART_CR_SMOD0_Msk); 

        UARTx->CR.V32 &= ~UART_CR_STOP_Msk;
			
        /* Set the Parity bit */
        UARTx->CR.V32 &= (~(UART_CR_MULTIE_Msk | UART_CR_PS_Msk));
        UARTx->CR.V32 |= UART_InitStruct->UART_Parity;
    }
    else if (UART_InitStruct->UART_Mode == UART_Mode_1)
    {
        assert_param(IS_UART_MODE_13_BAUDRATE(UART_InitStruct->UART_BaudRate));
        assert_param(IS_UART_MODE_123_STOP_BITS(UART_InitStruct->UART_StopBits));
        assert_param(IS_UART_MODE_01_DATALENGTH(UART_InitStruct->UART_DataLength));

        UARTx->BRT.V32 = baudRate;
        UARTx->CR.V32 |= UART_CR_SBRTEN_Msk;

        UARTx->CR.V32 &= ~UART_CR_STOP_Msk;
        UARTx->CR.V32 |= UART_InitStruct->UART_StopBits - 1;
        
        /* Set the Parity bit */
        UARTx->CR.V32 &= (~(UART_CR_MULTIE_Msk | UART_CR_PS_Msk));
        UARTx->CR.V32 |= UART_InitStruct->UART_Parity;        
    }
    else if (UART_InitStruct->UART_Mode == UART_Mode_2)
    {
        assert_param(IS_UART_MODE_2_BAUDRATE(UART_InitStruct->UART_BaudRate));
        assert_param(IS_UART_MODE_123_STOP_BITS(UART_InitStruct->UART_StopBits));
        assert_param(IS_UART_MODE_23_DATALENGTH(UART_InitStruct->UART_DataLength));
        assert_param(IS_UART_MODE_23_PARITY(UART_InitStruct->UART_Parity));

        UARTx->CR.V32 &= ~UART_CR_SMOD2_Msk;
        UARTx->CR.V32 |= UART_InitStruct->UART_BaudRate;
        UARTx->CR.V32 |= ((UART_InitStruct->UART_BaudRate>>27) & UART_CR_SMOD2_Msk);
        
        UARTx->CR.V32 &= ~UART_CR_STOP_Msk;
        UARTx->CR.V32 |= UART_InitStruct->UART_StopBits - 1;

        /* Set the Parity bit */
        UARTx->CR.V32 &= (~(UART_CR_MULTIE_Msk | UART_CR_PS_Msk));
        UARTx->CR.V32 |= UART_InitStruct->UART_Parity;         
    }
    else
    {
        assert_param(IS_UART_MODE_13_BAUDRATE(UART_InitStruct->UART_BaudRate));
        assert_param(IS_UART_MODE_123_STOP_BITS(UART_InitStruct->UART_StopBits));
        assert_param(IS_UART_MODE_23_DATALENGTH(UART_InitStruct->UART_DataLength));
        assert_param(IS_UART_MODE_23_PARITY(UART_InitStruct->UART_Parity));

        UARTx->BRT.V32 = baudRate;

        UARTx->CR.V32 |= UART_CR_SBRTEN_Msk;
        
        UARTx->CR.V32 &= ~UART_CR_STOP_Msk;
        UARTx->CR.V32 |= UART_InitStruct->UART_StopBits - 1;

        /* Set the Parity bit */
        UARTx->CR.V32 &= (~(UART_CR_MULTIE_Msk | UART_CR_PS_Msk));
        UARTx->CR.V32 |= UART_InitStruct->UART_Parity;         
    }
    //select standard UART or UART with FIFO    
    UARTx->CR.V32 &= ~(UART_CR_UATXFIFO_Msk | UART_CR_UARXFIFO_Msk);
    UARTx->CR.V32 |= (UART_InitStruct->UART_TX_FIFO_Func | UART_InitStruct->UART_RX_FIFO_Func);

    UARTx->CR.V32 &= ~(UART_CR_TEN_Msk | UART_CR_REN_Msk);
    /* Enable UART transmit or receive */
    UARTx->CR.V32 |= UART_InitStruct->UART_Enable; 
}

/**
  * @}
  */
  
/** @defgroup UART_Group2 General Functions
 *  @brief    General Functions
 *
@verbatim    
 ===============================================================================
                      General Functions
 ===============================================================================  
   
@endverbatim
  * @{
  */

/**
  * @brief  Set the Polarity of RX or TX pins.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_TX_Polarity: specifie the transmit polarity to be selected.
  *          This parameter can be one of the following values:
  *            @arg UART_TX_Polarity_DISABLE
  *            @arg UART_TX_Polarity_ENABLE
  * @param  UART_RX_Polarity: specifie the receive polarity to be selected.
  *          This parameter can be one of the following values:
  *            @arg UART_RX_Polarity_DISABLE
  *            @arg UART_RX_Polarity_ENABLE  
  * @retval None
  */
void UART_PolarityConfig(UART_TypeDef* UARTx, uint8_t UART_TX_Polarity, uint8_t UART_RX_Polarity)
{
    uint32_t tempData = 0;
    
    assert_param(IS_UART_TX_POLARITY(UART_TX_Polarity));
    assert_param(IS_UART_RX_POLARITY(UART_RX_Polarity));

    tempData = UARTx->CR.V32;
    tempData &= ~(UART_CR_RNEG_Msk | UART_CR_TNEG_Msk);

    tempData |= (uint32_t)((UART_TX_Polarity<<22) | (UART_RX_Polarity<<21));
    UARTx->CR.V32 = tempData;
}


/**
  * @brief  Check whether the specified UART flag is set or not.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_Flag: specifie the flag to check.
  *          This parameter can be one of the following values:
  *            @arg UART_FLAG_RI
  *            @arg UART_FLAG_TI
  *            @arg UART_FLAG_TC
  *            @arg UART_FLAG_TXCOL
  *            @arg UART_FLAG_RXOV
  *            @arg UART_FLAG_FE
  *            @arg UART_FLAG_PE
  *            @arg UART_FLAG_LBD  
  * @retval The new state of UART_Flag(SH_SET or SH_RESET).
  */
SH_FlagStatus UART_GetFlagStatus(UART_TypeDef* UARTx, uint8_t UART_Flag)
{
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));    
    assert_param(IS_UART_GET_ONE_FLAG(UART_Flag));
    
    if ((UARTx->FR.V32 & UART_Flag) != SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    return bitStatus;
}


/**
  * @brief  Clear the UART's pending flags.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_Flag: Specifie the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg UART_FLAG_RI
  *            @arg UART_FLAG_TI
  *            @arg UART_FLAG_TC
  *            @arg UART_FLAG_TXCOL
  *            @arg UART_FLAG_RXOV
  *            @arg UART_FLAG_FE
  *            @arg UART_FLAG_PE
  *            @arg UART_FLAG_LBD  
  * @retval None
  */
void UART_ClearFlag(UART_TypeDef* UARTx, uint8_t UART_Flag)
{    
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_FLAG(UART_Flag));
    
    UARTx->FR.V32 = ((uint32_t)UART_Flag << 16);
}

/**
  * @brief  Enable or disable the specified UART's interrupts.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_INT: specifie the UART interrupt sources to be enabled or disabled.
  *          This parameter can be any combination of the following values:
  *            @arg UART_INT_RX
  *            @arg UART_INT_TX
  *            @arg UART_INT_TC
  *            @arg UART_INT_LBD  
  * @param  NewState: New state of UART interrupt.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void UART_INTConfig(UART_TypeDef* UARTx, uint8_t UART_INT, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_INT(UART_INT));
    assert_param(IS_FUNCTION_STATE(NewState));
    
    if (NewState != SH_DISABLE)
    {
        UARTx->CR.V32 |= UART_INT;
    }
    else
    {
        UARTx->CR.V32 &= ~UART_INT;
    }
}

/**
  * @brief  Transmit single data through the UARTx peripheral.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  SendData: the data to transmit.
  * @retval None
  */
void UART_SendData(UART_TypeDef* UARTx, uint16_t SendData)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    /* Transmit data */
    UARTx->TDR.V32 = (SendData & (uint16_t)0x01FF);   
}

/**
  * @brief  Set the function of Bit9.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  Bit9Mode: set the function of Bit9.
  *            @arg UART_BIT9_Software
  *            @arg UART_BIT9_MultComm
  *            @arg UART_BIT9_Parity
  * @retval None
  */
void UART_Bit9Config(UART_TypeDef* UARTx, uint16_t Bit9Mode)
{
    uint32_t tempData = 0;
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_BIT9CONFIG(Bit9Mode));

    /* Transmit data */

    tempData = UARTx->CR.V32;
    tempData &= ~(UART_CR_MULTIE_Msk);

    tempData |= (uint32_t)(Bit9Mode);
    UARTx->CR.V32 = tempData;
}

/**
  * @brief  The parity selection.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  Parity: specifie the transmit polarity to be selected.
  *          This parameter can be one of the following values:
  *            @arg UART_Parity_Odd
  *            @arg UART_Parity_Even
  * @retval None
  */
void UART_SetParity(UART_TypeDef* UARTx, uint16_t Parity)
{
    uint32_t tempData = 0;
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_MODE_23_PARITY(Parity));

    /* Set Parity*/
    tempData = UARTx->CR.V32;
    tempData &= ~(UART_CR_PS_Msk);

    tempData |= (uint32_t)(Parity);
    UARTx->CR.V32 = tempData;
}


/**
  * @brief  Return the most recent received data by the UARTx peripheral.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @retval The received data.
  */
uint16_t UART_ReceiveData(UART_TypeDef* UARTx)
{
    uint16_t tempData = 0;

    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    tempData = (uint16_t)(UARTx->RDR.V32 & (uint16_t)0x01FF);
	
    return tempData;
}
 

/**
  * @brief  Return the most recent received 8bits data by the UATTx peripheral.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @retval The received 8bits data.
  */
uint8_t UART_ReceiveData8(UART_TypeDef* UARTx)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    return (uint8_t)UARTx->RDR.BIT.RDR;
}

/**
  * @brief  Return the most recent received 9bits data by the UATTx peripheral.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @retval The received 9bits data.
  */
uint16_t UART_GetBit9(UART_TypeDef* UARTx)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    
    return (uint16_t)UARTx->RDR.BIT.RB8;
}

/**
  * @brief  Set the UARTx address for master-slave communication.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_Address: specifie the address to set.
  *          This parameter can be 0 to 0xFF.
  * @retval None
  */
void UART_SetAddress(UART_TypeDef* UARTx, uint8_t UART_Address)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    UARTx->ADDR.V32 &= ~UART_ADDR_SADDR_Msk;
    UARTx->ADDR.V32 |= UART_Address; 
}

/**
  * @brief  Configure the UARTx address shield bits for master-slave communication.
  * @param  UARTx: where x can be 0,1 or 2 to select the UART peripheral.
  * @param  UART_AddressShield: specifie the address shield bits.
  *          This parameter can be 0 to 0xFF.
  * @retval None
  */
void UART_AddressShieldConfig(UART_TypeDef* UARTx, uint8_t UART_AddressShield)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    UARTx->ADDR.V32 &= ~UART_ADDR_SMAR_Msk;
    UARTx->ADDR.V32 |= (uint32_t)UART_AddressShield << 8; 
}



/**
  * @brief  Open or close the UART's send module.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  OnOffState: state of the UART send module.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void UART_SendOnOff(UART_TypeDef* UARTx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        UARTx->CR.V32 |= UART_CR_TEN_Msk; 
    }
    else
    {
        UARTx->CR.V32 &= (~UART_CR_TEN_Msk);
    }
}

/**
  * @brief  Open or close the UART's receive module.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  OnOffState: state of the UART receive module.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void UART_ReceiveOnOff(UART_TypeDef* UARTx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        UARTx->CR.V32 |= UART_CR_REN_Msk; 
    }
    else
    {
        UARTx->CR.V32 &= (~UART_CR_REN_Msk);
    }
}

/**
  * @}
  */
  
/** @defgroup UART_Group3 FIFO Functions
 *  @brief    FIFO Functions
 *
@verbatim    
 ===============================================================================
                      FIFO Functions
 ===============================================================================  
   
@endverbatim
  * @{
  */
  
/**
  * @brief  Open or close the UART's send module.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  OnOffState: state of the UART TX FIFO.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void UART_TxFIFOOnOff(UART_TypeDef* UARTx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        UARTx->CR.V32 |= UART_CR_UATXFIFO_Msk; 
    }
    else
    {
        UARTx->CR.V32 &= (~UART_CR_UATXFIFO_Msk);
    }
}

/**
  * @brief  Open or close the UART's receive module.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  OnOffState: state of the UART RX FIFO.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void UART_RxFIFOOnOff(UART_TypeDef* UARTx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        UARTx->CR.V32 |= UART_CR_UARXFIFO_Msk; 
    }
    else
    {
        UARTx->CR.V32 &= (~UART_CR_UARXFIFO_Msk);
    }
}

/**
  * @brief  send data to FIFO.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  FIFO_Data: the data to FIFO transmit.
  * @retval None
  */
void UART_SendToFIFO(UART_TypeDef* UARTx, uint32_t FIFO_Data)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    /* Send break characters */
    UARTx->TDR.V32 = FIFO_Data;
}

/**
  * @brief  receive data from FIFO.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @retval None
  */
uint32_t UART_ReceiveFromFIFO(UART_TypeDef* UARTx)
{
    uint32_t tempData = 0;
    
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    /* Receive a character */
    tempData = UARTx->RDR.V32;
    return tempData;
}

/**
  * @}
  */
  
/** @defgroup UART_Group4 LIN Functions
 *  @brief    LIN Functions
 *
@verbatim    
 ===============================================================================
                      LIN Functions
 ===============================================================================  
   
@endverbatim
  * @{
  */
/**
  * @brief  Open or close the UART's LIN mode.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  OnOffState: state of the UART LIN mode.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void UART_LINOnOff(UART_TypeDef* UARTx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        UARTx->CR.V32 |= UART_CR_LINEN_Msk; 
    }
    else
    {
        UARTx->CR.V32 &= (~UART_CR_LINEN_Msk);
    }
}

/**
  * @brief  Enable or Disable the UART's LIN mode.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @param  UART_LINDetectLength: specifie the LIN break detect length.
  *          This parameter can be one of the following values:
  *            @arg UART_LINDetectLength_10b
  *            @arg UART_LINDetectLength_11b
  * @retval None
  */
void UART_LINDetectLengthConfig(UART_TypeDef* UARTx, uint16_t UART_LINDetectLength)
{
    uint32_t temp = 0;
    
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));
    assert_param(IS_UART_LIN_DETECT_LENGTH(UART_LINDetectLength));

    temp = UARTx->CR.V32;
    temp &= ~UART_CR_LBDL_Msk;
    temp |= UART_LINDetectLength;
    UARTx->CR.V32 = temp;
}


/**
  * @brief  Transmits break characters.
  * @param  UARTx: where x can be 0, 1, 2 to select the UART peripheral.
  * @retval None
  */
void UART_SendBreak(UART_TypeDef* UARTx)
{
    /* Check the parameters */
    assert_param(IS_UART_ALL_PERIPH(UARTx));

    /* Send break characters */
    UARTx->CR.V32 |= UART_CR_SBK_Msk;
}


/**
  * @}
  */
  

/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


